@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Function_Type.cxx,v 1.15.2.15 2000/06/05 21:20:36 mike Exp $"
+// "$Id: Fl_Function_Type.cxx,v 1.15.2.16.2.12 2003/01/28 20:51:09 easysw Exp $"
 //
 // C function type code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2002 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -26,9 +26,9 @@
 #include <FL/Fl.H>
 #include "Fl_Type.h"
 #include <FL/fl_show_input.H>
-#include <string.h>
+#include "../src/flstring.h"
 #include <stdio.h>
-#include <ctype.h>
+#include <stdlib.h>
 
 extern int i18n_type;
 extern const char* i18n_include;
@@ -113,6 +113,8 @@ const char *c_check(const char *c, int type) {
 
 ////////////////////////////////////////////////////////////////
 
+int Fl_Function_Type::is_public() const {return public_;}
+
 Fl_Type *Fl_Function_Type::make() {
   Fl_Type *p = Fl_Type::current;
   while (p && !p->is_decl_block()) p = p->parent;
@@ -255,7 +257,32 @@ void Fl_Function_Type::write_code1() {
       *sptr = '\0';
 
       write_h("%s;\n", s);
-      write_c("%s::%s {\n", k, name());
+      // skip all function default param. init in body:
+      int skips=0,skipc=0;
+      int nc=0,plevel=0;
+      for (sptr=s,nptr=(char*)name(); *nptr; nc++,nptr++) {
+	if (!skips && *nptr=='(') plevel++;
+	else if (!skips && *nptr==')') plevel--;
+	if ( *nptr=='"' &&  !(nc &&  *(nptr-1)=='\\') ) 
+	  skips = skips ? 0 : 1;
+	else if(!skips && *nptr=='\'' &&  !(nc &&  *(nptr-1)=='\\'))
+	  skipc = skipc ? 0 : 1;
+	if(!skips && !skipc && plevel==1 && *nptr =='=' && 
+	   !(nc && *(nptr-1)=='\'') ) // ignore '=' case 
+	  while(*++nptr  && (skips || skipc || (*nptr!=',' && *nptr!=')' || plevel!=1) )) {
+	    if ( *nptr=='"' &&  *(nptr-1)!='\\' ) 
+	      skips = skips ? 0 : 1;
+	    else if(!skips && *nptr=='\'' &&  *(nptr-1)!='\\')
+	      skipc = skipc ? 0 : 1;
+	    if (!skips && !skipc && *nptr=='(') plevel++;
+	    else if (!skips && *nptr==')') plevel--;
+	  }
+
+	if (sptr < (s + sizeof(s) - 1))	*sptr++ = *nptr;
+      }
+      *sptr = '\0';
+ 
+      write_c("%s::%s {\n", k, s);
     } else {
       if (public_) {
 	if (cdecl_)
@@ -299,7 +326,8 @@ Fl_Type *Fl_Code_Type::make() {
 
 void Fl_Code_Type::open() {
   if (!code_panel) make_code_panel();
-  code_input->static_value(name());
+  const char *text = name();
+  code_input->buffer()->text( text ? text : "" );
   code_panel->show();
   const char* message = 0;
   for (;;) { // repeat as long as there are errors
@@ -310,9 +338,10 @@ void Fl_Code_Type::open() {
       else if (w == code_panel_ok) break;
       else if (!w) Fl::wait();
     }
-    const char*c = code_input->value();
+    char*c = code_input->buffer()->text();
     message = c_check(c); if (message) continue;
     name(c);
+    free(c);
     break;
   }
  BREAK2:
@@ -404,6 +433,8 @@ void Fl_CodeBlock_Type::write_code2() {
 
 ////////////////////////////////////////////////////////////////
 
+int Fl_Decl_Type::is_public() const {return public_;}
+
 Fl_Type *Fl_Decl_Type::make() {
   Fl_Type *p = Fl_Type::current;
   while (p && !p->is_decl_block()) p = p->parent;
@@ -459,11 +490,13 @@ Fl_Decl_Type Fl_Decl_type;
 void Fl_Decl_Type::write_code1() {
   const char* c = name();
   if (!c) return;
-  // handle putting #include or extern or typedef into decl:
+  // handle putting #include, extern, using or typedef into decl:
   if (!isalpha(*c) && *c != '~'
       || !strncmp(c,"extern",6) && isspace(c[6])
       || !strncmp(c,"class",5) && isspace(c[5])
       || !strncmp(c,"typedef",7) && isspace(c[7])
+      || !strncmp(c,"using",5) && isspace(c[5])
+      || !strncmp(c,"FL_EXPORT",9) && isspace(c[9])
 //    || !strncmp(c,"struct",6) && isspace(c[6])
       ) {
     if (public_)
@@ -570,11 +603,11 @@ const char* Fl_Type::class_name(const int need_nest) const {
       const char* q = 0;
       if(need_nest) q=p->class_name(need_nest);
       if (q) {
-	static char buffer[256];
-	if (q != buffer) strcpy(buffer, q);
-	strcat(buffer, "::");
-	strcat(buffer, p->name());
-	return buffer;
+	static char s[256];
+	if (q != s) strlcpy(s, q, sizeof(s));
+	strlcat(s, "::", sizeof(s));
+	strlcat(s, p->name(), sizeof(s));
+	return s;
       }
       return p->name();
     }
@@ -583,11 +616,19 @@ const char* Fl_Type::class_name(const int need_nest) const {
   return 0;
 }
 
+int Fl_Class_Type::is_public() const {return public_;}
+
+void Fl_Class_Type::prefix(const char*p) {
+  free((void*) class_prefix);
+  class_prefix=strdup(p ? p : "" );
+}
+
 Fl_Type *Fl_Class_Type::make() {
   Fl_Type *p = Fl_Type::current;
   while (p && !p->is_decl_block()) p = p->parent;
   Fl_Class_Type *o = new Fl_Class_Type();
   o->name("UserInterface");
+  o->class_prefix=0;
   o->subclass_of = 0;
   o->public_ = 1;
   o->add(p);
@@ -616,11 +657,19 @@ void Fl_Class_Type::read_property(const char *c) {
 
 void Fl_Class_Type::open() {
   if (!class_panel) make_class_panel();
-  c_name_input->static_value(name());
+  char fullname[1024]="";
+  if (prefix() && strlen(prefix())) 
+    sprintf(fullname,"%s %s",prefix(),name());
+  else 
+    strcpy(fullname, name());
+  c_name_input->static_value(fullname);
   c_subclass_input->static_value(subclass_of);
   c_public_button->value(public_);
   class_panel->show();
   const char* message = 0;
+
+  char *na=0,*pr=0,*p=0; // name and prefix substrings
+
   for (;;) { // repeat as long as there are errors
     if (message) fl_alert(message);
     for (;;) {
@@ -630,14 +679,30 @@ void Fl_Class_Type::open() {
       else if (!w) Fl::wait();
     }
     const char*c = c_name_input->value();
-    while (isspace(*c)) c++;
-    if (!*c) goto OOPS;
-    while (is_id(*c)) c++;
-    while (isspace(*c)) c++;
-    if (*c) {OOPS: message = "class name must be C++ identifier"; continue;}
+    char *s = strdup(c);
+    size_t len = strlen(s);
+    if (!*s) goto OOPS;
+    p = (char*) (s+len-1);
+    while (p>=s && isspace(*p)) *(p--)='\0';
+    if (p<s) goto OOPS;
+    while (p>=s && is_id(*p)) p--;
+    if ( (p<s && !is_id(*(p+1))) || !*(p+1) ) {
+      OOPS: message = "class name must be C++ identifier";
+      free((void*)s);
+      continue;
+    }
+    na=p+1; // now we have the name
+    if(p>s) *p--='\0';
+    while (p>=s && isspace(*p)) *(p--)='\0';
+    while (p>=s && is_id(*p))   p--;
+    if (p<s)                    p++;
+    if (is_id(*p) && p<na)      pr=p; // prefix detected
     c = c_subclass_input->value();
-    message = c_check(c); if (message) continue;
-    name(c_name_input->value());
+    message = c_check(c); 
+    if (message) { free((void*)s);continue;}
+    name(na);
+    prefix(pr);
+    free((void*)s);
     storestring(c, subclass_of);
     public_ = c_public_button->value();
     break;
@@ -661,7 +726,10 @@ void Fl_Class_Type::write_code1() {
   parent_class = current_class;
   current_class = this;
   write_public_state = 0;
-  write_h("\nclass %s ", name());
+  if (prefix() && strlen(prefix()))
+      write_h("\nclass %s %s ", prefix(), name());
+  else
+      write_h("\nclass %s ", name());
   if (subclass_of) write_h(": %s ", subclass_of);
   write_h("{\n");
 }
@@ -672,5 +740,5 @@ void Fl_Class_Type::write_code2() {
 }
 
 //
-// End of "$Id: Fl_Function_Type.cxx,v 1.15.2.15 2000/06/05 21:20:36 mike Exp $".
+// End of "$Id: Fl_Function_Type.cxx,v 1.15.2.16.2.12 2003/01/28 20:51:09 easysw Exp $".
 //
