@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_Window_Type.cxx,v 1.13.2.8 2000/06/05 21:20:40 mike Exp $"
+// "$Id: Fl_Window_Type.cxx,v 1.13.2.10.2.6 2002/08/09 22:57:00 easysw Exp $"
 //
 // Window type code for the Fast Light Tool Kit (FLTK).
 //
@@ -7,7 +7,7 @@
 // for interacting with the overlay, which allows the user to
 // select, move, and resize the children widgets.
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2002 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -38,9 +38,9 @@
 #include "alignment_panel.h"
 #include <stdio.h>
 
-int gridx = 5;
-int gridy = 5;
-int snap = 3;
+extern int gridx;
+extern int gridy;
+extern int snap;
 
 int include_H_from_C = 1;
 extern int i18n_type;
@@ -50,15 +50,39 @@ extern const char* i18n_file;
 extern const char* i18n_set;
 extern int modflag;
 
-void alignment_cb(Fl_Input *i, long v) {
+extern Fl_Preferences	fluid_prefs;
+
+void grid_cb(Fl_Input *i, long v) {
   int n = atoi(i->value());
   if (n < 0) n = 0;
   switch (v) {
-  case 1: gridx = n; break;
-  case 2: gridy = n; break;
-  case 3: snap  = n; break;
+    case 1:
+      gridx = n;
+      fluid_prefs.set("gridx", n);
+      break;
+    case 2:
+      gridy = n;
+      fluid_prefs.set("gridy", n);
+      break;
+    case 3:
+      snap = n;
+      fluid_prefs.set("snap", n);
+      break;
+  }
+
+  // Next go through all of the windows in the project and set the
+  // stepping for resizes...
+  Fl_Type *p;
+  Fl_Window_Type *w;
+
+  for (p = Fl_Type::first; p; p = p->next) {
+    if (p->is_window()) {
+      w = (Fl_Window_Type *)p;
+      ((Fl_Window *)(w->o))->size_range(gridx * snap, gridy * snap,
+                                        Fl::w(), Fl::h(),
+                                        gridx * snap, gridy * snap, 0);
+    }
   }
-  modflag = 1;
 }
 
 void i18n_type_cb(Fl_Choice *c, void *) {
@@ -112,15 +136,11 @@ void i18n_text_cb(Fl_Input *i, void *) {
 extern const char* header_file_name;
 extern const char* code_file_name;
 
-void show_alignment_cb(Fl_Widget *, void *) {
-  if(alignment_window==0) make_alignment_window();
+void show_project_cb(Fl_Widget *, void *) {
+  if(project_window==0) make_project_window();
   include_H_from_C_button->value(include_H_from_C);
   header_file_input->value(header_file_name);
   code_file_input->value(code_file_name);
-  char buf[128];
-  sprintf(buf,"%d",gridx); horizontal_input->value(buf);
-  sprintf(buf,"%d",gridy); vertical_input->value(buf);
-  sprintf(buf,"%d",snap); snap_input->value(buf);
   i18n_type_chooser->value(i18n_type);
   i18n_function_input->value(i18n_function);
   i18n_file_input->value(i18n_file);
@@ -146,7 +166,22 @@ void show_alignment_cb(Fl_Widget *, void *) {
       i18n_function_input->hide();
       break;
   }
-  alignment_window->show();
+  project_window->hotspot(project_window);
+  project_window->show();
+}
+
+void show_grid_cb(Fl_Widget *, void *) {
+  char buf[128];
+  sprintf(buf,"%d",gridx); horizontal_input->value(buf);
+  sprintf(buf,"%d",gridy); vertical_input->value(buf);
+  sprintf(buf,"%d",snap); snap_input->value(buf);
+  grid_window->hotspot(grid_window);
+  grid_window->show();
+}
+
+void show_settings_cb(Fl_Widget *, void *) {
+  settings_window->hotspot(settings_window);
+  settings_window->show();
 }
 
 void header_input_cb(Fl_Input* i, void*) {
@@ -178,7 +213,7 @@ class Overlay_Window : public Fl_Overlay_Window {
 public:
   Fl_Window_Type *window;
   int handle(int);
-  Overlay_Window(int w,int h) : Fl_Overlay_Window(w,h) {Fl_Group::current(0);}
+  Overlay_Window(int W,int H) : Fl_Overlay_Window(W,H) {Fl_Group::current(0);}
   void resize(int,int,int,int);
 };
 void Overlay_Window::draw() {
@@ -187,11 +222,11 @@ void Overlay_Window::draw() {
   if ((damage()&FL_DAMAGE_ALL) &&
       (!box() || (box()>=4&&!(box()&2)) || box()>=_FL_ROUNDED_BOX)) {
     // if so, draw checkerboard so user can see what areas are clear:
-    for (int y = 0; y < h(); y += CHECKSIZE) 
-      for (int x = 0; x < w(); x += CHECKSIZE) {
-	fl_color(((y/(2*CHECKSIZE))&1) != ((x/(2*CHECKSIZE))&1) ?
+    for (int Y = 0; Y < h(); Y += CHECKSIZE) 
+      for (int X = 0; X < w(); X += CHECKSIZE) {
+	fl_color(((Y/(2*CHECKSIZE))&1) != ((X/(2*CHECKSIZE))&1) ?
 		 FL_WHITE : FL_BLACK);
-	fl_rectf(x,y,CHECKSIZE,CHECKSIZE);
+	fl_rectf(X,Y,CHECKSIZE,CHECKSIZE);
       }
   }
   Fl_Overlay_Window::draw();
@@ -211,21 +246,26 @@ Fl_Type *Fl_Window_Type::make() {
     fl_message("Please select a function");
     return 0;
   }
-  Fl_Window_Type *o = new Fl_Window_Type();
+  Fl_Window_Type *myo = new Fl_Window_Type();
   if (!this->o) {// template widget
     this->o = new Fl_Window(100,100);
     Fl_Group::current(0);
   }
-  o->factory = this;
-  o->drag = 0;
-  o->numselected = 0;
+  // Set the size ranges for this window; in order to avoid opening the
+  // X display we use an arbitrary maximum size...
+  ((Fl_Window *)(this->o))->size_range(gridx * snap, gridy * snap,
+                                       3072, 2048,
+                                       gridx * snap, gridy * snap, 0);
+  myo->factory = this;
+  myo->drag = 0;
+  myo->numselected = 0;
   Overlay_Window *w = new Overlay_Window(100,100);
-  w->window = o;
-  o->o = w;
-  o->add(p);
-  o->modal = 0;
-  o->non_modal = 0;
-  return o;
+  w->window = myo;
+  myo->o = w;
+  myo->add(p);
+  myo->modal = 0;
+  myo->non_modal = 0;
+  return myo;
 }
 
 void Fl_Window_Type::add_child(Fl_Type* cc, Fl_Type* before) {
@@ -356,42 +396,42 @@ void Overlay_Window::resize(int X,int Y,int W,int H) {
 // calculate actual move by moving mouse position (mx,my) to
 // nearest multiple of gridsize, and snap to original position
 void Fl_Window_Type::newdx() {
-  int dx, dy;
+  int mydx, mydy;
   if (Fl::event_state(FL_ALT)) {
-    dx = mx-x1;
-    dy = my-y1;
+    mydx = mx-x1;
+    mydy = my-y1;
   } else {
     int dx0 = mx-x1;
     int ix = (drag&RIGHT) ? br : bx;
-    dx = gridx ? ((ix+dx0+gridx/2)/gridx)*gridx - ix : dx0;
+    mydx = gridx ? ((ix+dx0+gridx/2)/gridx)*gridx - ix : dx0;
     if (dx0 > snap) {
-      if (dx < 0) dx = 0;
+      if (mydx < 0) mydx = 0;
     } else if (dx0 < -snap) {
-      if (dx > 0) dx = 0;
+      if (mydx > 0) mydx = 0;
     } else 
-      dx = 0;
+      mydx = 0;
     int dy0 = my-y1;
     int iy = (drag&BOTTOM) ? by : bt;
-    dy = gridy ? ((iy+dy0+gridy/2)/gridy)*gridy - iy : dy0;
+    mydy = gridy ? ((iy+dy0+gridy/2)/gridy)*gridy - iy : dy0;
     if (dy0 > snap) {
-      if (dy < 0) dy = 0;
+      if (mydy < 0) mydy = 0;
     } else if (dy0 < -snap) {
-      if (dy > 0) dy = 0;
+      if (mydy > 0) mydy = 0;
     } else 
-      dy = 0;
+      mydy = 0;
   }
-  if (this->dx != dx || this->dy != dy) {
-    this->dx = dx; this->dy = dy;
-    ((Overlay_Window *)(this->o))->redraw_overlay();
+  if (dx != mydx || dy != mydy) {
+    dx = mydx; dy = mydy;
+    ((Overlay_Window *)o)->redraw_overlay();
   }
 }
 
 // Move a widget according to dx and dy calculated above
-void Fl_Window_Type::newposition(Fl_Widget_Type *o,int &X,int &Y,int &R,int &T) {
-  X = o->o->x();
-  Y = o->o->y();
-  R = X+o->o->w();
-  T = Y+o->o->h();
+void Fl_Window_Type::newposition(Fl_Widget_Type *myo,int &X,int &Y,int &R,int &T) {
+  X = myo->o->x();
+  Y = myo->o->y();
+  R = X+myo->o->w();
+  T = Y+myo->o->h();
   if (!drag) return;
   if (drag&DRAG) {
     X += dx;
@@ -415,11 +455,11 @@ void Fl_Window_Type::draw_overlay() {
     for (Fl_Type *q=next; q && q->level>level; q=q->next)
       if (q->selected && q->is_widget() && !q->is_menu_item()) {
 	numselected++;
-	Fl_Widget_Type* o = (Fl_Widget_Type*)q;
-	if (o->o->x() < bx) bx = o->o->x();
-	if (o->o->y() < by) by = o->o->y();
-	if (o->o->x()+o->o->w() > br) br = o->o->x()+o->o->w();
-	if (o->o->y()+o->o->h() > bt) bt = o->o->y()+o->o->h();
+	Fl_Widget_Type* myo = (Fl_Widget_Type*)q;
+	if (myo->o->x() < bx) bx = myo->o->x();
+	if (myo->o->y() < by) by = myo->o->y();
+	if (myo->o->x()+myo->o->w() > br) br = myo->o->x()+myo->o->w();
+	if (myo->o->y()+myo->o->h() > bt) bt = myo->o->y()+myo->o->h();
       }
     recalc = 0;
   }
@@ -432,25 +472,25 @@ void Fl_Window_Type::draw_overlay() {
   if (overlays_invisible && !drag) return;
   if (selected) fl_rect(0,0,o->w(),o->h());
   if (!numselected) return;
-  int bx,by,br,bt;
-  bx = o->w(); by = o->h(); br = 0; bt = 0;
+  int mybx,myby,mybr,mybt;
+  mybx = o->w(); myby = o->h(); mybr = 0; mybt = 0;
   for (Fl_Type *q=next; q && q->level>level; q = q->next)
     if (q->selected && q->is_widget() && !q->is_menu_item()) {
-      Fl_Widget_Type* o = (Fl_Widget_Type*)q;
+      Fl_Widget_Type* myo = (Fl_Widget_Type*)q;
       int x,y,r,t;
-      newposition(o,x,y,r,t);
+      newposition(myo,x,y,r,t);
       fl_rect(x,y,r-x,t-y);
-      if (x < bx) bx = x;
-      if (y < by) by = y;
-      if (r > br) br = r;
-      if (t > bt) bt = t;
+      if (x < mybx) mybx = x;
+      if (y < myby) myby = y;
+      if (r > mybr) mybr = r;
+      if (t > mybt) mybt = t;
     }
   if (selected) return;
-  if (numselected>1) fl_rect(bx,by,br-bx,bt-by);
-  fl_rectf(bx,by,5,5);
-  fl_rectf(br-5,by,5,5);
-  fl_rectf(br-5,bt-5,5,5);
-  fl_rectf(bx,bt-5,5,5);
+  if (numselected>1) fl_rect(mybx,myby,mybr-mybx,mybt-myby);
+  fl_rectf(mybx,myby,5,5);
+  fl_rectf(mybr-5,myby,5,5);
+  fl_rectf(mybr-5,mybt-5,5,5);
+  fl_rectf(mybx,mybt-5,5,5);
 }
 
 // Calculate new bounding box of selected widgets:
@@ -490,18 +530,18 @@ void Fl_Window_Type::moveallchildren()
   Fl_Type *i;
   for (i=next; i && i->level>level;) {
     if (i->selected && i->is_widget() && !i->is_menu_item()) {
-      Fl_Widget_Type* o = (Fl_Widget_Type*)i;
+      Fl_Widget_Type* myo = (Fl_Widget_Type*)i;
       int x,y,r,t;
-      newposition(o,x,y,r,t);
-      o->o->resize(x,y,r-x,t-y);
+      newposition(myo,x,y,r,t);
+      myo->o->resize(x,y,r-x,t-y);
       // move all the children, whether selected or not:
       Fl_Type* p;
-      for (p = o->next; p && p->level>o->level; p = p->next)
+      for (p = myo->next; p && p->level>myo->level; p = p->next)
 	if (p->is_widget() && !p->is_menu_item()) {
-	  Fl_Widget_Type* o = (Fl_Widget_Type*)p;
-	  int x,y,r,t;
-	  newposition(o,x,y,r,t);
-	  o->o->resize(x,y,r-x,t-y);
+	  Fl_Widget_Type* myo2 = (Fl_Widget_Type*)p;
+	  int X,Y,R,T;
+	  newposition(myo2,X,Y,R,T);
+	  myo2->o->resize(X,Y,R-X,T-Y);
 	}
       i = p;
     } else {
@@ -527,9 +567,9 @@ int Fl_Window_Type::handle(int event) {
     // test for popup menu:
     if (Fl::event_button() >= 3) {
       in_this_only = this; // modifies how some menu items work.
-      static const Fl_Menu_Item* prev;
-      const Fl_Menu_Item* m = New_Menu->popup(mx,my,"New",prev);
-      if (m && m->callback()) {prev = m; m->do_callback(this->o);}
+      static const Fl_Menu_Item* myprev;
+      const Fl_Menu_Item* m = New_Menu->popup(mx,my,"New",myprev);
+      if (m && m->callback()) {myprev = m; m->do_callback(this->o);}
       in_this_only = 0;
       return 1;
     }
@@ -537,10 +577,10 @@ int Fl_Window_Type::handle(int event) {
     selection = this;
     {for (Fl_Type* i=next; i && i->level>level; i=i->next)
       if (i->is_widget() && !i->is_menu_item()) {
-      Fl_Widget_Type* o = (Fl_Widget_Type*)i;
-      for (Fl_Widget *o1 = o->o; o1; o1 = o1->parent())
+      Fl_Widget_Type* myo = (Fl_Widget_Type*)i;
+      for (Fl_Widget *o1 = myo->o; o1; o1 = o1->parent())
 	if (!o1->visible()) goto CONTINUE2;
-      if (Fl::event_inside(o->o)) selection = o;
+      if (Fl::event_inside(myo->o)) selection = myo;
     CONTINUE2:;
     }}
     // see if user grabs edges of selected region:
@@ -600,14 +640,14 @@ int Fl_Window_Type::handle(int event) {
       // select everything in box:
       for (Fl_Type*i=next; i&&i->level>level; i=i->next)
 	if (i->is_widget() && !i->is_menu_item()) {
-	Fl_Widget_Type* o = (Fl_Widget_Type*)i;
-	for (Fl_Widget *o1 = o->o; o1; o1 = o1->parent())
+	Fl_Widget_Type* myo = (Fl_Widget_Type*)i;
+	for (Fl_Widget *o1 = myo->o; o1; o1 = o1->parent())
 	  if (!o1->visible()) goto CONTINUE;
-	if (Fl::event_inside(o->o)) selection = o;
-	if (o->o->x()>=x1 && o->o->y()>y1 &&
-	    o->o->x()+o->o->w()<mx && o->o->y()+o->o->h()<my) {
+	if (Fl::event_inside(myo->o)) selection = myo;
+	if (myo->o->x()>=x1 && myo->o->y()>y1 &&
+	    myo->o->x()+myo->o->w()<mx && myo->o->y()+myo->o->h()<my) {
 	  n++;
-	  select(o, toggle ? !o->selected : 1);
+	  select(myo, toggle ? !myo->selected : 1);
 	}
       CONTINUE:;
       }
@@ -685,7 +725,7 @@ int Fl_Window_Type::handle(int event) {
 ////////////////////////////////////////////////////////////////
 
 #include <stdio.h>
-#include <string.h>
+#include "../src/flstring.h"
 
 void Fl_Window_Type::write_code1() {
   Fl_Widget_Type::write_code1();
@@ -732,25 +772,25 @@ void Fl_Window_Type::read_property(const char *c) {
   }
 }
 
-int Fl_Window_Type::read_fdesign(const char* name, const char* value) {
+int Fl_Window_Type::read_fdesign(const char* propname, const char* value) {
   int x;
   o->box(FL_NO_BOX); // because fdesign always puts an Fl_Box next
-  if (!strcmp(name,"Width")) {
+  if (!strcmp(propname,"Width")) {
     if (sscanf(value,"%d",&x) == 1) o->size(x,o->h());
-  } else if (!strcmp(name,"Height")) {
+  } else if (!strcmp(propname,"Height")) {
     if (sscanf(value,"%d",&x) == 1) o->size(o->w(),x);
-  } else if (!strcmp(name,"NumberofWidgets")) {
+  } else if (!strcmp(propname,"NumberofWidgets")) {
     return 1; // we can figure out count from file
-  } else if (!strcmp(name,"border")) {
+  } else if (!strcmp(propname,"border")) {
     if (sscanf(value,"%d",&x) == 1) ((Fl_Window*)o)->border(x);
-  } else if (!strcmp(name,"title")) {
+  } else if (!strcmp(propname,"title")) {
     label(value);
   } else {
-    return Fl_Widget_Type::read_fdesign(name,value);
+    return Fl_Widget_Type::read_fdesign(propname,value);
   }
   return 1;
 }
 
 //
-// End of "$Id: Fl_Window_Type.cxx,v 1.13.2.8 2000/06/05 21:20:40 mike Exp $".
+// End of "$Id: Fl_Window_Type.cxx,v 1.13.2.10.2.6 2002/08/09 22:57:00 easysw Exp $".
 //
