@@ -1,9 +1,9 @@
 //
-// "$Id: Shortcut_Button.h,v 1.3.2.2 2000/06/05 21:20:41 mike Exp $"
+// "$Id: Shortcut_Button.h,v 1.3.2.3.2.2 2002/08/09 22:57:00 easysw Exp $"
 //
 // Shortcut header file for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2002 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -30,10 +30,10 @@ public:
   int svalue;
   int handle(int);
   void draw();
-  Shortcut_Button(int x, int y, int w, int h, const char* l = 0) :
-    Fl_Button(x,y,w,h,l) {svalue = 0;}
+  Shortcut_Button(int X,int Y,int W,int H, const char* l = 0) :
+    Fl_Button(X,Y,W,H,l) {svalue = 0;}
 };
 
 //
-// End of "$Id: Shortcut_Button.h,v 1.3.2.2 2000/06/05 21:20:41 mike Exp $".
+// End of "$Id: Shortcut_Button.h,v 1.3.2.3.2.2 2002/08/09 22:57:00 easysw Exp $".
 //
