@@ -1,5 +1,5 @@
 //
-// "$Id: factory.cxx,v 1.4.2.8 2000/11/21 21:37:08 easysw Exp $"
+// "$Id: factory.cxx,v 1.4.2.11.2.10 2002/10/30 22:30:30 matthiaswm Exp $"
 //
 // Widget factory code for the Fast Light Tool Kit (FLTK).
 //
@@ -11,7 +11,7 @@
 // to a factory instance for every class (both the ones defined
 // here and ones in other files)
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2002 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -34,19 +34,15 @@
 #include <FL/Fl.H>
 #include <FL/Fl_Group.H>
 #include <FL/Fl_Menu_Item.H>
-#include <string.h>
+#include <FL/Fl_Pixmap.H>
 #include <stdio.h>
-#include <config.h>
-
-#if defined(WIN32) || defined(__EMX__)
-#define strcasecmp stricmp
-#endif
+#include "../src/flstring.h"
 
 #include "Fl_Widget_Type.h"
 
-#if !HAVE_STRCASECMP
-#  include <ctype.h>
+extern Fl_Pixmap *pixmap[];
 
+#if !HAVE_STRCASECMP
 //
 // 'strcasecmp()' - Do a case-insensitive compare...
 //
@@ -81,6 +77,7 @@ public:
   Fl_Widget *widget(int x,int y,int w, int h) {
     return new Fl_Box(x,y,w,h,"label");}
   Fl_Widget_Type *_make() {return new Fl_Box_Type();}
+  int pixmapID() { return 5; }
 };
 static Fl_Box_Type Fl_Box_type;
 
@@ -100,6 +97,7 @@ public:
     return new Fl_Button(x,y,w,h,"button");}
   Fl_Widget_Type *_make() {return new Fl_Button_Type();}
   int is_button() const {return 1;}
+  int pixmapID() { return 2; }
 };
 static Fl_Button_Type Fl_Button_type;
 
@@ -112,6 +110,7 @@ public:
   Fl_Widget *widget(int x,int y,int w,int h) {
     return new Fl_Return_Button(x,y,w,h,0);}
   Fl_Widget_Type *_make() {return new Fl_Return_Button_Type();}
+  int pixmapID() { return 23; }
 };
 static Fl_Return_Button_Type Fl_Return_Button_type;
 
@@ -124,6 +123,7 @@ public:
   Fl_Widget *widget(int x,int y,int w,int h) {
     return new Fl_Repeat_Button(x,y,w,h,0);}
   Fl_Widget_Type *_make() {return new Fl_Repeat_Button_Type();}
+  int pixmapID() { return 25; }
 };
 static Fl_Repeat_Button_Type Fl_Repeat_Button_type;
 
@@ -136,6 +136,7 @@ public:
   Fl_Widget *widget(int x,int y,int w,int h) {
     return new Fl_Light_Button(x,y,w,h,"button");}
   Fl_Widget_Type *_make() {return new Fl_Light_Button_Type();}
+  int pixmapID() { return 24; }
 };
 static Fl_Light_Button_Type Fl_Light_Button_type;
 
@@ -148,6 +149,7 @@ public:
   Fl_Widget *widget(int x,int y,int w,int h) {
     return new Fl_Check_Button(x,y,w,h,"button");}
   Fl_Widget_Type *_make() {return new Fl_Check_Button_Type();}
+  int pixmapID() { return 3; }
 };
 static Fl_Check_Button_Type Fl_Check_Button_type;
 
@@ -160,6 +162,7 @@ public:
   Fl_Widget *widget(int x,int y,int w,int h) {
     return new Fl_Round_Button(x,y,w,h,"button");}
   Fl_Widget_Type *_make() {return new Fl_Round_Button_Type();}
+  int pixmapID() { return 4; }
 };
 static Fl_Round_Button_Type Fl_Round_Button_type;
 
@@ -168,6 +171,9 @@ static Fl_Round_Button_Type Fl_Round_Button_type;
 extern int compile_only;
 
 #include <FL/Fl_Browser.H>
+#include <FL/Fl_Check_Browser.H>
+#include <FL/Fl_File_Browser.H>
+
 static Fl_Menu_Item browser_type_menu[] = {
   {"No Select",0,0,(void*)FL_NORMAL_BROWSER},
   {"Select",0,0,(void*)FL_SELECT_BROWSER},
@@ -193,17 +199,84 @@ public:
     return b;
   }
   Fl_Widget_Type *_make() {return new Fl_Browser_Type();}
+  int pixmapID() { return 31; }
 };
 static Fl_Browser_Type Fl_Browser_type;
 
 int Fl_Browser_Type::textstuff(int w, Fl_Font& f, int& s, Fl_Color& c) {
-  Fl_Browser *o = (Fl_Browser*)(w==4 ? ((Fl_Widget_Type*)this->factory)->o : this->o);
+  Fl_Browser *myo = (Fl_Browser*)(w==4 ? ((Fl_Widget_Type*)factory)->o : o);
   switch (w) {
     case 4:
-    case 0: f = o->textfont(); s = o->textsize(); c = o->textcolor(); break;
-    case 1: o->textfont(f); break;
-    case 2: o->textsize(s); break;
-    case 3: o->textcolor(c); break;
+    case 0: f = myo->textfont(); s = myo->textsize(); c = myo->textcolor(); break;
+    case 1: myo->textfont(f); break;
+    case 2: myo->textsize(s); break;
+    case 3: myo->textcolor(c); break;
+  }
+  return 1;
+}
+
+class Fl_Check_Browser_Type : public Fl_Widget_Type {
+  Fl_Menu_Item *subtypes() {return browser_type_menu;}
+  int textstuff(int w, Fl_Font& f, int& s, Fl_Color& c);
+public:
+  virtual const char *type_name() {return "Fl_Check_Browser";}
+  Fl_Widget *widget(int x,int y,int w,int h) {
+    Fl_Check_Browser* b = new Fl_Check_Browser(x,y,w,h);
+    // Fl_Check_Browser::add calls fl_height(), which requires the X display open.
+    // Avoid this when compiling so it works w/o a display:
+    if (!compile_only) {
+      char buffer[20];
+      for (int i = 1; i <= 20; i++) {
+	sprintf(buffer,"Browser Line %d",i);
+	b->add(buffer);
+      }
+    }
+    return b;
+  }
+  Fl_Widget_Type *_make() {return new Fl_Check_Browser_Type();}
+  int pixmapID() { return 32; }
+};
+static Fl_Check_Browser_Type Fl_Check_Browser_type;
+
+int Fl_Check_Browser_Type::textstuff(int w, Fl_Font& f, int& s, Fl_Color& c) {
+  Fl_Check_Browser *myo = (Fl_Check_Browser*)(w==4 ? ((Fl_Widget_Type*)factory)->o : o);
+  switch (w) {
+    case 4:
+    case 0: f = myo->textfont(); s = myo->textsize(); c = myo->textcolor(); break;
+    case 1: myo->textfont(f); break;
+    case 2: myo->textsize(s); break;
+    case 3: myo->textcolor(c); break;
+  }
+  return 1;
+}
+
+class Fl_File_Browser_Type : public Fl_Widget_Type {
+  Fl_Menu_Item *subtypes() {return browser_type_menu;}
+  int textstuff(int w, Fl_Font& f, int& s, Fl_Color& c);
+public:
+  virtual const char *type_name() {return "Fl_File_Browser";}
+  Fl_Widget *widget(int x,int y,int w,int h) {
+    Fl_File_Browser* b = new Fl_File_Browser(x,y,w,h);
+    // Fl_File_Browser::add calls fl_height(), which requires the X display open.
+    // Avoid this when compiling so it works w/o a display:
+    if (!compile_only) {
+      b->load(".");
+    }
+    return b;
+  }
+  Fl_Widget_Type *_make() {return new Fl_File_Browser_Type();}
+  int pixmapID() { return 33; }
+};
+static Fl_File_Browser_Type Fl_File_Browser_type;
+
+int Fl_File_Browser_Type::textstuff(int w, Fl_Font& f, int& s, Fl_Color& c) {
+  Fl_File_Browser *myo = (Fl_File_Browser*)(w==4 ? ((Fl_Widget_Type*)factory)->o : o);
+  switch (w) {
+    case 4:
+    case 0: f = myo->textfont(); s = myo->textsize(); c = myo->textcolor(); break;
+    case 1: myo->textfont(f); break;
+    case 2: myo->textsize(s); break;
+    case 3: myo->textcolor(c); break;
   }
   return 1;
 }
@@ -219,6 +292,7 @@ class Fl_Counter_Type : public Fl_Widget_Type {
   Fl_Menu_Item *subtypes() {return counter_type_menu;}
   int textstuff(int w, Fl_Font& f, int& s, Fl_Color& c);
   int is_valuator() const {return 1;}
+  int pixmapID() { return 41; }
 public:
   virtual const char *type_name() {return "Fl_Counter";}
   Fl_Widget *widget(int x,int y,int w,int h) {
@@ -228,13 +302,13 @@ public:
 static Fl_Counter_Type Fl_Counter_type;
 
 int Fl_Counter_Type::textstuff(int w, Fl_Font& f, int& s, Fl_Color& c) {
-  Fl_Counter *o = (Fl_Counter*)(w==4 ? ((Fl_Widget_Type*)this->factory)->o : this->o);
+  Fl_Counter *myo = (Fl_Counter*)(w==4 ? ((Fl_Widget_Type*)factory)->o : o);
   switch (w) {
     case 4:
-    case 0: f = o->textfont(); s = o->textsize(); c = o->textcolor(); break;
-    case 1: o->textfont(f); break;
-    case 2: o->textsize(s); break;
-    case 3: o->textcolor(c); break;
+    case 0: f = myo->textfont(); s = myo->textsize(); c = myo->textcolor(); break;
+    case 1: myo->textfont(f); break;
+    case 2: myo->textsize(s); break;
+    case 3: myo->textcolor(c); break;
   }
   return 1;
 }
@@ -255,22 +329,109 @@ class Fl_Input_Type : public Fl_Widget_Type {
 public:
   virtual const char *type_name() {return "Fl_Input";}
   Fl_Widget *widget(int x,int y,int w,int h) {
-    Fl_Input *o = new Fl_Input(x,y,w,h,"input:");
-    o->value("Text Input");
-    return o;
+    Fl_Input *myo = new Fl_Input(x,y,w,h,"input:");
+    myo->value("Text Input");
+    return myo;
   }
   Fl_Widget_Type *_make() {return new Fl_Input_Type();}
+  int pixmapID() { return 14; }
 };
 static Fl_Input_Type Fl_Input_type;
 
 int Fl_Input_Type::textstuff(int w, Fl_Font& f, int& s, Fl_Color& c) {
-  Fl_Input_ *o = (Fl_Input_*)(w==4 ? ((Fl_Widget_Type*)this->factory)->o : this->o);
+  Fl_Input_ *myo = (Fl_Input_*)(w==4 ? ((Fl_Widget_Type*)factory)->o : o);
   switch (w) {
     case 4:
-    case 0: f = o->textfont(); s = o->textsize(); c = o->textcolor(); break;
-    case 1: o->textfont(f); break;
-    case 2: o->textsize(s); break;
-    case 3: o->textcolor(c); break;
+    case 0: f = myo->textfont(); s = myo->textsize(); c = myo->textcolor(); break;
+    case 1: myo->textfont(f); break;
+    case 2: myo->textsize(s); break;
+    case 3: myo->textcolor(c); break;
+  }
+  return 1;
+}
+
+////////////////////////////////////////////////////////////////
+
+#include <FL/Fl_File_Input.H>
+class Fl_File_Input_Type : public Fl_Widget_Type {
+  Fl_Menu_Item *subtypes() {return 0;}
+  int textstuff(int w, Fl_Font& f, int& s, Fl_Color& c);
+public:
+  virtual const char *type_name() {return "Fl_File_Input";}
+  Fl_Widget *widget(int x,int y,int w,int h) {
+    Fl_File_Input *myo = new Fl_File_Input(x,y,w,h,"file:");
+    myo->value("/now/is/the/time/for/a/filename.ext");
+    return myo;
+  }
+  Fl_Widget_Type *_make() {return new Fl_File_Input_Type();}
+  int pixmapID() { return 30; }
+};
+static Fl_File_Input_Type Fl_File_Input_type;
+
+int Fl_File_Input_Type::textstuff(int w, Fl_Font& f, int& s, Fl_Color& c) {
+  Fl_File_Input *myo = (Fl_File_Input*)(w==4 ? ((Fl_Widget_Type*)factory)->o : o);
+  switch (w) {
+    case 4:
+    case 0: f = myo->textfont(); s = myo->textsize(); c = myo->textcolor(); break;
+    case 1: myo->textfont(f); break;
+    case 2: myo->textsize(s); break;
+    case 3: myo->textcolor(c); break;
+  }
+  return 1;
+}
+
+////////////////////////////////////////////////////////////////
+
+#include <FL/Fl_Text_Display.H>
+class Fl_Text_Display_Type : public Fl_Widget_Type {
+  int textstuff(int w, Fl_Font& f, int& s, Fl_Color& c);
+public:
+  virtual const char *type_name() {return "Fl_Text_Display";}
+  Fl_Widget *widget(int x,int y,int w,int h) {
+    Fl_Text_Display *myo = new Fl_Text_Display(x,y,w,h);
+    return myo;
+  }
+  Fl_Widget_Type *_make() {return new Fl_Text_Display_Type();}
+  int pixmapID() { return 28; }
+};
+static Fl_Text_Display_Type Fl_Text_Display_type;
+
+int Fl_Text_Display_Type::textstuff(int w, Fl_Font& f, int& s, Fl_Color& c) {
+  Fl_Text_Display *myo = (Fl_Text_Display*)(w==4 ? ((Fl_Widget_Type*)factory)->o : o);
+  switch (w) {
+    case 4:
+    case 0: f = myo->textfont(); s = myo->textsize(); c = myo->textcolor(); break;
+    case 1: myo->textfont(f); break;
+    case 2: myo->textsize(s); break;
+    case 3: myo->textcolor(c); break;
+  }
+  return 1;
+}
+
+////////////////////////////////////////////////////////////////
+
+#include <FL/Fl_Text_Editor.H>
+class Fl_Text_Editor_Type : public Fl_Widget_Type {
+  int textstuff(int w, Fl_Font& f, int& s, Fl_Color& c);
+public:
+  virtual const char *type_name() {return "Fl_Text_Editor";}
+  Fl_Widget *widget(int x,int y,int w,int h) {
+    Fl_Text_Editor *myo = new Fl_Text_Editor(x,y,w,h);
+    return myo;
+  }
+  Fl_Widget_Type *_make() {return new Fl_Text_Editor_Type();}
+  int pixmapID() { return 29; }
+};
+static Fl_Text_Editor_Type Fl_Text_Editor_type;
+
+int Fl_Text_Editor_Type::textstuff(int w, Fl_Font& f, int& s, Fl_Color& c) {
+  Fl_Text_Editor *myo = (Fl_Text_Editor*)(w==4 ? ((Fl_Widget_Type*)factory)->o : o);
+  switch (w) {
+    case 4:
+    case 0: f = myo->textfont(); s = myo->textsize(); c = myo->textcolor(); break;
+    case 1: myo->textfont(f); break;
+    case 2: myo->textsize(s); break;
+    case 3: myo->textcolor(c); break;
   }
   return 1;
 }
@@ -284,11 +445,43 @@ public:
   Fl_Widget *widget(int x,int y,int w,int h) {
     return new Fl_Clock(x,y,w,h);}
   Fl_Widget_Type *_make() {return new Fl_Clock_Type();}
+  int pixmapID() { return 34; }
 };
 static Fl_Clock_Type Fl_Clock_type;
 
 ////////////////////////////////////////////////////////////////
 
+#include <FL/Fl_Help_View.H>
+class Fl_Help_View_Type : public Fl_Widget_Type {
+public:
+  virtual const char *type_name() {return "Fl_Help_View";}
+  Fl_Widget *widget(int x,int y,int w,int h) {
+    Fl_Help_View *myo = new Fl_Help_View(x,y,w,h);
+    myo->value("<HTML><BODY><H1>Fl_Help_View Widget</H1>"
+               "<P>This is a Fl_Help_View widget.</P></BODY></HTML>");
+    return myo;}
+  Fl_Widget_Type *_make() {return new Fl_Help_View_Type();}
+  int pixmapID() { return 35; }
+};
+static Fl_Help_View_Type Fl_Help_View_type;
+
+////////////////////////////////////////////////////////////////
+
+#include <FL/Fl_Progress.H>
+class Fl_Progress_Type : public Fl_Widget_Type {
+public:
+  virtual const char *type_name() {return "Fl_Progress";}
+  Fl_Widget *widget(int x,int y,int w,int h) {
+    Fl_Progress *myo = new Fl_Progress(x,y,w,h);
+    myo->value(50);
+    return myo;}
+  Fl_Widget_Type *_make() {return new Fl_Progress_Type();}
+  int pixmapID() { return 36; }
+};
+static Fl_Progress_Type Fl_Progress_type;
+
+////////////////////////////////////////////////////////////////
+
 #include <FL/Fl_Adjuster.H>
 class Fl_Adjuster_Type : public Fl_Widget_Type {
   int is_valuator() const {return 1;}
@@ -297,6 +490,7 @@ public:
   Fl_Widget *widget(int x,int y,int w,int h) {
     return new Fl_Adjuster(x,y,w,h);}
   Fl_Widget_Type *_make() {return new Fl_Adjuster_Type();}
+  int pixmapID() { return 40; }
 };
 static Fl_Adjuster_Type Fl_Adjuster_type;
 
@@ -316,6 +510,7 @@ public:
   Fl_Widget *widget(int x,int y,int w,int h) {
     return new Fl_Dial(x,y,w,h);}
   Fl_Widget_Type *_make() {return new Fl_Dial_Type();}
+  int pixmapID() { return 42; }
 };
 static Fl_Dial_Type Fl_Dial_type;
 
@@ -334,6 +529,7 @@ public:
   Fl_Widget *widget(int x,int y,int w,int h) {
     return new Fl_Roller(x,y,w,h);}
   Fl_Widget_Type *_make() {return new Fl_Roller_Type();}
+  int pixmapID() { return 43; }
 };
 static Fl_Roller_Type Fl_Roller_type;
 
@@ -356,6 +552,7 @@ public:
   Fl_Widget *widget(int x,int y,int w,int h) {
     return new Fl_Slider(x,y,w,h);}
   Fl_Widget_Type *_make() {return new Fl_Slider_Type();}
+  int pixmapID() { return 37; }
 };
 static Fl_Slider_Type Fl_Slider_type;
 
@@ -370,6 +567,7 @@ public:
   Fl_Widget *widget(int x,int y,int w,int h) {
     return new Fl_Scrollbar(x,y,w,h);}
   Fl_Widget_Type *_make() {return new Fl_Scrollbar_Type();}
+  int pixmapID() { return 38; }
 };
 static Fl_Scrollbar_Type Fl_Scrollbar_type;
 
@@ -377,19 +575,20 @@ static Fl_Scrollbar_Type Fl_Scrollbar_type;
 
 #include <FL/Fl_Output.H>
 static Fl_Menu_Item output_type_menu[] = {
-  {"Normal",0,0,(void*)FL_NORMAL_INPUT},
-  {"Multiline",0,0,(void*)FL_MULTILINE_INPUT},
+  {"Normal",0,0,(void*)FL_NORMAL_OUTPUT},
+  {"Multiline",0,0,(void*)FL_MULTILINE_OUTPUT},
   {0}};
 class Fl_Output_Type : public Fl_Input_Type {
   Fl_Menu_Item *subtypes() {return output_type_menu;}
 public:
   virtual const char *type_name() {return "Fl_Output";}
   Fl_Widget *widget(int x,int y,int w,int h) {
-    Fl_Output *o = new Fl_Output(x,y,w,h,"output:");
-    o->value("Text Output");
-    return o;
+    Fl_Output *myo = new Fl_Output(x,y,w,h,"output:");
+    myo->value("Text Output");
+    return myo;
   }
   Fl_Widget_Type *_make() {return new Fl_Output_Type();}
+  int pixmapID() { return 27; }
 };
 static Fl_Output_Type Fl_Output_type;
 
@@ -402,21 +601,22 @@ public:
   int textstuff(int w, Fl_Font& f, int& s, Fl_Color& c);
   int is_valuator() const {return 1;}
   Fl_Widget *widget(int x,int y,int w,int h) {
-    Fl_Value_Input *o = new Fl_Value_Input(x,y,w,h,"value:");
-    return o;
+    Fl_Value_Input *myo = new Fl_Value_Input(x,y,w,h,"value:");
+    return myo;
   }
   Fl_Widget_Type *_make() {return new Fl_Value_Input_Type();}
+  int pixmapID() { return 44; }
 };
 static Fl_Value_Input_Type Fl_Value_Input_type;
 
 int Fl_Value_Input_Type::textstuff(int w, Fl_Font& f, int& s, Fl_Color& c) {
-  Fl_Value_Input *o = (Fl_Value_Input*)(w==4 ? ((Fl_Widget_Type*)this->factory)->o : this->o);
+  Fl_Value_Input *myo = (Fl_Value_Input*)(w==4 ? ((Fl_Widget_Type*)factory)->o : o);
   switch (w) {
     case 4:
-    case 0: f = o->textfont(); s = o->textsize(); c = o->textcolor(); break;
-    case 1: o->textfont(f); break;
-    case 2: o->textsize(s); break;
-    case 3: o->textcolor(c); break;
+    case 0: f = myo->textfont(); s = myo->textsize(); c = myo->textcolor(); break;
+    case 1: myo->textfont(f); break;
+    case 2: myo->textsize(s); break;
+    case 3: myo->textcolor(c); break;
   }
   return 1;
 }
@@ -430,21 +630,22 @@ public:
   int textstuff(int w, Fl_Font& f, int& s, Fl_Color& c);
   int is_valuator() const {return 1;}
   Fl_Widget *widget(int x,int y,int w,int h) {
-    Fl_Value_Output *o = new Fl_Value_Output(x,y,w,h,"value:");
-    return o;
+    Fl_Value_Output *myo = new Fl_Value_Output(x,y,w,h,"value:");
+    return myo;
   }
   Fl_Widget_Type *_make() {return new Fl_Value_Output_Type();}
+  int pixmapID() { return 45; }
 };
 static Fl_Value_Output_Type Fl_Value_Output_type;
 
 int Fl_Value_Output_Type::textstuff(int w, Fl_Font& f, int& s, Fl_Color& c) {
-  Fl_Value_Output *o = (Fl_Value_Output*)(w==4 ? ((Fl_Widget_Type*)this->factory)->o : this->o);
+  Fl_Value_Output *myo = (Fl_Value_Output*)(w==4 ? ((Fl_Widget_Type*)factory)->o : o);
   switch (w) {
     case 4:
-    case 0: f = o->textfont(); s = o->textsize(); c = o->textcolor(); break;
-    case 1: o->textfont(f); break;
-    case 2: o->textsize(s); break;
-    case 3: o->textcolor(c); break;
+    case 0: f = myo->textfont(); s = myo->textsize(); c = myo->textcolor(); break;
+    case 1: myo->textfont(f); break;
+    case 2: myo->textsize(s); break;
+    case 3: myo->textcolor(c); break;
   }
   return 1;
 }
@@ -459,17 +660,18 @@ public:
   Fl_Widget *widget(int x,int y,int w,int h) {
     return new Fl_Value_Slider(x,y,w,h);}
   Fl_Widget_Type *_make() {return new Fl_Value_Slider_Type();}
+  int pixmapID() { return 39; }
 };
 static Fl_Value_Slider_Type Fl_Value_Slider_type;
 
 int Fl_Value_Slider_Type::textstuff(int w, Fl_Font& f, int& s, Fl_Color& c) {
-  Fl_Value_Slider *o = (Fl_Value_Slider*)(w==4 ? ((Fl_Widget_Type*)this->factory)->o : this->o);
+  Fl_Value_Slider *myo = (Fl_Value_Slider*)(w==4 ? ((Fl_Widget_Type*)factory)->o : o);
   switch (w) {
     case 4:
-    case 0: f = o->textfont(); s = o->textsize(); c = o->textcolor(); break;
-    case 1: o->textfont(f); break;
-    case 2: o->textsize(s); break;
-    case 3: o->textcolor(c); break;
+    case 0: f = myo->textfont(); s = myo->textsize(); c = myo->textcolor(); break;
+    case 1: myo->textfont(f); break;
+    case 2: myo->textsize(s); break;
+    case 3: myo->textcolor(c); break;
   }
   return 1;
 }
@@ -493,6 +695,7 @@ extern class Fl_Menu_Bar_Type Fl_Menu_Bar_type;
 extern class Fl_Menu_Button_Type Fl_Menu_Button_type;
 extern class Fl_Menu_Item_Type Fl_Menu_Item_type;
 extern class Fl_Submenu_Type Fl_Submenu_type;
+extern class Fl_Wizard_Type Fl_Wizard_type;
 
 extern void select(Fl_Type *,int);
 extern void select_only(Fl_Type *);
@@ -518,14 +721,15 @@ Fl_Menu_Item New_Menu[] = {
   {0,0,cb,(void*)&Fl_Tabs_type},
   {0,0,cb,(void*)&Fl_Scroll_type},
   {0,0,cb,(void*)&Fl_Tile_type},
+  {0,0,cb,(void*)&Fl_Wizard_type},
 {0},
 {"buttons",0,0,0,FL_SUBMENU},
   {0,0,cb,(void*)&Fl_Button_type},
   {0,0,cb,(void*)&Fl_Return_Button_type},
   {0,0,cb,(void*)&Fl_Light_Button_type},
   {0,0,cb,(void*)&Fl_Check_Button_type},
-  {0,0,cb,(void*)&Fl_Round_Button_type},
   {0,0,cb,(void*)&Fl_Repeat_Button_type},
+  {0,0,cb,(void*)&Fl_Round_Button_type},
 {0},
 {"valuators",0,0,0,FL_SUBMENU},
   {0,0,cb,(void*)&Fl_Slider_type},
@@ -539,8 +743,11 @@ Fl_Menu_Item New_Menu[] = {
   {0,0,cb,(void*)&Fl_Value_Output_type},
 {0},
 {"text",0,0,0,FL_SUBMENU},
+  {0,0,cb,(void*)&Fl_File_Input_type},
   {0,0,cb,(void*)&Fl_Input_type},
   {0,0,cb,(void*)&Fl_Output_type},
+  {0,0,cb,(void*)&Fl_Text_Display_type},
+  {0,0,cb,(void*)&Fl_Text_Editor_type},
 {0},
 {"menus",0,0,0,FL_SUBMENU},
   {0,0,cb,(void*)&Fl_Menu_Bar_type},
@@ -549,20 +756,53 @@ Fl_Menu_Item New_Menu[] = {
   {0,0,cb, (void*)&Fl_Submenu_type},
   {0,0,cb, (void*)&Fl_Menu_Item_type},
 {0},
-{"other",0,0,0,FL_SUBMENU},
+{"browsers",0,0,0,FL_SUBMENU},
   {0,0,cb,(void*)&Fl_Browser_type},
+  {0,0,cb,(void*)&Fl_Check_Browser_type},
+  {0,0,cb,(void*)&Fl_File_Browser_type},
+{0},
+{"other",0,0,0,FL_SUBMENU},
   {0,0,cb,(void*)&Fl_Box_type},
   {0,0,cb,(void*)&Fl_Clock_type},
+  {0,0,cb,(void*)&Fl_Help_View_type},
+  {0,0,cb,(void*)&Fl_Progress_type},
 {0},
 {0}};
 
+#include <FL/Fl_Multi_Label.H>
+
+// modify a menuitem to display an icon in front of the label
+static void make_iconlabel( Fl_Menu_Item *mi, Fl_Image *ic, const char *txt )
+{
+  if (ic) {
+    char *t1 = new char[strlen(txt)+3];
+    strcpy( t1, " " );
+    strcat(t1, txt);
+    mi->image( ic );
+    Fl_Multi_Label *ml = new Fl_Multi_Label;
+    ml->labela = (char*)ic;
+    ml->labelb = t1;
+    ml->typea = _FL_IMAGE_LABEL;
+    ml->typeb = FL_NORMAL_LABEL;
+    ml->label( mi );
+  }
+  else if (txt!=mi->text)
+    mi->label(txt);
+}
+
 void fill_in_New_Menu() {
   for (unsigned i = 0; i < sizeof(New_Menu)/sizeof(*New_Menu); i++) {
     Fl_Menu_Item *m = New_Menu+i;
-    if (m->user_data() && !m->text) {
-      const char *n = ((Fl_Type*)(m->user_data()))->type_name();
-      if (!strncmp(n,"Fl_",3)) n += 3;
-      m->text = n;
+    if (m->user_data()) {
+      Fl_Type *t = (Fl_Type*)m->user_data();
+      if (m->text) {
+	make_iconlabel( m, pixmap[t->pixmapID()], m->label() );
+      }
+      else {
+	const char *n = t->type_name();
+	if (!strncmp(n,"Fl_",3)) n += 3;
+	make_iconlabel( m, pixmap[t->pixmapID()], n );
+      }
     }
   }
 }
@@ -722,5 +962,5 @@ int lookup_symbol(const char *name, int &v, int numberok) {
 }
 
 //
-// End of "$Id: factory.cxx,v 1.4.2.8 2000/11/21 21:37:08 easysw Exp $".
+// End of "$Id: factory.cxx,v 1.4.2.11.2.10 2002/10/30 22:30:30 matthiaswm Exp $".
 //
