@@ -1,9 +1,9 @@
 //
-// "$Id: Fl.cxx,v 1.24.2.34 2000/11/20 19:02:20 easysw Exp $"
+// "$Id: Fl.cxx,v 1.24.2.41.2.59 2003/01/30 21:41:14 easysw Exp $"
 //
 // Main event handling code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -26,9 +26,10 @@
 #include <FL/Fl.H>
 #include <FL/Fl_Window.H>
 #include <FL/x.H>
+#include <FL/Fl_Tooltip.H>
 #include <ctype.h>
 #include <stdlib.h>
-#include <string.h>
+#include "flstring.h"
 
 
 //
@@ -40,53 +41,64 @@ Fl_Widget	*Fl::belowmouse_,
 		*Fl::focus_,
 		*Fl::selection_owner_;
 int		Fl::damage_,
+		Fl::e_number,
 		Fl::e_x,
 		Fl::e_y,
 		Fl::e_x_root,
 		Fl::e_y_root,
+		Fl::e_dx,
+		Fl::e_dy,
 		Fl::e_state,
 		Fl::e_clicks,
 		Fl::e_is_click,
 		Fl::e_keysym;
 char		*Fl::e_text = (char *)"";
 int		Fl::e_length;
+int		Fl::visible_focus_ = 1,
+		Fl::dnd_text_ops_ = 1;
+
+
+//
+// 'Fl::version()' - Return the API version number...
+//
+
+double
+Fl::version() {
+  return FL_VERSION;
+}
+
 
 //
 // 'Fl:event_inside()' - Return whether or not the mouse event is inside
 //                       the given rectangle.
 //
 
-int Fl::event_inside(int x,int y,int w,int h) /*const*/ {
-  int mx = event_x() - x;
-  int my = event_y() - y;
-  return (mx >= 0 && mx < w && my >= 0 && my < h);
+int Fl::event_inside(int xx,int yy,int ww,int hh) /*const*/ {
+  int mx = e_x - xx;
+  int my = e_y - yy;
+  return (mx >= 0 && mx < ww && my >= 0 && my < hh);
 }
 
 int Fl::event_inside(const Fl_Widget *o) /*const*/ {
-  return event_inside(o->x(),o->y(),o->w(),o->h());
+  int mx = e_x - o->x();
+  int my = e_y - o->y();
+  return (mx >= 0 && mx < o->w() && my >= 0 && my < o->h());
 }
 
 ////////////////////////////////////////////////////////////////
-// Timeouts and Fl::wait()
-
-void (*Fl::idle)();
-
-// Timeouts are insert-sorted into order.  This works good if there
-// are only a small number:
-
-static struct Timeout {
+// Timeouts are stored in a sorted list, so only the first one needs
+// to be checked to see if any should be called.
+  
+struct Timeout {
   double time;
   void (*cb)(void*);
   void* arg;
-} * timeout;
-static int numtimeouts;
-static int timeout_array_size;
-
-extern int fl_wait(double time); // warning: assummes time >= 0.0
-extern int fl_ready();
+  Timeout* next;
+};
+static Timeout* first_timeout, *free_timeout;
 
 #ifndef WIN32
-#include <sys/time.h>
+#  include <sys/time.h>
 #endif
 
 // I avoid the overhead of getting the current time when we have no
@@ -96,68 +108,166 @@ extern int fl_ready();
 static char reset_clock = 1;
 
 static void elapse_timeouts() {
-
 #ifdef WIN32
-
   unsigned long newclock = GetTickCount();
   static unsigned long prevclock;
-  if (reset_clock) {
-    prevclock = newclock;
-    reset_clock = 0;
-    return;
-  }
-  if (newclock <= prevclock) return;
   double elapsed = (newclock-prevclock)/1000.0;
   prevclock = newclock;
-
 #else
-
   static struct timeval prevclock;
   struct timeval newclock;
   gettimeofday(&newclock, NULL);
-  if (reset_clock) {
-    prevclock.tv_sec = newclock.tv_sec;
-    prevclock.tv_usec = newclock.tv_usec;
-    reset_clock = 0;
-    return;
-  }
   double elapsed = newclock.tv_sec - prevclock.tv_sec +
     (newclock.tv_usec - prevclock.tv_usec)/1000000.0;
   prevclock.tv_sec = newclock.tv_sec;
   prevclock.tv_usec = newclock.tv_usec;
-  if (elapsed <= 0) return;
-
 #endif
-
-  for (int i=0; i<numtimeouts; i++) timeout[i].time -= elapsed;
+  if (reset_clock) {
+    reset_clock = 0;
+  } else if (elapsed > 0) {
+    for (Timeout* t = first_timeout; t; t = t->next) t->time -= elapsed;
+  }
 }
 
+// Continuously-adjusted error value, this is a number <= 0 for how late
+// we were at calling the last timeout. This appears to make repeat_timeout
+// very accurate even when processing takes a significant portion of the
+// time interval:
+static double missed_timeout_by;
+
+void Fl::add_timeout(double time, Fl_Timeout_Handler cb, void *argp) {
+  elapse_timeouts();
+  repeat_timeout(time, cb, argp);
+}
+
+void Fl::repeat_timeout(double time, Fl_Timeout_Handler cb, void *argp) {
+  time += missed_timeout_by; if (time < -.05) time = 0;
+  Timeout* t = free_timeout;
+  if (t) free_timeout = t->next;
+  else t = new Timeout;
+  t->time = time;
+  t->cb = cb;
+  t->arg = argp;
+  // insert-sort the new timeout:
+  Timeout** p = &first_timeout; 
+  while (*p && (*p)->time <= time) p = &((*p)->next);
+  t->next = *p;
+  *p = t;
+}
+
+int Fl::has_timeout(Fl_Timeout_Handler cb, void *argp) {
+  for (Timeout* t = first_timeout; t; t = t->next)
+    if (t->cb == cb && t->arg == argp) return 1;
+  return 0;
+}
+
+void Fl::remove_timeout(Fl_Timeout_Handler cb, void *argp) {
+  // This version removes all matching timeouts, not just the first one.
+  // This may change in the future.
+  for (Timeout** p = &first_timeout; *p;) {
+    Timeout* t = *p;
+    if (t->cb == cb && (t->arg == argp || !argp)) {
+      *p = t->next;
+      t->next = free_timeout;
+      free_timeout = t;
+    } else {
+      p = &(t->next);
+    }
+  }
+}
+
+////////////////////////////////////////////////////////////////
+// Checks are just stored in a list. They are called in the reverse
+// order that they were added (this may change in the future).
+// This is a bit messy because I want to allow checks to be added,
+// removed, and have wait() called from inside them, to do this
+// next_check points at the next unprocessed one for the outermost
+// call to Fl::wait().
+
+struct Check {
+  void (*cb)(void*);
+  void* arg;
+  Check* next;
+};
+static Check* first_check, *next_check, *free_check;
+
+void Fl::add_check(Fl_Timeout_Handler cb, void *argp) {
+  Check* t = free_check;
+  if (t) free_check = t->next;
+  else t = new Check;
+  t->cb = cb;
+  t->arg = argp;
+  t->next = first_check;
+  if (next_check == first_check) next_check = t;
+  first_check = t;
+}
+
+void Fl::remove_check(Fl_Timeout_Handler cb, void *argp) {
+  for (Check** p = &first_check; *p;) {
+    Check* t = *p;
+    if (t->cb == cb && t->arg == argp) {
+      if (next_check == t) next_check = t->next;
+      *p = t->next;
+      t->next = free_check;
+      free_check = t;
+    } else {
+      p = &(t->next);
+    }
+  }
+}
+
+////////////////////////////////////////////////////////////////
+// wait/run/check/ready:
+
+void (*Fl::idle)(); // see Fl_add_idle.cxx for the add/remove functions
+
+extern int fl_ready(); // in Fl_<platform>.cxx
+extern int fl_wait(double time); // in Fl_<platform>.cxx
+
 static char in_idle;
 
 double Fl::wait(double time_to_wait) {
-  if (numtimeouts) {
+  if (first_timeout) {
     elapse_timeouts();
-    if (timeout[0].time <= time_to_wait) time_to_wait = timeout[0].time;
-    while (numtimeouts) {
-      if (timeout[0].time > 0) break;
+    Timeout *t;
+    while ((t = first_timeout)) {
+      if (t->time > 0) break;
       // The first timeout in the array has expired.
+      missed_timeout_by = t->time;
       // We must remove timeout from array before doing the callback:
-      void (*cb)(void*) = timeout[0].cb;
-      void *arg = timeout[0].arg;
-      numtimeouts--;
-      if (numtimeouts)
-	memmove(timeout, timeout+1, numtimeouts*sizeof(Timeout));
+      void (*cb)(void*) = t->cb;
+      void *argp = t->arg;
+      first_timeout = t->next;
+      t->next = free_timeout;
+      free_timeout = t;
       // Now it is safe for the callback to do add_timeout:
-      cb(arg);
+      cb(argp);
     }
   } else {
     reset_clock = 1; // we are not going to check the clock
   }
+  // checks are a bit messy so that add/remove and wait may be called
+  // from inside them without causing an infinite loop:
+  if (next_check == first_check) {
+    while (next_check) {
+      Check* checkp = next_check;
+      next_check = checkp->next;
+      (checkp->cb)(checkp->arg);
+    }
+    next_check = first_check;
+  }
+//  if (idle && !fl_ready()) {
   if (idle) {
-    if (!in_idle) {in_idle = 1; idle(); in_idle = 0;}
+    if (!in_idle) {
+      in_idle = 1;
+      idle();
+      in_idle = 0;
+    }
     // the idle function may turn off idle, we can then wait:
     if (idle) time_to_wait = 0.0;
   }
+  if (first_timeout && first_timeout->time < time_to_wait)
+    time_to_wait = first_timeout->time;
   if (time_to_wait <= 0.0) {
     // do flush second so that the results of events are visible:
     int ret = fl_wait(0.0);
@@ -187,58 +297,18 @@ int Fl::check() {
   return Fl_X::first != 0; // return true if there is a window
 }
 
+extern int fl_ready();
+
 int Fl::ready() {
-  if (numtimeouts) {
+  if (first_timeout) {
     elapse_timeouts();
-    if (timeout[0].time <= 0) return 1;
+    if (first_timeout->time <= 0) return 1;
   } else {
     reset_clock = 1;
   }
   return fl_ready();
 }
 
-void Fl::add_timeout(double t, Fl_Timeout_Handler cb, void *v) {
-  elapse_timeouts();
-  repeat_timeout(t, cb, v);
-}
-
-void Fl::repeat_timeout(double t, Fl_Timeout_Handler cb, void *v) {
-
-  if (numtimeouts >= timeout_array_size) {
-    timeout_array_size = 2*timeout_array_size+1;
-    timeout = (Timeout*)realloc(timeout, timeout_array_size*sizeof(Timeout));
-  }
-
-  // insert-sort the new timeout:
-  int i;
-  for (i=0; i<numtimeouts; i++) {
-    if (timeout[i].time > t) {
-      for (int j=numtimeouts; j>i; j--) timeout[j] = timeout[j-1];
-      break;
-    }
-  }
-  timeout[i].time = t;
-  timeout[i].cb = cb;
-  timeout[i].arg = v;
-
-  numtimeouts++;
-}
-
-int Fl::has_timeout(Fl_Timeout_Handler cb, void *v) {
-  for (int i=0; i<numtimeouts; i++)
-    if (timeout[i].cb == cb && timeout[i].arg==v) return 1;
-  return 0;
-}
-
-void Fl::remove_timeout(Fl_Timeout_Handler cb, void *v) {
-  int i,j;
-  for (i=j=0; i<numtimeouts; i++) {
-    if (timeout[i].cb == cb && timeout[i].arg==v) ;
-    else {if (j<i) timeout[j]=timeout[i]; j++;}
-  }
-  numtimeouts = j;
-}
-
 ////////////////////////////////////////////////////////////////
 // Window list management:
 
@@ -247,7 +317,11 @@ Fl_X* Fl_X::first;
 Fl_Window* fl_find(Window xid) {
   Fl_X *window;
   for (Fl_X **pp = &Fl_X::first; (window = *pp); pp = &window->next)
+#ifdef __APPLE__
+    if (window->xid == xid && !window->w->window()) {
+#else
     if (window->xid == xid) {
+#endif // __APPLE__
       if (window != Fl_X::first && !Fl::modal()) {
 	// make this window be first to speed up searches
 	// this is not done if modal is true to avoid messing up modal stack
@@ -261,13 +335,13 @@ Fl_Window* fl_find(Window xid) {
 }
 
 Fl_Window* Fl::first_window() {
-  Fl_X* x = Fl_X::first;
-  return x ? x->w : 0;
+  Fl_X* i = Fl_X::first;
+  return i ? i->w : 0;
 }
 
-Fl_Window* Fl::next_window(const Fl_Window* w) {
-  Fl_X* x = Fl_X::i(w)->next;
-  return x ? x->w : 0;
+Fl_Window* Fl::next_window(const Fl_Window* window) {
+  Fl_X* i = Fl_X::i(window)->next;
+  return i ? i->w : 0;
 }
 
 void Fl::first_window(Fl_Window* window) {
@@ -276,25 +350,31 @@ void Fl::first_window(Fl_Window* window) {
 }
 
 void Fl::redraw() {
-  for (Fl_X* x = Fl_X::first; x; x = x->next) x->w->redraw();
+  for (Fl_X* i = Fl_X::first; i; i = i->next) i->w->redraw();
 }
 
 void Fl::flush() {
   if (damage()) {
     damage_ = 0;
-    for (Fl_X* x = Fl_X::first; x; x = x->next) {
-      if (x->w->damage() && x->w->visible_r()) {
-       if (x->wait_for_expose) {
-         // leave Fl::damage() set so programs can tell damage still exists
-         damage_ = 1;
-       } else {
-         x->flush();
-         x->w->clear_damage();
-       }
-      }
+    for (Fl_X* i = Fl_X::first; i; i = i->next) {
+      if (i->wait_for_expose) {damage_ = 1; continue;}
+      Fl_Window* wi = i->w;
+      if (!wi->visible_r()) continue;
+      if (wi->damage()) {i->flush(); wi->clear_damage();}
+      // destroy damage regions for windows that don't use them:
+      if (i->region) {XDestroyRegion(i->region); i->region = 0;}
     }
   }
-#ifndef WIN32
+
+#ifdef WIN32
+  GdiFlush();
+#elif defined (__APPLE__)
+  GrafPtr port; GetPort( &port );
+  if ( port ) 
+  {
+    QDFlushPortBuffer( port, 0 );
+  }
+#else
   if (fl_display) XFlush(fl_display);
 #endif
 }
@@ -304,21 +384,39 @@ void Fl::flush() {
 
 struct handler_link {
   int (*handle)(int);
-  const handler_link *next;
+  handler_link *next;
 };
 
-static const handler_link *handlers = 0;
+static handler_link *handlers = 0;
 
-void Fl::add_handler(int (*h)(int)) {
+void Fl::add_handler(int (*ha)(int)) {
   handler_link *l = new handler_link;
-  l->handle = h;
+  l->handle = ha;
   l->next = handlers;
   handlers = l;
 }
 
-static int send_handlers(int event) {
-  for (const handler_link *h = handlers; h; h = h->next)
-    if (h->handle(event)) return 1;
+void Fl::remove_handler(int (*ha)(int)) {
+  handler_link *l, *p;
+
+  // Search for the handler in the list...
+  for (l = handlers, p = 0; l && l->handle != ha; p = l, l = l->next);
+
+  if (l) {
+    // Found it, so remove it from the list...
+    if (p) p->next = l->next;
+    else handlers = l->next;
+
+    // And free the record...
+    delete l;
+  }
+}
+
+int (*fl_local_grab)(int); // used by fl_dnd.cxx
+
+static int send_handlers(int e) {
+  for (const handler_link *hl = handlers; hl; hl = hl->next)
+    if (hl->handle(e)) return 1;
   return 0;
 }
 
@@ -327,25 +425,30 @@ static int send_handlers(int event) {
 Fl_Widget* fl_oldfocus; // kludge for Fl_Group...
 
 void Fl::focus(Fl_Widget *o) {
+  if (o && !o->visible_focus()) return;
   if (grab()) return; // don't do anything while grab is on
   Fl_Widget *p = focus_;
   if (o != p) {
     Fl::compose_reset();
     focus_ = o;
     fl_oldfocus = 0;
-    for (; p && !p->contains(o); p = p->parent()) {
+    for (; p; p = p->parent()) {
       p->handle(FL_UNFOCUS);
       fl_oldfocus = p;
     }
   }
 }
 
+static char dnd_flag = 0; // make 'belowmouse' send DND_LEAVE instead of LEAVE
+
 void Fl::belowmouse(Fl_Widget *o) {
   if (grab()) return; // don't do anything while grab is on
   Fl_Widget *p = belowmouse_;
   if (o != p) {
     belowmouse_ = o;
-    for (; p && !p->contains(o); p = p->parent()) p->handle(FL_LEAVE);
+    for (; p && !p->contains(o); p = p->parent()) {
+      p->handle(dnd_flag ? FL_DND_LEAVE : FL_LEAVE);
+    }
   }
 }
 
@@ -358,6 +461,10 @@ Fl_Window *fl_xmousewin;// which window X thinks has FL_ENTER
 Fl_Window *Fl::grab_;	// most recent Fl::grab()
 Fl_Window *Fl::modal_;	// topmost modal() window
 
+static void nothing(Fl_Widget *) {}
+void (*Fl_Tooltip::enter)(Fl_Widget *) = nothing;
+void (*Fl_Tooltip::exit)(Fl_Widget *) = nothing;
+
 // Update modal(), focus() and other state according to system state,
 // and send FL_ENTER, FL_LEAVE, FL_FOCUS, and/or FL_UNFOCUS events.
 // This is the only function that produces these events in response
@@ -372,7 +479,9 @@ void fl_fix_focus() {
   // set focus based on Fl::modal() and fl_xfocus
   Fl_Widget* w = fl_xfocus;
   if (w) {
-    Fl::e_keysym = 0; // make sure widgets don't think a keystroke moved focus
+    if (Fl::e_keysym < (FL_Button + FL_LEFT_MOUSE) ||
+        Fl::e_keysym > (FL_Button + FL_RIGHT_MOUSE))
+      Fl::e_keysym = 0; // make sure widgets don't think a keystroke moved focus
     while (w->parent()) w = w->parent();
     if (Fl::modal()) w = Fl::modal();
     if (!w->contains(Fl::focus()))
@@ -380,15 +489,17 @@ void fl_fix_focus() {
   } else
     Fl::focus(0);
 
+// MRS: Originally we checked the button state, but a user reported that it
+//      broke click-to-focus in FLWM?!?
+//  if (!(Fl::event_state() & 0x7f00000 /*FL_BUTTONS*/)) {
   if (!Fl::pushed()) {
-
     // set belowmouse based on Fl::modal() and fl_xmousewin:
     w = fl_xmousewin;
     if (w) {
       if (Fl::modal()) w = Fl::modal();
       if (!w->contains(Fl::belowmouse())) {
-	Fl::belowmouse(w);
 	w->handle(FL_ENTER);
+	if (!w->contains(Fl::belowmouse())) Fl::belowmouse(w);
       } else {
 	// send a FL_MOVE event so the enter/leave state is up to date
 	Fl::e_x = Fl::e_x_root-fl_xmousewin->x();
@@ -397,12 +508,13 @@ void fl_fix_focus() {
       }
     } else {
       Fl::belowmouse(0);
+      Fl_Tooltip::enter(0);
     }
   }
 }
 
 #ifndef WIN32
-Fl_Widget *fl_selection_requestor; // from Fl_cutpaste.C
+extern Fl_Widget *fl_selection_requestor; // from Fl_x.cxx
 #endif
 
 // This function is called by ~Fl_Widget() and by Fl_Widget::deactivate
@@ -415,14 +527,15 @@ Fl_Widget *fl_selection_requestor; // from Fl_cutpaste.C
 
 void fl_throw_focus(Fl_Widget *o) {
   if (o->contains(Fl::pushed())) Fl::pushed_ = 0;
-  if (o->contains(Fl::selection_owner())) Fl::selection_owner_ = 0;
 #ifndef WIN32
   if (o->contains(fl_selection_requestor)) fl_selection_requestor = 0;
 #endif
   if (o->contains(Fl::belowmouse())) Fl::belowmouse_ = 0;
   if (o->contains(Fl::focus())) Fl::focus_ = 0;
   if (o == fl_xfocus) fl_xfocus = 0;
+  if (o == Fl_Tooltip::current()) Fl_Tooltip::current(0);
   if (o == fl_xmousewin) fl_xmousewin = 0;
+  Fl_Tooltip::exit(o);
   fl_fix_focus();
 }
 
@@ -444,53 +557,81 @@ static int send(int event, Fl_Widget* to, Fl_Window* window) {
   return ret;
 }
 
-int Fl::handle(int event, Fl_Window* window)
+int Fl::handle(int e, Fl_Window* window)
 {
-  Fl_Widget* w = window;
+  e_number = e;
+  if (fl_local_grab) return fl_local_grab(e);
 
-  switch (event) {
+  Fl_Widget* wi = window;
+
+  switch (e) {
 
   case FL_CLOSE:
     if (grab() || modal() && window != modal()) return 0;
-    w->do_callback();
+    wi->do_callback();
     return 1;
 
   case FL_SHOW:
-    ((Fl_Widget*)w)->show();
+    wi->show(); // this calls Fl_Widget::show(), not Fl_Window::show()
     return 1;
 
   case FL_HIDE:
-    ((Fl_Widget*)w)->hide();
+    wi->hide(); // this calls Fl_Widget::hide(), not Fl_Window::hide()
     return 1;
 
   case FL_PUSH:
-    if (grab()) w = grab();
-    else if (modal() && w != modal()) return 0;
-    pushed_ = w;
-    if (send(event, w, window)) return 1;
+    if (grab()) wi = grab();
+    else if (modal() && wi != modal()) return 0;
+    pushed_ = wi;
+    Fl_Tooltip::current(wi);
+    if (send(e, wi, window)) return 1;
     // raise windows that are clicked on:
     window->show();
     return 1;
 
+  case FL_DND_ENTER:
+  case FL_DND_DRAG:
+    dnd_flag = 1;
+    break;
+
+  case FL_DND_LEAVE:
+    dnd_flag = 1;
+    belowmouse(0);
+    dnd_flag = 0;
+    return 1;
+
+  case FL_DND_RELEASE:
+    wi = belowmouse();
+    break;
+
   case FL_MOVE:
   case FL_DRAG:
     fl_xmousewin = window; // this should already be set, but just in case.
     if (pushed()) {
-      w = pushed();
-      event = FL_DRAG;
-    } else if (modal() && w != modal()) {
-      w = 0;
+      wi = pushed();
+      if (grab()) wi = grab();
+      e_number = e = FL_DRAG;
+      break;
     }
-    if (grab()) w = grab();
-    break;
+    if (modal() && wi != modal()) wi = 0;
+    if (grab()) wi = grab();
+    {Fl_Widget* pbm = belowmouse();
+    int ret = (wi && send(e, wi, window));
+    if (pbm != belowmouse()) Fl_Tooltip::enter(belowmouse());
+    return ret;}
 
   case FL_RELEASE: {
-    if (pushed()) {
-      w = pushed();
+//    printf("FL_RELEASE: window=%p, pushed() = %p, grab() = %p, modal() = %p\n",
+//           window, pushed(), grab(), modal());
+
+    if (grab()) {
+      wi = grab();
       pushed_ = 0; // must be zero before callback is done!
-    }
-    if (grab()) w = grab();
-    int r = send(event, w, window);
+    } else if (pushed()) {
+      wi = pushed();
+      pushed_ = 0; // must be zero before callback is done!
+    } else if (modal() && wi != modal()) return 0;
+    int r = send(e, wi, window);
     fl_fix_focus();
     return r;}
 
@@ -502,11 +643,13 @@ int Fl::handle(int event, Fl_Window* window)
     return 1;
 
   case FL_KEYBOARD:
+    Fl_Tooltip::enter((Fl_Widget*)0);
+
     fl_xfocus = window; // this should not happen!  But maybe it does:
 
     // Try it as keystroke, sending it to focus and all parents:
-    for (w = grab() ? grab() : focus(); w; w = w->parent())
-      if (send(FL_KEYBOARD, w, window)) return 1;
+    for (wi = grab() ? grab() : focus(); wi; wi = wi->parent())
+      if (send(FL_KEYBOARD, wi, window)) return 1;
 
     // recursive call to try shortcut:
     if (handle(FL_SHORTCUT, window)) return 1;
@@ -516,23 +659,22 @@ int Fl::handle(int event, Fl_Window* window)
     {char* c = (char*)event_text(); // cast away const
     if (!isalpha(*c)) return 0;
     *c = isupper(*c) ? tolower(*c) : toupper(*c);}
-    event = FL_SHORTCUT;
+    e_number = e = FL_SHORTCUT;
 
   case FL_SHORTCUT:
-
-    if (grab()) {w = grab(); break;} // send it to grab window
+    if (grab()) {wi = grab(); break;} // send it to grab window
 
     // Try it as shortcut, sending to mouse widget and all parents:
-    w = belowmouse(); if (!w) {w = modal(); if (!w) w = window;}
-    for (; w; w = w->parent()) if (send(FL_SHORTCUT, w, window)) return 1;
+    wi = belowmouse(); if (!wi) {wi = modal(); if (!wi) wi = window;}
+    for (; wi; wi = wi->parent()) if (send(FL_SHORTCUT, wi, window)) return 1;
 
     // try using add_handle() functions:
     if (send_handlers(FL_SHORTCUT)) return 1;
 
     // make Escape key close windows:
     if (event_key()==FL_Escape) {
-      w = modal(); if (!w) w = window;
-      w->do_callback();
+      wi = modal(); if (!wi) wi = window;
+      wi->do_callback();
       return 1;
     }
 
@@ -541,48 +683,78 @@ int Fl::handle(int event, Fl_Window* window)
   case FL_ENTER:
     fl_xmousewin = window;
     fl_fix_focus();
+    Fl_Tooltip::enter(belowmouse());
     return 1;
 
   case FL_LEAVE:
+    if (!pushed_) {
+      belowmouse(0);
+      Fl_Tooltip::enter(0);
+    }
     if (window == fl_xmousewin) {fl_xmousewin = 0; fl_fix_focus();}
     return 1;
 
+  case FL_MOUSEWHEEL:
+    fl_xfocus = window; // this should not happen!  But maybe it does:
+
+    // Try it as keystroke, sending it to focus and all parents:
+    for (wi = grab() ? grab() : focus(); wi; wi = wi->parent())
+      if (send(FL_MOUSEWHEEL, wi, window)) return 1;
   default:
     break;
   }
-  if (w && send(event, w, window)) return 1;
-  return send_handlers(event);
+  if (wi && send(e, wi, window)) {
+    dnd_flag = 0;
+    return 1;
+  }
+  dnd_flag = 0;
+  return send_handlers(e);
 }
 
 ////////////////////////////////////////////////////////////////
 // hide() destroys the X window, it does not do unmap!
 
+#if !defined(WIN32) && USE_XFT
+extern void fl_destroy_xft_draw(Window);
+#endif
+
 void Fl_Window::hide() {
   clear_visible();
+
   if (!shown()) return;
 
   // remove from the list of windows:
-  Fl_X* x = i;
+  Fl_X* ip = i;
   Fl_X** pp = &Fl_X::first;
-  for (; *pp != x; pp = &(*pp)->next) if (!*pp) return;
-  *pp = x->next;
+  for (; *pp != ip; pp = &(*pp)->next) if (!*pp) return;
+  *pp = ip->next;
+
+#ifdef __APPLE__
+  // remove all childwindow links
+  for ( Fl_X *pc = Fl_X::first; pc; pc = pc->next )
+  { 
+    if ( pc->xidNext == ip ) pc->xidNext = ip->xidNext;
+    if ( pc->xidChildren == ip ) pc->xidChildren = ip->xidNext;   
+  }
+#endif // __APPLE__
+
   i = 0;
 
   // recursively remove any subwindows:
-  for (Fl_X *w = Fl_X::first; w;) {
-    Fl_Window* W = w->w;
+  for (Fl_X *wi = Fl_X::first; wi;) {
+    Fl_Window* W = wi->w;
     if (W->window() == this) {
       W->hide();
       W->set_visible();
-      w = Fl_X::first;
-    } else w = w->next;
+      wi = Fl_X::first;
+    } else wi = wi->next;
   }
 
   if (this == Fl::modal_) { // we are closing the modal window, find next one:
-    Fl_Window* w;
-    for (w = Fl::first_window(); w; w = Fl::next_window(w))
-      if (w->modal()) break;
-    Fl::modal_ = w;
+    Fl_Window* W;
+    for (W = Fl::first_window(); W; W = Fl::next_window(W))
+      if (W->modal()) break;
+    Fl::modal_ = W;
   }
 
   // Make sure no events are sent to this window:
@@ -590,22 +762,39 @@ void Fl_Window::hide() {
   handle(FL_HIDE);
 
 #ifdef WIN32
-  if (x->private_dc) ReleaseDC(x->xid,x->private_dc);
-  if (x->xid == fl_window && fl_gc) {
+  if (ip->private_dc) ReleaseDC(ip->xid,ip->private_dc);
+  if (ip->xid == fl_window && fl_gc) {
     ReleaseDC(fl_window, fl_gc);
     fl_window = (HWND)-1;
     fl_gc = 0;
   }
+#elif defined(__APPLE__)
+  if ( ip->xid == fl_window )
+    fl_window = 0;
 #else
-  if (x->region) XDestroyRegion(x->region);
+  if (ip->region) XDestroyRegion(ip->region);
 #endif
-  XDestroyWindow(fl_display, x->xid);
+
+#ifdef __APPLE__
+  if ( !parent() ) // don't destroy shared windows!
+  {
+    //+ RemoveTrackingHandler( dndTrackingHandler, ip->xid );
+    //+ RemoveReceiveHandler( dndReceiveHandler, ip->xid );
+    XDestroyWindow(fl_display, ip->xid);
+  }
+#else
+# if USE_XFT
+  fl_destroy_xft_draw(ip->xid);
+# endif
+  XDestroyWindow(fl_display, ip->xid);
+#endif
+  
 #ifdef WIN32
   // Try to stop the annoying "raise another program" behavior
   if (non_modal() && Fl::first_window() && Fl::first_window()->shown())
     Fl::first_window()->show();
 #endif
-  delete x;
+  delete ip;
 }
 
 Fl_Window::~Fl_Window() {
@@ -620,108 +809,160 @@ Fl_Window::~Fl_Window() {
 // Fl_Window::show() or Fl_Window::hide() is called, or in response to
 // iconize/deiconize events from the system.
 
-int Fl_Window::handle(int event) {
-  if (parent()) switch (event) {
-  case FL_SHOW:
-    if (!shown()) show();
-    else XMapWindow(fl_display, fl_xid(this)); // extra map calls are harmless
-    break;
-  case FL_HIDE:
-    if (shown()) {
-      // Find what really turned invisible, if is was a parent window
-      // we do nothing.  We need to avoid unnecessary unmap calls
-      // because they cause the display to blink when the parent is
-      // remapped.  However if this or any intermediate non-window
-      // widget has really had hide() called directly on it, we must
-      // unmap because when the parent window is remapped we don't
-      // want to reappear.
-      if (visible()) {
-       Fl_Widget* p = parent(); for (;p->visible();p = p->parent()) {}
-       if (p->type() >= FL_WINDOW) break; // don't do the unmap
+int Fl_Window::handle(int ev)
+{
+  if (parent()) {
+    switch (ev) {
+    case FL_SHOW:
+      if (!shown()) show();
+      else XMapWindow(fl_display, fl_xid(this)); // extra map calls are harmless
+      break;
+    case FL_HIDE:
+      if (shown()) {
+	// Find what really turned invisible, if is was a parent window
+	// we do nothing.  We need to avoid unnecessary unmap calls
+	// because they cause the display to blink when the parent is
+	// remapped.  However if this or any intermediate non-window
+	// widget has really had hide() called directly on it, we must
+	// unmap because when the parent window is remapped we don't
+	// want to reappear.
+	if (visible()) {
+	 Fl_Widget* p = parent(); for (;p->visible();p = p->parent()) {}
+	 if (p->type() >= FL_WINDOW) break; // don't do the unmap
+	}
+	XUnmapWindow(fl_display, fl_xid(this));
       }
-      XUnmapWindow(fl_display, fl_xid(this));
+      break;
     }
-    break;
+//  } else if (ev == FL_FOCUS || ev == FL_UNFOCUS) {
+//    Fl_Tooltip::exit(Fl_Tooltip::current());
   }
-  return Fl_Group::handle(event);
+
+  return Fl_Group::handle(ev);
 }
 
 ////////////////////////////////////////////////////////////////
-// ~Fl_Widget() calls this: this function must get rid of any
-// global pointers to the widget.  This is also called by hide()
-// and deactivate().
+// Back compatability cut & paste functions for fltk 1.1 only:
 
-// call this to free a selection (or change the owner):
-void Fl::selection_owner(Fl_Widget *owner) {
-  if (selection_owner_ && owner != selection_owner_)
-    selection_owner_->handle(FL_SELECTIONCLEAR);
-  if (focus_ && owner != focus_ && focus_ != selection_owner_)
-    focus_->handle(FL_SELECTIONCLEAR); // clear non-X-selection highlight
-  selection_owner_ = owner;
+void Fl::selection_owner(Fl_Widget *owner) {selection_owner_ = owner;}
+
+void Fl::selection(Fl_Widget &owner, const char* text, int len) {
+  selection_owner_ = &owner;
+  Fl::copy(text, len, 0);
 }
 
+void Fl::paste(Fl_Widget &receiver) {
+  Fl::paste(receiver, 0);
+}
+
+////////////////////////////////////////////////////////////////
+
 #include <FL/fl_draw.H>
 
-void Fl_Widget::redraw() {damage(FL_DAMAGE_ALL);}
+void Fl_Widget::redraw() {
+  damage(FL_DAMAGE_ALL);
+}
 
-void Fl_Widget::damage(uchar flags) {
+void Fl_Widget::redraw_label() {
+  if (window()) {
+    if (box() == FL_NO_BOX) {
+      // Widgets with the FL_NO_BOX boxtype need a parent to
+      // redraw, since it is responsible for redrawing the
+      // background...
+      int X = x() > 0 ? x() - 1 : 0;
+      int Y = y() > 0 ? y() - 1 : 0;
+      window()->damage(FL_DAMAGE_ALL, X, Y, w() + 2, h() + 2);
+    }
+
+    if (align() && !(align() & FL_ALIGN_INSIDE) && window()->shown()) {
+      // If the label is not inside the widget, compute the location of
+      // the label and redraw the window within that bounding box...
+      int W = 0, H = 0;
+      label_.measure(W, H);
+      W += 5; // Add a little to the size of the label to cover overflow
+      H += 5;
+
+      if (align() & FL_ALIGN_BOTTOM) {
+	window()->damage(FL_DAMAGE_EXPOSE, x(), y() + h(), w(), H);
+      } else if (align() & FL_ALIGN_TOP) {
+	window()->damage(FL_DAMAGE_EXPOSE, x(), y() - H, w(), H);
+      } else if (align() & FL_ALIGN_LEFT) {
+	window()->damage(FL_DAMAGE_EXPOSE, x() - W, y(), W, h());
+      } else if (align() & FL_ALIGN_RIGHT) {
+	window()->damage(FL_DAMAGE_EXPOSE, x() + w(), y(), W, h());
+      } else {
+        window()->damage(FL_DAMAGE_ALL);
+      }
+    } else {
+      // The label is inside the widget, so just redraw the widget itself...
+      damage(FL_DAMAGE_ALL);
+    }
+  }
+}
+
+void Fl_Widget::damage(uchar fl) {
   if (type() < FL_WINDOW) {
     // damage only the rectangle covered by a child widget:
-    damage(flags, x(), y(), w(), h());
+    damage(fl, x(), y(), w(), h());
   } else {
     // damage entire window by deleting the region:
     Fl_X* i = Fl_X::i((Fl_Window*)this);
     if (!i) return; // window not mapped, so ignore it
     if (i->region) {XDestroyRegion(i->region); i->region = 0;}
-    damage_ |= flags;
+    damage_ |= fl;
     Fl::damage(FL_DAMAGE_CHILD);
   }
 }
 
-void Fl_Widget::damage(uchar flags, int X, int Y, int W, int H) {
-  Fl_Widget* window = this;
+void Fl_Widget::damage(uchar fl, int X, int Y, int W, int H) {
+  Fl_Widget* wi = this;
   // mark all parent widgets between this and window with FL_DAMAGE_CHILD:
-  while (window->type() < FL_WINDOW) {
-    window->damage_ |= flags;
-    window = window->parent();
-    if (!window) return;
-    flags = FL_DAMAGE_CHILD;
+  while (wi->type() < FL_WINDOW) {
+    wi->damage_ |= fl;
+    wi = wi->parent();
+    if (!wi) return;
+    fl = FL_DAMAGE_CHILD;
   }
-  Fl_X* i = Fl_X::i((Fl_Window*)window);
+  Fl_X* i = Fl_X::i((Fl_Window*)wi);
   if (!i) return; // window not mapped, so ignore it
 
-  if (X<=0 && Y<=0 && W>=window->w() && H>=window->h()) {
+  if (X<=0 && Y<=0 && W>=wi->w() && H>=wi->h()) {
     // if damage covers entire window delete region:
-    window->damage(flags);
+    wi->damage(fl);
     return;
   }
 
   // clip the damage to the window and quit if none:
   if (X < 0) {W += X; X = 0;}
   if (Y < 0) {H += Y; Y = 0;}
-  if (W > window->w()-X) W = window->w()-X;
-  if (H > window->h()-Y) H = window->h()-Y;
+  if (W > wi->w()-X) W = wi->w()-X;
+  if (H > wi->h()-Y) H = wi->h()-Y;
   if (W <= 0 || H <= 0) return;
 
-  if (window->damage()) {
+  if (wi->damage()) {
     // if we already have damage we must merge with existing region:
     if (i->region) {
-#ifndef WIN32
+#ifdef WIN32
+      Fl_Region R = XRectangleRegion(X, Y, W, H);
+      CombineRgn(i->region, i->region, R, RGN_OR);
+      XDestroyRegion(R);
+#elif defined(__APPLE__)
+      Fl_Region R = NewRgn(); 
+      SetRectRgn(R, X, Y, X+W, Y+H);
+      UnionRgn(R, i->region, i->region);
+      DisposeRgn(R);
+#else
       XRectangle R;
       R.x = X; R.y = Y; R.width = W; R.height = H;
       XUnionRectWithRegion(&R, i->region, i->region);
-#else
-      Region R = XRectangleRegion(X, Y, W, H);
-      CombineRgn(i->region, i->region, R, RGN_OR);
-      XDestroyRegion(R);
 #endif
     }
-    window->damage_ |= flags;
+    wi->damage_ |= fl;
   } else {
     // create a new region:
     if (i->region) XDestroyRegion(i->region);
     i->region = XRectangleRegion(X,Y,W,H);
-    window->damage_ = flags;
+    wi->damage_ = fl;
   }
   Fl::damage(FL_DAMAGE_CHILD);
 }
@@ -734,5 +975,5 @@ void Fl_Window::flush() {
 }
 
 //
-// End of "$Id: Fl.cxx,v 1.24.2.34 2000/11/20 19:02:20 easysw Exp $".
+// End of "$Id: Fl.cxx,v 1.24.2.41.2.59 2003/01/30 21:41:14 easysw Exp $".
 //
