@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Choice.cxx,v 1.10.2.4 2000/06/05 21:20:49 mike Exp $"
+// "$Id: Fl_Choice.cxx,v 1.10.2.5.2.13 2003/01/30 21:41:34 easysw Exp $"
 //
 // Choice widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -34,32 +34,58 @@
 extern char fl_draw_shortcut;
 
 void Fl_Choice::draw() {
-  draw_box();
-  if (box() == FL_FLAT_BOX) return; // for XForms compatability
-  int H = labelsize()/2+1;
-  draw_box(FL_THIN_UP_BOX,x()+w()-3*H,y()+(h()-H)/2,2*H,H,color());
+  int dx = Fl::box_dx(FL_DOWN_BOX);
+  int dy = Fl::box_dy(FL_DOWN_BOX);
+  int H = h() - 2 * dy;
+  int W = (H > 20) ? 20 : H;
+  int X = x() + w() - W - dx;
+  int Y = y() + dy;
+  int w1 = (W - 4) / 3; if (w1 < 1) w1 = 1;
+  int x1 = X + (W - 2 * w1 - 1) / 2;
+  int y1 = Y + (H - w1 - 1) / 2;
+
+  if (Fl::scheme()) {
+    draw_box(FL_UP_BOX, color());
+
+    fl_color(active_r() ? labelcolor() : fl_inactive(labelcolor()));
+    fl_polygon(x1, y1 + 3, x1 + w1, y1 + w1 + 3, x1 + 2 * w1, y1 + 3);
+    fl_polygon(x1, y1 + 1, x1 + w1, y1 - w1 + 1, x1 + 2 * w1, y1 + 1);
+  } else {
+    draw_box(FL_DOWN_BOX, color());
+    draw_box(FL_UP_BOX,X,Y,W,H,FL_GRAY);
+
+    fl_color(active_r() ? labelcolor() : fl_inactive(labelcolor()));
+    fl_polygon(x1, y1, x1 + w1, y1 + w1, x1 + 2 * w1, y1);
+  }
+
+  W += 2 * dx;
+
   if (mvalue()) {
     Fl_Menu_Item m = *mvalue();
     if (active_r()) m.activate(); else m.deactivate();
-    int BW = Fl::box_dx(box());
-    fl_clip(x(), y(), w()-3*H, h());
+    fl_clip(x() + dx, y() + dy + 1, w() - W, H - 2);
     fl_draw_shortcut = 2; // hack value to make '&' disappear
-    m.draw(x()+BW, y(), w()-2*BW-3*H, h(), this);
+    m.draw(x() + dx, y() + dy + 1, w() - W, H - 2, this,
+           Fl::focus() == this);
     fl_draw_shortcut = 0;
     fl_pop_clip();
   }
+
   draw_label();
 }
 
-Fl_Choice::Fl_Choice(int x,int y,int w,int h, const char *l)
-: Fl_Menu_(x,y,w,h,l) {
+Fl_Choice::Fl_Choice(int X, int Y, int W, int H, const char *l)
+: Fl_Menu_(X,Y,W,H,l) {
   align(FL_ALIGN_LEFT);
   when(FL_WHEN_RELEASE);
   textfont(FL_HELVETICA);
-  down_box(FL_NO_BOX);
+  box(FL_FLAT_BOX);
+  down_box(FL_BORDER_BOX);
+  color(FL_BACKGROUND2_COLOR);
 }
 
 int Fl_Choice::value(int v) {
+  if (v < 0 || v >= (size() - 1)) return 0;
   if (!Fl_Menu_::value(v)) return 0;
   redraw();
   return 1;
@@ -69,7 +95,15 @@ int Fl_Choice::handle(int e) {
   if (!menu() || !menu()->text) return 0;
   const Fl_Menu_Item* v;
   switch (e) {
+  case FL_ENTER:
+  case FL_LEAVE:
+    return 1;
+
+  case FL_KEYBOARD:
+    if (Fl::event_key() != ' ' ||
+        (Fl::event_state() & (FL_SHIFT | FL_CTRL | FL_ALT | FL_META))) return 0;
   case FL_PUSH:
+    if (Fl::visible_focus()) Fl::focus(this);
     Fl::event_is_click(0);
   J1:
     v = menu()->pulldown(x(), y(), w(), h(), mvalue(), this);
@@ -84,11 +118,17 @@ int Fl_Choice::handle(int e) {
     if (v != mvalue()) redraw();
     picked(v);
     return 1;
+  case FL_FOCUS:
+  case FL_UNFOCUS:
+    if (Fl::visible_focus()) {
+      redraw();
+      return 1;
+    } else return 0;
   default:
     return 0;
   }
 }
 
 //
-// End of "$Id: Fl_Choice.cxx,v 1.10.2.4 2000/06/05 21:20:49 mike Exp $".
+// End of "$Id: Fl_Choice.cxx,v 1.10.2.5.2.13 2003/01/30 21:41:34 easysw Exp $".
 //
