@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Double_Window.cxx,v 1.12.2.3 2000/11/20 19:02:20 easysw Exp $"
+// "$Id: Fl_Double_Window.cxx,v 1.12.2.4.2.7 2003/01/30 21:41:40 easysw Exp $"
 //
 // Double-buffered window code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -59,7 +59,7 @@ static int can_xdbe() {
 #endif
 
 void Fl_Double_Window::show() {
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__APPLE__)
   if (!shown()) { // don't set the background pixel
     fl_open_display();
     Fl_X::make_xid(this);
@@ -94,6 +94,90 @@ void fl_copy_offscreen(int x,int y,int w,int h,HBITMAP bitmap,int srcx,int srcy)
 
 extern void fl_restore_clip();
 
+#elif defined(__APPLE__)
+
+/**
+ * Mac:
+ */
+GWorldPtr fl_create_offscreen(int w, int h) {
+  GWorldPtr gw;
+  Rect bounds;
+  bounds.left=0; bounds.right=w; bounds.top=0; bounds.bottom=h;
+  QDErr err = NewGWorld(&gw, 0, &bounds, 0L, 0L, 0); // 'useTempMem' should not be used (says the Carbon port manual)
+  if ( err == -108 )
+    { }
+//    fl_message( "The application memory is low. Please increase the initial memory assignment.\n" ); 
+  if (err!=noErr || gw==0L) return 0L;
+  return gw;
+}
+
+/**
+ * Mac:
+ */
+void fl_copy_offscreen(int x,int y,int w,int h,GWorldPtr gWorld,int srcx,int srcy) {
+  Rect src;
+  if ( !gWorld ) return;
+  src.top = srcy; src.left = srcx; src.bottom = srcy+h; src.right = srcx+w;
+  Rect dst;
+  GrafPtr dstPort; GetPort(&dstPort);
+  dst.top = y; dst.left = x; dst.bottom = y+h; dst.right = x+w;
+  RGBColor rgb;
+  rgb.red = 0xffff; rgb.green = 0xffff; rgb.blue = 0xffff;
+  RGBBackColor( &rgb );
+  rgb.red = 0x0000; rgb.green = 0x0000; rgb.blue = 0x0000;
+  RGBForeColor( &rgb );
+  CopyBits(GetPortBitMapForCopyBits(gWorld), GetPortBitMapForCopyBits(dstPort), &src, &dst, srcCopy, 0L);
+}
+
+/**
+ * Mac:
+ */
+void fl_delete_offscreen(GWorldPtr gWorld) {
+  DisposeGWorld(gWorld);
+}
+
+static GrafPtr prevPort;
+static GDHandle prevGD;
+
+/**
+ * Mac:
+ */
+void fl_begin_offscreen(GWorldPtr gWorld) {
+  GetGWorld( &prevPort, &prevGD );
+  if ( gWorld )
+  {
+    SetGWorld( gWorld, 0 ); // sets the correct port
+    PixMapHandle pm = GetGWorldPixMap(gWorld);
+    Boolean ret = LockPixels(pm);
+    if ( ret == false )
+    {
+      Rect rect;
+      GetPortBounds( gWorld, &rect );
+      UpdateGWorld( &gWorld, 0, &rect, 0, 0, 0 );
+      pm = GetGWorldPixMap( gWorld );
+      LockPixels( pm );
+    }
+    fl_window = 0;
+  }
+  fl_push_no_clip();
+}
+
+/**
+ * Mac:
+ */
+void fl_end_offscreen() {
+  GWorldPtr currPort;
+  GDHandle currGD;
+  GetGWorld( &currPort, &currGD );
+  fl_pop_clip();
+  PixMapHandle pm = GetGWorldPixMap(currPort);
+  UnlockPixels(pm);
+  SetGWorld( prevPort, prevGD );
+  fl_window = GetWindowFromPort( prevPort );
+}
+
+extern void fl_restore_clip();
+
 #endif
 
 // Fl_Overlay_Window relies on flush(1) copying the back buffer to the
@@ -103,6 +187,15 @@ extern void fl_restore_clip();
 void Fl_Double_Window::flush() {flush(0);}
 
 void Fl_Double_Window::flush(int eraseoverlay) {
+#ifdef WIN32
+  // Windows seems to have trouble doing the double-buffer thing in sub-
+  // windows, so only create the backbuffer stuff for top-level windows.
+  if (parent()) {
+    Fl_Window::flush();
+    return;
+  }
+#endif // WIN32
+
   make_current(); // make sure fl_gc is non-zero
   Fl_X *myi = Fl_X::i(this);
   if (!myi->other_xid) {
@@ -111,7 +204,17 @@ void Fl_Double_Window::flush(int eraseoverlay) {
       XdbeAllocateBackBufferName(fl_display, fl_xid(this), XdbeUndefined);
     else
 #endif
+#ifdef __APPLE__
+    // the Apple OS X window manager double buffers ALL windows
+    // anyway, so there is no need to waste memory and time.
+    //
+    // BTW: Windows2000 and later also forces doublebuffering if
+    // transparent windows are beeing used (alpha channel)
+    if ( ( !QDIsPortBuffered( GetWindowPort(myi->xid) ) ) || force_doublebuffering_ )
       myi->other_xid = fl_create_offscreen(w(), h());
+#else
+    myi->other_xid = fl_create_offscreen(w(), h());
+#endif
     clear_damage(FL_DAMAGE_ALL);
   }
 #if USE_XDBE
@@ -149,6 +252,15 @@ void Fl_Double_Window::flush(int eraseoverlay) {
     draw();
     DeleteDC(fl_gc);
     fl_gc = _sgc;
+#elif defined(__APPLE__)
+    if ( myi->other_xid ) {
+      fl_begin_offscreen( myi->other_xid );
+      fl_clip_region( 0 );   
+      draw();
+      fl_end_offscreen();
+    } else {
+      draw();
+    }
 #else // X:
     fl_window = myi->other_xid;
     draw();
@@ -159,7 +271,11 @@ void Fl_Double_Window::flush(int eraseoverlay) {
   // on Irix (at least) it is faster to reduce the area copied to
   // the current clip region:
   int X,Y,W,H; fl_clip_box(0,0,w(),h(),X,Y,W,H);
+#ifdef __APPLE__
+  if (myi->other_xid) fl_copy_offscreen(X, Y, W, H, myi->other_xid, X, Y);
+#else
   fl_copy_offscreen(X, Y, W, H, myi->other_xid, X, Y);
+#endif
 }
 
 void Fl_Double_Window::resize(int X,int Y,int W,int H) {
@@ -192,5 +308,5 @@ Fl_Double_Window::~Fl_Double_Window() {
 }
 
 //
-// End of "$Id: Fl_Double_Window.cxx,v 1.12.2.3 2000/11/20 19:02:20 easysw Exp $".
+// End of "$Id: Fl_Double_Window.cxx,v 1.12.2.4.2.7 2003/01/30 21:41:40 easysw Exp $".
 //
