@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Gl_Window.cxx,v 1.12.2.18 2000/12/06 15:45:13 easysw Exp $"
+// "$Id: Fl_Gl_Window.cxx,v 1.12.2.22.2.17 2003/01/30 21:41:49 easysw Exp $"
 //
 // OpenGL window code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -23,15 +23,14 @@
 // Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
-#include <config.h>
+#include "flstring.h"
 #if HAVE_GL
 
 #include <FL/Fl.H>
 #include <FL/x.H>
-#include <FL/Fl_Gl_Window.H>
 #include "Fl_Gl_Choice.H"
+#include <FL/Fl_Gl_Window.H>
 #include <stdlib.h>
-#include <string.h>
 
 ////////////////////////////////////////////////////////////////
 
@@ -54,7 +53,7 @@
 #define COPY 3		// unchanged
 #define NODAMAGE 4	// unchanged even by X expose() events
 
-static char SWAP_TYPE; // 0 = determine it from environment variable
+static char SWAP_TYPE = 0 ; // 0 = determine it from environment variable
 
 ////////////////////////////////////////////////////////////////
 
@@ -63,66 +62,85 @@ int Fl_Gl_Window::can_do(int a, const int *b) {
 }
 
 void Fl_Gl_Window::show() {
-#if !defined(_WIN32) && !defined(WIN32)
   if (!shown()) {
     if (!g) {
       g = Fl_Gl_Choice::find(mode_,alist);
-      if (!g) {Fl::error("Insufficient GL support"); return;}
+
+      if (!g && (mode_ & FL_DOUBLE) == FL_SINGLE) {
+        g = Fl_Gl_Choice::find(mode_ | FL_DOUBLE,alist);
+	if (g) mode_ |= FL_FAKE_SINGLE;
+      }
+
+      if (!g) {
+        Fl::error("Insufficient GL support");
+	return;
+      }
     }
+#if !defined(WIN32) && !defined(__APPLE__)
     Fl_X::make_xid(this, g->vis, g->colormap);
     if (overlay && overlay != this) ((Fl_Gl_Window*)overlay)->show();
-  }
 #endif
+  }
   Fl_Window::show();
 }
 
 void Fl_Gl_Window::invalidate() {
   valid(0);
-#if !defined(_WIN32) && !defined(WIN32)
+#ifndef WIN32
   if (overlay) ((Fl_Gl_Window*)overlay)->valid(0);
 #endif
 }
 
 int Fl_Gl_Window::mode(int m, const int *a) {
   if (m == mode_ && a == alist) return 0;
-  mode_ = m; alist = a;
-#if defined(_WIN32) || defined(WIN32)
-  // destroy context and g:
-  if (shown()) {hide(); show();}
-#else
-  // under X, if the visual changes we must make a new X window (!):
-  if (shown()) {
-    Fl_Gl_Choice *g1 = g;
-    g = Fl_Gl_Choice::find(mode_,alist);
-    if (!g || g->vis->visualid != g1->vis->visualid || g->d != g1->d) {
-      hide(); show();
-    }
-  }
+#ifndef __APPLE__
+  int oldmode = mode_;
+  Fl_Gl_Choice* oldg = g;
 #endif
+  context(0);
+  mode_ = m; alist = a;
+  if (shown()) {
+    g = Fl_Gl_Choice::find(m, a);
+#if defined(WIN32)
+    if (!g || (oldmode^m)&FL_DOUBLE) {
+      hide();
+      show();
+    }
+#elif defined(__APPLE__)
+    redraw();
+#else
+    // under X, if the visual changes we must make a new X window (yuck!):
+    if (!g || g->vis->visualid!=oldg->vis->visualid || (oldmode^m)&FL_DOUBLE) {
+      hide();
+      show();
+    }
+#endif
+  } else {
+    g = 0;
+  }
   return 1;
 }
 
+#define NON_LOCAL_CONTEXT 0x80000000
+
 void Fl_Gl_Window::make_current() {
-  if (!context) {
-#if defined(_WIN32) || defined(WIN32)
-    context = wglCreateContext(fl_private_dc(this, mode_,&g));
-    if (fl_first_context) wglShareLists(fl_first_context, (GLXContext)context);
-    else fl_first_context = (GLXContext)context;
-#else
-    context = glXCreateContext(fl_display, g->vis, fl_first_context, 1);
-    if (!fl_first_context) fl_first_context = (GLXContext)context;
-#endif
+  if (!context_) {
+    mode_ &= ~NON_LOCAL_CONTEXT;
+    context_ = fl_create_gl_context(this, g);
     valid(0);
   }
-  fl_set_gl_context(this, (GLXContext)context);
-#if (defined(_WIN32) || defined(WIN32)) && USE_COLORMAP
+  fl_set_gl_context(this, context_);
+#if defined(WIN32) && USE_COLORMAP
   if (fl_palette) {
     fl_GetDC(fl_xid(this));
     SelectPalette(fl_gc, fl_palette, FALSE);
     RealizePalette(fl_gc);
   }
 #endif // USE_COLORMAP
-  glDrawBuffer(GL_BACK);
+  if (mode_ & FL_FAKE_SINGLE) {
+    glDrawBuffer(GL_FRONT);
+    glReadBuffer(GL_FRONT);
+  }
   current_ = this;
 }
 
@@ -142,37 +160,52 @@ void Fl_Gl_Window::ortho() {
 }
 
 void Fl_Gl_Window::swap_buffers() {
-#if defined(_WIN32) || defined(WIN32)
-#if HAVE_GL_OVERLAY
+#ifdef WIN32
+#  if HAVE_GL_OVERLAY
   // Do not swap the overlay, to match GLX:
-  wglSwapLayerBuffers(Fl_X::i(this)->private_dc, WGL_SWAP_MAIN_PLANE);
-#else
+  BOOL ret = wglSwapLayerBuffers(Fl_X::i(this)->private_dc, WGL_SWAP_MAIN_PLANE);
+  DWORD err = GetLastError();;
+#  else
   SwapBuffers(Fl_X::i(this)->private_dc);
-#endif
+#  endif
+#elif defined(__APPLE__)
+  aglSwapBuffers((AGLContext)context_);
 #else
   glXSwapBuffers(fl_display, fl_xid(this));
 #endif
 }
 
-#if HAVE_GL_OVERLAY && defined(_WIN32)
+#if HAVE_GL_OVERLAY && defined(WIN32)
 uchar fl_overlay; // changes how fl_color() works
 int fl_overlay_depth = 0;
 #endif
 
 void Fl_Gl_Window::flush() {
   uchar save_valid = valid_;
-#if defined(_WIN32) || defined(WIN32)
-  // SGI 320 messes up overlay with user-defined cursors:
-  bool fixcursor =
-    Fl_X::i(this)->cursor && Fl_X::i(this)->cursor != fl_default_cursor;
-  if (fixcursor) SetCursor(0);
+
+#ifdef __APPLE__
+  //: clear previous clipping in this shared port
+  GrafPtr port = GetWindowPort( fl_xid(this) );
+  Rect rect; SetRect( &rect, 0, 0, 0x7fff, 0x7fff );
+  GrafPtr old; GetPort( &old );
+  SetPort( port );
+  ClipRect( &rect );
+  SetPort( old );
 #endif
 
-#if HAVE_GL_OVERLAY && (defined(_WIN32) || defined(WIN32))
-  // Draw into hardware overlay planes:
+#if HAVE_GL_OVERLAY && defined(WIN32)
+
+  bool fixcursor = false; // for fixing the SGI 320 bug
+
+  // Draw into hardware overlay planes if they are damaged:
   if (overlay && overlay != this
       && (damage()&(FL_DAMAGE_OVERLAY|FL_DAMAGE_EXPOSE) || !save_valid)) {
-    fl_set_gl_context(this, (GLXContext)overlay);
+    // SGI 320 messes up overlay with user-defined cursors:
+    if (Fl_X::i(this)->cursor && Fl_X::i(this)->cursor != fl_default_cursor) {
+      fixcursor = true; // make it restore cursor later
+      SetCursor(0);
+    }
+    fl_set_gl_context(this, (GLContext)overlay);
     if (fl_overlay_depth)
       wglRealizeLayerPalette(Fl_X::i(this)->private_dc, 1, TRUE);
     glDisable(GL_SCISSOR_TEST);
@@ -182,7 +215,8 @@ void Fl_Gl_Window::flush() {
     fl_overlay = 0;
     valid(save_valid);
     wglSwapLayerBuffers(Fl_X::i(this)->private_dc, WGL_SWAP_OVERLAY1);
-    if (damage() == FL_DAMAGE_OVERLAY) { // main layer is undamaged
+    // if only the overlay was damaged we are done, leave main layer alone:
+    if (damage() == FL_DAMAGE_OVERLAY) {
       if (fixcursor) SetCursor(Fl_X::i(this)->cursor);
       return;
     }
@@ -191,14 +225,21 @@ void Fl_Gl_Window::flush() {
 
   make_current();
 
-  if (g->d) {
+  if (mode_ & FL_DOUBLE) {
+
+    glDrawBuffer(GL_BACK);
 
     if (!SWAP_TYPE) {
+#ifdef __APPLE__
+      SWAP_TYPE = COPY;
+#else
       SWAP_TYPE = UNDEFINED;
+#endif
       const char* c = getenv("GL_SWAP_TYPE");
       if (c) {
 	if (!strcmp(c,"COPY")) SWAP_TYPE = COPY;
 	else if (!strcmp(c, "NODAMAGE")) SWAP_TYPE = NODAMAGE;
+	else if (!strcmp(c, "SWAP")) SWAP_TYPE = SWAP;
       }
     }
 
@@ -224,18 +265,12 @@ void Fl_Gl_Window::flush() {
 	if (damage1_ || damage() != FL_DAMAGE_OVERLAY || !save_valid) draw();
 	// we use a seperate context for the copy because rasterpos must be 0
 	// and depth test needs to be off:
-	static GLXContext ortho_context = 0;
+	static GLContext ortho_context = 0;
 	static Fl_Gl_Window* ortho_window = 0;
-	int init = !ortho_context;
-	if (init) {
-#if defined(_WIN32) || defined(WIN32)
-	  ortho_context = wglCreateContext(Fl_X::i(this)->private_dc);
-#else
-	  ortho_context =glXCreateContext(fl_display,g->vis,fl_first_context,1);
-#endif
-	}
+	int orthoinit = !ortho_context;
+	if (orthoinit) ortho_context = fl_create_gl_context(this, g);
 	fl_set_gl_context(this, ortho_context);
-	if (init || !save_valid || ortho_window != this) {
+	if (orthoinit || !save_valid || ortho_window != this) {
 	  glDisable(GL_DEPTH_TEST);
 	  glReadBuffer(GL_BACK);
 	  glDrawBuffer(GL_FRONT);
@@ -250,11 +285,9 @@ void Fl_Gl_Window::flush() {
 	damage1_ = 0;
 
       } else {
-
 	damage1_ = damage();
 	clear_damage(~0); draw();
 	swap_buffers();
-
       }
 
     }
@@ -274,7 +307,7 @@ void Fl_Gl_Window::flush() {
 
   }
 
-#if defined(_WIN32) || defined(WIN32)
+#if HAVE_GL_OVERLAY && defined(WIN32)
   if (fixcursor) SetCursor(Fl_X::i(this)->cursor);
 #endif
   valid(1);
@@ -283,7 +316,15 @@ void Fl_Gl_Window::flush() {
 void Fl_Gl_Window::resize(int X,int Y,int W,int H) {
   if (W != w() || H != h()) {
     valid(0);
-#if !defined(_WIN32) && !defined(WIN32)
+#ifdef __APPLE__
+  GLint xywh[4];
+  xywh[0] = X;
+  xywh[1] = window()->h() - Y - H;
+  xywh[2] = W;
+  xywh[3] = H;
+  aglSetInteger(context_, AGL_BUFFER_RECT, xywh);
+  aglUpdateContext(context_);
+#elif !defined(WIN32)
     if (!resizable() && overlay && overlay != this)
       ((Fl_Gl_Window*)overlay)->resize(0,0,W,H);
 #endif
@@ -291,26 +332,18 @@ void Fl_Gl_Window::resize(int X,int Y,int W,int H) {
   Fl_Window::resize(X,Y,W,H);
 }
 
+void Fl_Gl_Window::context(void* v, int destroy_flag) {
+  if (context_ && !(mode_&NON_LOCAL_CONTEXT)) fl_delete_gl_context(context_);
+  context_ = (GLContext)v;
+  if (destroy_flag) mode_ &= ~NON_LOCAL_CONTEXT;
+  else mode_ |= NON_LOCAL_CONTEXT;
+}    
+
 void Fl_Gl_Window::hide() {
-  if (context) {
-    fl_no_gl_context();
-    if (context != fl_first_context) {
-#if defined(_WIN32) || defined(WIN32)
-      wglDeleteContext((GLXContext)context);
-#else
-      glXDestroyContext(fl_display, (GLXContext)context);
-#endif
-    }
-// This causes incompatibility with some OpenGL libraries
-// I don't think this is not necessary in any case, right?
-//#ifdef GLX_MESA_release_buffers
-//    glXReleaseBuffersMESA(fl_display, fl_xid(this));
-//#endif
-    context = 0;
-  }
-#if HAVE_GL_OVERLAY && (defined(_WIN32) || defined(WIN32))
-  if (overlay && overlay != this && (GLXContext)overlay != fl_first_context) {
-    wglDeleteContext((GLXContext)overlay);
+  context(0);
+#if HAVE_GL_OVERLAY && defined(WIN32)
+  if (overlay && overlay != this) {
+    fl_delete_gl_context((GLContext)overlay);
     overlay = 0;
   }
 #endif
@@ -325,11 +358,18 @@ Fl_Gl_Window::~Fl_Gl_Window() {
 void Fl_Gl_Window::init() {
   end(); // we probably don't want any children
   box(FL_NO_BOX);
-  mode_ = FL_RGB | FL_DEPTH | FL_DOUBLE;
-  alist = 0;
-  context = 0;
-  g = 0;
-  overlay = 0;
+
+  mode_    = FL_RGB | FL_DEPTH | FL_DOUBLE;
+  alist    = 0;
+  context_ = 0;
+  g        = 0;
+  overlay  = 0;
+  valid_   = 0;
+  damage1_ = 0;
+
+  int H = h();
+  h(1); // Make sure we actually do something in resize()...
+  resize(x(), y(), w(), H);
 }
 
 void Fl_Gl_Window::draw_overlay() {}
@@ -337,5 +377,5 @@ void Fl_Gl_Window::draw_overlay() {}
 #endif
 
 //
-// End of "$Id: Fl_Gl_Window.cxx,v 1.12.2.18 2000/12/06 15:45:13 easysw Exp $".
+// End of "$Id: Fl_Gl_Window.cxx,v 1.12.2.22.2.17 2003/01/30 21:41:49 easysw Exp $".
 //
