@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Input.cxx,v 1.10.2.13 2000/10/17 07:23:42 spitzak Exp $"
+// "$Id: Fl_Input.cxx,v 1.10.2.15.2.17 2003/01/30 21:41:57 easysw Exp $"
 //
 // Input widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -24,7 +24,7 @@
 //
 
 // This is the "user interface", it decodes user actions into what to
-// do to the text.  See also Fl_Input_.C, where the text is actually
+// do to the text.  See also Fl_Input_.cxx, where the text is actually
 // manipulated (and some ui, in particular the mouse, is done...).
 // In theory you can replace this code with another subclass to change
 // the keybindings.
@@ -32,14 +32,15 @@
 #include <FL/Fl.H>
 #include <FL/Fl_Input.H>
 #include <FL/fl_draw.H>
-#include <string.h>
+#include <FL/fl_ask.H>
+#include "flstring.h"
 
 void Fl_Input::draw() {
-  if (type() == FL_HIDDEN_INPUT) return;
+  if (input_type() == FL_HIDDEN_INPUT) return;
   Fl_Boxtype b = box();
   if (damage() & FL_DAMAGE_ALL) draw_box(b, color());
-  Fl_Input_::drawtext(x()+Fl::box_dx(b)+3, y()+Fl::box_dy(b),
-		      w()-Fl::box_dw(b)-6, h()-Fl::box_dh(b));
+  Fl_Input_::drawtext(x()+Fl::box_dx(b), y()+Fl::box_dy(b),
+		      w()-Fl::box_dw(b), h()-Fl::box_dh(b));
 }
 
 // kludge so shift causes selection to extend:
@@ -68,21 +69,27 @@ int Fl_Input::handle_key() {
   if (Fl::compose(del)) {
 
     // Insert characters into numeric fields after checking for legality:
-    if (type() == FL_FLOAT_INPUT || type() == FL_INT_INPUT) {
+    if (input_type() == FL_FLOAT_INPUT || input_type() == FL_INT_INPUT) {
       Fl::compose_reset(); // ignore any foreign letters...
+
       // This is complex to allow "0xff12" hex to be typed:
       if (!position() && (ascii == '+' || ascii == '-') ||
 	  (ascii >= '0' && ascii <= '9') ||
 	  (position()==1 && index(0)=='0' && (ascii=='x' || ascii == 'X')) ||
 	  (position()>1 && index(0)=='0' && (index(1)=='x'||index(1)=='X')
 	   && (ascii>='A'&& ascii<='F' || ascii>='a'&& ascii<='f')) ||
-	  type()==FL_FLOAT_INPUT && ascii && strchr(".eE+-", ascii))
-	replace(position(), mark(), &ascii, 1);
+	  input_type()==FL_FLOAT_INPUT && ascii && strchr(".eE+-", ascii)) {
+	if (readonly()) fl_beep();
+	else replace(position(), mark(), &ascii, 1);
+      }
       return 1;
     }
 
-    replace(position(), del ? position()-del : mark(),
-	    Fl::event_text(), Fl::event_length());
+    if (del || Fl::event_length()) {
+      if (readonly()) fl_beep();
+      else replace(position(), del ? position()-del : mark(),
+	           Fl::event_text(), Fl::event_length());
+    }
     return 1;
   }
 
@@ -131,13 +138,21 @@ int Fl_Input::handle_key() {
       position(size(), 0);
       maybe_do_callback();
       return 1;
-    } else if (type() == FL_MULTILINE_INPUT)
+    } else if (input_type() == FL_MULTILINE_INPUT && !readonly())
       return replace(position(), mark(), "\n", 1);
     else 
       return 0;	// reserved for shortcuts
   case FL_Tab:
-    if (Fl::event_state(FL_CTRL|FL_SHIFT) || type()!=FL_MULTILINE_INPUT) return 0;
+    if (Fl::event_state(FL_CTRL|FL_SHIFT) || input_type()!=FL_MULTILINE_INPUT || readonly()) return 0;
     return replace(position(), mark(), &ascii, 1);
+#ifdef __APPLE__
+  case 'c' :
+  case 'v' :
+  case 'x' :
+  case 'z' :
+    if (Fl::event_state(FL_META)) ascii = ctrl(Fl::event_key());
+    break;
+#endif // __APPLE__
   }
 
   int i;
@@ -147,9 +162,13 @@ int Fl_Input::handle_key() {
   case ctrl('B'):
     return shift_position(position()-1) + NORMAL_INPUT_MOVE;
   case ctrl('C'): // copy
-    return copy();
+    return copy(1);
   case ctrl('D'):
   case ctrl('?'):
+    if (readonly()) {
+      fl_beep();
+      return 1;
+    }
     if (mark() != position()) return cut();
     else return cut(1);
   case ctrl('E'):
@@ -157,10 +176,18 @@ int Fl_Input::handle_key() {
   case ctrl('F'):
     return shift_position(position()+1) + NORMAL_INPUT_MOVE;
   case ctrl('H'):
+    if (readonly()) {
+      fl_beep();
+      return 1;
+    }
     if (mark() != position()) cut();
     else cut(-1);
     return 1;
   case ctrl('K'):
+    if (readonly()) {
+      fl_beep();
+      return 1;
+    }
     if (position()>=size()) return 0;
     i = line_end(position());
     if (i == position() && i < size()) i++;
@@ -185,24 +212,44 @@ int Fl_Input::handle_key() {
     shift_up_down_position(line_start(i));
     return 1;
   case ctrl('U'):
+    if (readonly()) {
+      fl_beep();
+      return 1;
+    }
     return cut(0, size());
   case ctrl('V'):
   case ctrl('Y'):
-    Fl::paste(*this);
+    if (readonly()) {
+      fl_beep();
+      return 1;
+    }
+    Fl::paste(*this, 1);
     return 1;
   case ctrl('X'):
   case ctrl('W'):
-    copy();
+    if (readonly()) {
+      fl_beep();
+      return 1;
+    }
+    copy(1);
     return cut();
   case ctrl('Z'):
   case ctrl('_'):
+    if (readonly()) {
+      fl_beep();
+      return 1;
+    }
     return undo();
   case ctrl('I'):
   case ctrl('J'):
   case ctrl('L'):
   case ctrl('M'):
+    if (readonly()) {
+      fl_beep();
+      return 1;
+    }
     // insert a few selected control characters literally:
-    if (type() != FL_FLOAT_INPUT && type() != FL_INT_INPUT)
+    if (input_type() != FL_FLOAT_INPUT && input_type() != FL_INT_INPUT)
       return replace(position(), mark(), &ascii, 1);
   }
 
@@ -210,9 +257,9 @@ int Fl_Input::handle_key() {
 }
 
 int Fl_Input::handle(int event) {
-
+  static int dnd_save_position, dnd_save_mark, drag_start = -1, newpos;
+  static Fl_Widget *dnd_save_focus;
   switch (event) {
-
   case FL_FOCUS:
     switch (Fl::event_key()) {
     case FL_Right:
@@ -238,43 +285,127 @@ int Fl_Input::handle(int event) {
     break;
 
   case FL_KEYBOARD:
-    return handle_key();
+    if (Fl::event_key() == FL_Tab && mark() != position()) {
+      // Set the current cursor position to the end of the selection...
+      if (mark() > position())
+        position(mark());
+      else
+        position(position());
+      return (1);
+    } else return handle_key();
 
   case FL_PUSH:
+    if (Fl::dnd_text_ops()) {
+      int oldpos = position(), oldmark = mark();
+      Fl_Boxtype b = box();
+      Fl_Input_::handle_mouse(
+	x()+Fl::box_dx(b), y()+Fl::box_dy(b),
+	w()-Fl::box_dw(b), h()-Fl::box_dh(b), 0);
+      newpos = position(); 
+      position( oldpos, oldmark );
+      if (Fl::focus()==this && !Fl::event_state(FL_SHIFT) && input_type()!=FL_SECRET_INPUT &&
+	  (newpos >= mark() && newpos < position() ||
+	  newpos >= position() && newpos < mark())) {
+	// user clicked in the selection, may be trying to drag
+	drag_start = newpos;
+	return 1;
+      }
+      drag_start = -1;
+    }
+
     if (Fl::focus() != this) {
       Fl::focus(this);
       handle(FL_FOCUS);
-#if 0 // Misguided attempt to simulate Windoze select-all-on-first-click
-      // that it does for *some* (but not all) text fields:
-      if (type() != FL_MULTILINE_INPUT) {
-        position(size(), 0); // select everything
-        return 1;
+    }
+    break;
+
+  case FL_DRAG:
+    if (Fl::dnd_text_ops()) {
+      if (drag_start >= 0) {
+	if (Fl::event_is_click()) return 1; // debounce the mouse
+	// save the position because sometimes we don't get DND_ENTER:
+	dnd_save_position = position();
+	dnd_save_mark = mark();
+	// drag the data:
+	copy(0); Fl::dnd();
+	return 1;
       }
-#endif
     }
     break;
 
   case FL_RELEASE:
     if (Fl::event_button() == 2) {
       Fl::event_is_click(0); // stop double click from picking a word
-      Fl::paste(*this);
+      Fl::paste(*this, 0);
     } else if (!Fl::event_is_click()) {
       // copy drag-selected text to the clipboard.
-      copy();
+      copy(0);
+    } else if (Fl::event_is_click() && drag_start >= 0) {
+      // user clicked in the field and wants to reset the cursor position...
+      position(drag_start, drag_start);
+      drag_start = -1;
+    } else if (Fl::event_clicks()) {
+      // user double or triple clicked to select word or whole text
+      copy(0);
     }
+    // For output widgets, do the callback so the app knows the user
+    // did something with the mouse...
+    if (readonly()) do_callback();
+    return 1;
+
+  case FL_DND_ENTER:
+    Fl::belowmouse(this); // send the leave events first
+    dnd_save_position = position();
+    dnd_save_mark = mark();
+    dnd_save_focus = Fl::focus();
+    if (dnd_save_focus != this) {
+      Fl::focus(this);
+      handle(FL_FOCUS);
+    }
+    // fall through:
+  case FL_DND_DRAG: 
+    //int p = mouse_position(X, Y, W, H);
+#if DND_OUT_XXXX
+    if (Fl::focus()==this && (p>=dnd_save_position && p<=dnd_save_mark ||
+		      p>=dnd_save_mark && p<=dnd_save_position)) {
+      position(dnd_save_position, dnd_save_mark);
+      return 0;
+    }
+#endif
+    {
+      Fl_Boxtype b = box();
+      Fl_Input_::handle_mouse(
+	x()+Fl::box_dx(b), y()+Fl::box_dy(b),
+	w()-Fl::box_dw(b), h()-Fl::box_dh(b), 0);
+    }
+    return 1;
+
+  case FL_DND_LEAVE:
+    position(dnd_save_position, dnd_save_mark);
+#if DND_OUT_XXXX
+    if (!focused())
+#endif
+    if (dnd_save_focus != this) {
+      Fl::focus(dnd_save_focus);
+      handle(FL_UNFOCUS);
+    }
+    return 1;
+
+  case FL_DND_RELEASE:
+    take_focus();
     return 1;
 
   }
   Fl_Boxtype b = box();
   return Fl_Input_::handletext(event,
-	x()+Fl::box_dx(b)+3, y()+Fl::box_dy(b),
-	w()-Fl::box_dw(b)-6, h()-Fl::box_dh(b));
+	x()+Fl::box_dx(b), y()+Fl::box_dy(b),
+	w()-Fl::box_dw(b), h()-Fl::box_dh(b));
 }
 
-Fl_Input::Fl_Input(int x, int y, int w, int h, const char *l)
-: Fl_Input_(x, y, w, h, l) {
+Fl_Input::Fl_Input(int X, int Y, int W, int H, const char *l)
+: Fl_Input_(X, Y, W, H, l) {
 }
 
 //
-// End of "$Id: Fl_Input.cxx,v 1.10.2.13 2000/10/17 07:23:42 spitzak Exp $".
+// End of "$Id: Fl_Input.cxx,v 1.10.2.15.2.17 2003/01/30 21:41:57 easysw Exp $".
 //
