@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Menu.cxx,v 1.18.2.11 2000/11/20 19:02:20 easysw Exp $"
+// "$Id: Fl_Menu.cxx,v 1.18.2.12.2.23 2003/01/30 21:42:05 easysw Exp $"
 //
 // Menu code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -33,6 +33,7 @@
 #include <FL/Fl_Menu_Window.H>
 #include <FL/Fl_Menu_.H>
 #include <FL/fl_draw.H>
+#include <stdio.h>
 
 int Fl_Menu_Item::size() const {
   const Fl_Menu_Item* m = this;
@@ -92,7 +93,7 @@ public:
   const Fl_Menu_Item* menu;
   menuwindow(const Fl_Menu_Item* m, int X, int Y, int W, int H,
 	     const Fl_Menu_Item* picked, const Fl_Menu_Item* title,
-	     int menubar = 0, int menubar_title = 0);
+	     int menubar = 0, int menubar_title = 0, int right_edge = 0);
   ~menuwindow();
   void set_selected(int);
   int find_selected(int mx, int my);
@@ -108,11 +109,13 @@ extern char fl_draw_shortcut;
 // width of label, including effect of & characters:
 int Fl_Menu_Item::measure(int* hp, const Fl_Menu_* m) const {
   Fl_Label l;
-  l.value = text;
-  l.type = labeltype_;
-  l.font = labelsize_ ? labelfont_ : uchar(m ? m->textfont() : FL_HELVETICA);
-  l.size = labelsize_ ? labelsize_ : m ? m->textsize() : FL_NORMAL_SIZE;
-  l.color = FL_BLACK; // this makes no difference?
+  l.value   = text;
+  l.image   = 0;
+  l.deimage = 0;
+  l.type    = labeltype_;
+  l.font    = labelsize_ ? labelfont_ : uchar(m ? m->textfont() : FL_HELVETICA);
+  l.size    = labelsize_ ? labelsize_ : m ? m->textsize() : (uchar)FL_NORMAL_SIZE;
+  l.color   = FL_BLACK; // this makes no difference?
   fl_draw_shortcut = 1;
   int w = 0; int h = 0; l.measure(w, hp ? *hp : h);
   fl_draw_shortcut = 0;
@@ -123,26 +126,28 @@ int Fl_Menu_Item::measure(int* hp, const Fl_Menu_* m) const {
 void Fl_Menu_Item::draw(int x, int y, int w, int h, const Fl_Menu_* m,
 			int selected) const {
   Fl_Label l;
-  l.value = text;
-  l.type = labeltype_;
-  l.font = labelsize_ ? labelfont_ : uchar(m ? m->textfont() : FL_HELVETICA);
-  l.size = labelsize_ ? labelsize_ : m ? m->textsize() : FL_NORMAL_SIZE;
-  l.color = labelcolor_ ? labelcolor_ : m ? m->textcolor() : int(FL_BLACK);
-  if (!active()) l.color = inactive((Fl_Color)l.color);
+  l.value   = text;
+  l.image   = 0;
+  l.deimage = 0;
+  l.type    = labeltype_;
+  l.font    = labelsize_ ? labelfont_ : uchar(m ? m->textfont() : FL_HELVETICA);
+  l.size    = labelsize_ ? labelsize_ : m ? m->textsize() : (uchar)FL_NORMAL_SIZE;
+  l.color   = labelcolor_ ? labelcolor_ : m ? m->textcolor() : int(FL_BLACK);
+  if (!active()) l.color = fl_inactive((Fl_Color)l.color);
   Fl_Color color = m ? m->color() : FL_GRAY;
   if (selected) {
     Fl_Color r = m ? m->selection_color() : FL_SELECTION_COLOR;
     Fl_Boxtype b = m && m->down_box() ? m->down_box() : FL_FLAT_BOX;
-    if (contrast(r,color)!=r) { // back compatability boxtypes
+    if (fl_contrast(r,color)!=r) { // back compatability boxtypes
       if (selected == 2) { // menu title
 	r = color;
 	b = m ? m->box() : FL_UP_BOX;
       } else {
 	r = (Fl_Color)(FL_COLOR_CUBE-1); // white
-	l.color = contrast((Fl_Color)labelcolor_, r);
+	l.color = fl_contrast((Fl_Color)labelcolor_, r);
       }
     } else {
-      l.color = contrast((Fl_Color)labelcolor_, r);
+      l.color = fl_contrast((Fl_Color)labelcolor_, r);
     }
     if (selected == 2) { // menu title
       fl_draw_box(b, x, y, w, h, r);
@@ -154,26 +159,59 @@ void Fl_Menu_Item::draw(int x, int y, int w, int h, const Fl_Menu_* m,
   }
 
   if (flags & (FL_MENU_TOGGLE|FL_MENU_RADIO)) {
-    int y1 = y+(h-14)/2;
-    fl_color(FL_DARK3);
+    int d = (h - FL_NORMAL_SIZE + 1) / 2;
+    int W = h - 2 * d;
+
     if (flags & FL_MENU_RADIO) {
-      fl_line(x+2, y1+7, x+8, y1+1, x+14, y1+7);
-      if (selected) {
-	fl_color(color); 
-	fl_polygon(x+3, y1+7, x+8, y1+2, x+13, y1+7, x+8, y1+12);
-      }
-      fl_color(FL_LIGHT3); fl_line(x+14, y1+7, x+8, y1+13, x+2, y1+7);
+      fl_draw_box(FL_ROUND_DOWN_BOX, x+2, y+d, W, W, FL_BACKGROUND2_COLOR);
       if (value()) {
-	fl_color(FL_BLACK); 
-	fl_polygon(x+4, y1+7, x+8, y1+3, x+12, y1+7, x+8, y1+11);
+	fl_color(labelcolor_);
+	int tW = (W - Fl::box_dw(FL_ROUND_DOWN_BOX)) / 2 + 1;
+	if ((W - tW) & 1) tW++;	// Make sure difference is even to center
+	int td = Fl::box_dx(FL_ROUND_DOWN_BOX) + 1;
+	switch (tW) {
+	  // Larger circles draw fine...
+	  default :
+            fl_pie(x + td + 2, y + d + td, tW, tW, 0.0, 360.0);
+	    break;
+
+          // Small circles don't draw well on many systems...
+	  case 6 :
+	    fl_rectf(x + td + 4, y + d + td, tW - 4, tW);
+	    fl_rectf(x + td + 3, y + d + td + 1, tW - 2, tW - 2);
+	    fl_rectf(x + td + 2, y + d + td + 2, tW, tW - 4);
+	    break;
+
+	  case 5 :
+	  case 4 :
+	  case 3 :
+	    fl_rectf(x + td + 3, y + d + td, tW - 2, tW);
+	    fl_rectf(x + td + 2, y + d + td + 1, tW, tW - 2);
+	    break;
+
+	  case 2 :
+	  case 1 :
+	    fl_rectf(x + td + 2, y + d + td, tW, tW);
+	    break;
+	}
       }
     } else {
-      fl_yxline(x+3, y1+11, y1+2, x+12);
-      if (selected) {fl_color(color); fl_rectf(x+4, y1+3, 9, 9);}
-      fl_color(FL_LIGHT3); fl_xyline(x+4, y1+12, x+13, y1+3);
-      if (value()) {fl_color(FL_BLACK); fl_rectf(x+5, y1+4, 7, 7);}
+      fl_draw_box(FL_DOWN_BOX, x+2, y+d, W, W, FL_BACKGROUND2_COLOR);
+      if (value()) {
+	fl_color(labelcolor_);
+	int tx = x + 5;
+	int tw = W - 6;
+	int d1 = tw/3;
+	int d2 = tw-d1;
+	int ty = y + d + (W+d2)/2-d1-2;
+	for (int n = 0; n < 3; n++, ty++) {
+	  fl_line(tx, ty, tx+d1, ty+d1);
+	  fl_line(tx+d1, ty+d1, tx+tw-1, ty+d1-d2+1);
+	}
+      }
     }
-    x += 14; w -= 14;
+    x += W + 3;
+    w -= W + 3;
   }
 
   if (!fl_draw_shortcut) fl_draw_shortcut = 1;
@@ -187,14 +225,16 @@ menutitle::menutitle(int X, int Y, int W, int H, const Fl_Menu_Item* L) :
   set_modal();
   clear_border();
   menu = L;
-  if (L->labelcolor_) clear_overlay();
+  if (L->labelcolor_ || Fl::scheme() || L->labeltype_ > FL_NO_LABEL) clear_overlay();
 }
 
 menuwindow::menuwindow(const Fl_Menu_Item* m, int X, int Y, int Wp, int Hp,
 		       const Fl_Menu_Item* picked, const Fl_Menu_Item* t, 
-		       int menubar, int menubar_title)
+		       int menubar, int menubar_title, int right_edge)
   : Fl_Menu_Window(X, Y, Wp, Hp, 0)
 {
+  if (!right_edge) right_edge = Fl::w();
+
   end();
   set_modal();
   clear_border();
@@ -206,7 +246,7 @@ menuwindow::menuwindow(const Fl_Menu_Item* m, int X, int Y, int Wp, int Hp,
   } else {
     box(FL_UP_BOX);
   }
-  color(button ? button->color() : FL_GRAY);
+  color(button && !Fl::scheme() ? button->color() : FL_GRAY);
   selected = -1;
   {int j = 0;
   if (m) for (const Fl_Menu_Item* m1=m; ; m1 = m1->next(), j++) {
@@ -232,15 +272,15 @@ menuwindow::menuwindow(const Fl_Menu_Item* m, int X, int Y, int Wp, int Hp,
   if (t) Wtitle = t->measure(&Htitle, button) + 12;
   int W = 0;
   if (m) for (; m->text; m = m->next()) {
-    int h; int w1 = m->measure(&h, button);
-    if (h+LEADING>itemheight) itemheight = h+LEADING;
+    int hh; int w1 = m->measure(&hh, button);
+    if (hh+LEADING>itemheight) itemheight = hh+LEADING;
     if (m->flags&(FL_SUBMENU|FL_SUBMENU_POINTER)) w1 += 14;
     if (w1 > W) W = w1;
     if (m->shortcut_) {
       w1 = int(fl_width(fl_shortcut_label(m->shortcut_))) + 8;
       if (w1 > hotKeysw) hotKeysw = w1;
     }
-    if (m->labelcolor_) clear_overlay();
+    if (m->labelcolor_ || Fl::scheme() || m->labeltype_ > FL_NO_LABEL) clear_overlay();
   }
   if (selected >= 0 && !Wp) X -= W/2;
   int BW = Fl::box_dx(box());
@@ -248,7 +288,7 @@ menuwindow::menuwindow(const Fl_Menu_Item* m, int X, int Y, int Wp, int Hp,
   if (Wp > W) W = Wp;
   if (Wtitle > W) W = Wtitle;
 
-  if (!Wp) {if (X < 0) X = 0; if (X > Fl::w()-W) X= Fl::w()-W;}
+  if (!Wp) {if (X < 0) X = 0; if (X > Fl::w()-W) X= right_edge-W;}
   x(X); w(W);
   h((numitems ? itemheight*numitems-LEADING : 0)+2*BW+3);
   if (selected >= 0)
@@ -290,42 +330,41 @@ void menuwindow::autoscroll(int n) {
 
 ////////////////////////////////////////////////////////////////
 
-void menuwindow::drawentry(const Fl_Menu_Item* m, int n, int erase) {
+void menuwindow::drawentry(const Fl_Menu_Item* m, int n, int eraseit) {
   if (!m) return; // this happens if -1 is selected item and redrawn
 
   int BW = Fl::box_dx(box());
-  int x = BW;
-  int W = this->w();
-  int w = W-2*BW-1;
-  int y = BW+1+n*itemheight;
-  int h = itemheight - LEADING;
+  int xx = BW;
+  int W = w();
+  int ww = W-2*BW-1;
+  int yy = BW+1+n*itemheight;
+  int hh = itemheight - LEADING;
 
-  if (erase && n != selected) {
-    fl_color(button ? button->color() : FL_GRAY);
-    fl_rectf(x+1, y-(LEADING-2)/2, w-2, h+(LEADING-2));
+  if (eraseit && n != selected) {
+    fl_color(button && !Fl::scheme() ? button->color() : FL_GRAY);
+    fl_rectf(xx+1, yy-(LEADING-2)/2, ww-2, hh+(LEADING-2));
   }
 
-  m->draw(x, y, w, h, button, n==selected);
+  m->draw(xx, yy, ww, hh, button, n==selected);
 
   // the shortcuts and arrows assumme fl_color() was left set by draw():
   if (m->submenu()) {
-    int sz = (h-5)&-2;
-    int y1 = y+(h-sz)/2;
-    int x1 = x+w-sz-3;
+    int sz = (hh-5)&-2;
+    int y1 = yy+(hh-sz)/2;
+    int x1 = xx+ww-sz-3;
     fl_polygon(x1, y1, x1, y1+sz, x1+sz, y1+sz/2);
   } else if (m->shortcut_) {
     Fl_Font f = button ? button->textfont() : FL_HELVETICA;
     fl_font(f, button ? button->textsize() : FL_NORMAL_SIZE);
-    fl_draw(fl_shortcut_label(m->shortcut_), x, y, w-3, h, FL_ALIGN_RIGHT);
+    fl_draw(fl_shortcut_label(m->shortcut_), xx, yy, ww-3, hh, FL_ALIGN_RIGHT);
   }
 
   if (m->flags & FL_MENU_DIVIDER) {
     fl_color(FL_DARK3);
-    fl_xyline(BW-1, y+h+(LEADING-2)/2, W-2*BW+2);
+    fl_xyline(BW-1, yy+hh+(LEADING-2)/2, W-2*BW+2);
     fl_color(FL_LIGHT3);
-    fl_xyline(BW-1, y+h+((LEADING-2)/2+1), W-2*BW+2);
+    fl_xyline(BW-1, yy+hh+((LEADING-2)/2+1), W-2*BW+2);
   }
-
 }
 
 void menutitle::draw() {
@@ -333,7 +372,6 @@ void menutitle::draw() {
 }
 
 void menuwindow::draw() {
-
   if (damage() != FL_DAMAGE_CHILD) {	// complete redraw
     fl_draw_box(box(), 0, 0, w(), h(), color());
     if (menu) {
@@ -361,12 +399,12 @@ int menuwindow::find_selected(int mx, int my) {
   my -= y();
   if (my < 0 || my >= h()) return -1;
   if (!itemheight) { // menubar
-    int x = 3; int n = 0;
+    int xx = 3; int n = 0;
     const Fl_Menu_Item* m = menu;
     for (; ; m = m->next(), n++) {
       if (!m->text) return -1;
-      x += m->measure(0, button) + 16;
-      if (x > mx) break;
+      xx += m->measure(0, button) + 16;
+      if (xx > mx) break;
     }
     return n;
   }
@@ -379,9 +417,9 @@ int menuwindow::find_selected(int mx, int my) {
 // return horizontal position for item n in a menubar:
 int menuwindow::titlex(int n) {
   const Fl_Menu_Item* m;
-  int x = 3;
-  for (m=menu; n--; m = m->next()) x += m->measure(0, button) + 16;
-  return x;
+  int xx = 3;
+  for (m=menu; n--; m = m->next()) xx += m->measure(0, button) + 16;
+  return xx;
 }
 
 ////////////////////////////////////////////////////////////////
@@ -424,16 +462,16 @@ static inline void setitem(const Fl_Menu_Item* i, int m, int n) {
 }
 
 static void setitem(int m, int n) {
-  menustate &p = *(::p);
-  p.current_item = (n >= 0) ? p.p[m]->menu->next(n) : 0;
-  p.menu_number = m;
-  p.item_number = n;
+  menustate &pp = *p;
+  pp.current_item = (n >= 0) ? pp.p[m]->menu->next(n) : 0;
+  pp.menu_number = m;
+  pp.item_number = n;
 }
 
 static int forward(int menu) { // go to next item in menu menu if possible
-  menustate &p = *(::p);
-  menuwindow &m = *(p.p[menu]);
-  int item = (menu == p.menu_number) ? p.item_number : m.selected;
+  menustate &pp = *p;
+  menuwindow &m = *(pp.p[menu]);
+  int item = (menu == pp.menu_number) ? pp.item_number : m.selected;
   while (++item < m.numitems) {
     const Fl_Menu_Item* m1 = m.menu->next(item);
     if (m1->activevisible()) {setitem(m1, menu, item); return 1;}
@@ -442,9 +480,9 @@ static int forward(int menu) { // go to next item in menu menu if possible
 }
 
 static int backward(int menu) { // previous item in menu menu if possible
-  menustate &p = *(::p);
-  menuwindow &m = *(p.p[menu]);
-  int item = (menu == p.menu_number) ? p.item_number : m.selected;
+  menustate &pp = *p;
+  menuwindow &m = *(pp.p[menu]);
+  int item = (menu == pp.menu_number) ? pp.item_number : m.selected;
   if (item < 0) item = m.numitems;
   while (--item >= 0) {
     const Fl_Menu_Item* m1 = m.menu->next(item);
@@ -454,94 +492,97 @@ static int backward(int menu) { // previous item in menu menu if possible
 }
 
 int menuwindow::handle(int e) {
-  menustate &p = *(::p);
+  menustate &pp = *p;
   switch (e) {
   case FL_KEYBOARD:
     switch (Fl::event_key()) {
-    case FL_Tab:
-      if (Fl::event_shift()&FL_SHIFT) goto BACKTAB;
-    case ' ':
-      if (!forward(p.menu_number)) {p.item_number = -1; forward(p.menu_number);}
-      return 1;
     case FL_BackSpace:
     case 0xFE20: // backtab
     BACKTAB:
-      if (!backward(p.menu_number)) {p.item_number = -1;backward(p.menu_number);}
+      if (!backward(pp.menu_number)) {pp.item_number = -1;backward(pp.menu_number);}
       return 1;
     case FL_Up:
-      if (p.menubar && p.menu_number == 0) ;
-      else if (backward(p.menu_number));
-      else if (p.menubar && p.menu_number==1) setitem(0, p.p[0]->selected);
+      if (pp.menubar && pp.menu_number == 0) {
+        // Do nothing...
+      } else if (backward(pp.menu_number)) {
+        // Do nothing...
+      } else if (pp.menubar && pp.menu_number==1) {
+        setitem(0, pp.p[0]->selected);
+      }
       return 1;
+    case FL_Tab:
+      if (Fl::event_shift()) goto BACKTAB;
     case FL_Down:
-      if (p.menu_number || !p.menubar) forward(p.menu_number);
-      else if (p.menu_number < p.nummenus-1) forward(p.menu_number+1);
+      if (pp.menu_number || !pp.menubar) forward(pp.menu_number);
+      else if (pp.menu_number < pp.nummenus-1) forward(pp.menu_number+1);
       return 1;
     case FL_Right:
-      if (p.menubar && (p.menu_number<=0 || p.menu_number==1 && p.nummenus==2))
+      if (pp.menubar && (pp.menu_number<=0 || pp.menu_number==1 && pp.nummenus==2))
 	forward(0);
-      else if (p.menu_number < p.nummenus-1) forward(p.menu_number+1);
+      else if (pp.menu_number < pp.nummenus-1) forward(pp.menu_number+1);
       return 1;
     case FL_Left:
-      if (p.menubar && p.menu_number<=1) backward(0);
-      else if (p.menu_number>0)
-	setitem(p.menu_number-1, p.p[p.menu_number-1]->selected);
+      if (pp.menubar && pp.menu_number<=1) backward(0);
+      else if (pp.menu_number>0)
+	setitem(pp.menu_number-1, pp.p[pp.menu_number-1]->selected);
       return 1;
     case FL_Enter:
-      p.state = DONE_STATE;
+    case ' ':
+      pp.state = DONE_STATE;
       return 1;
     case FL_Escape:
       setitem(0, -1, 0);
-      p.state = DONE_STATE;
+      pp.state = DONE_STATE;
       return 1;
     }
     break;
   case FL_SHORTCUT: {
-    for (int mymenu = p.nummenus; mymenu--;) {
-      menuwindow &mw = *(p.p[mymenu]);
+    for (int mymenu = pp.nummenus; mymenu--;) {
+      menuwindow &mw = *(pp.p[mymenu]);
       int item; const Fl_Menu_Item* m = mw.menu->find_shortcut(&item);
       if (m) {
 	setitem(m, mymenu, item);
-	if (!m->submenu()) p.state = DONE_STATE;
+	if (!m->submenu()) pp.state = DONE_STATE;
 	return 1;
       }
     }} break;
+  case FL_ENTER:
+  case FL_MOVE:
   case FL_PUSH:
-  //case FL_MOVE:
   case FL_DRAG: {
     int mx = Fl::event_x_root();
     int my = Fl::event_y_root();
     int item=0; int mymenu;
-    for (mymenu = p.nummenus-1; ; mymenu--) {
-      item = p.p[mymenu]->find_selected(mx, my);
+    for (mymenu = pp.nummenus-1; ; mymenu--) {
+      item = pp.p[mymenu]->find_selected(mx, my);
       if (item >= 0) break;
       if (mymenu <= 0) break;
     }
     setitem(mymenu, item);
     if (e == FL_PUSH) {
-      if (p.current_item && p.current_item->submenu() // this is a menu title
-	  && item != p.p[mymenu]->selected // and it is not already on
-	  && !p.current_item->callback_) // and it does not have a callback
-	p.state = MENU_PUSH_STATE;
+      if (pp.current_item && pp.current_item->submenu() // this is a menu title
+	  && item != pp.p[mymenu]->selected // and it is not already on
+	  && !pp.current_item->callback_) // and it does not have a callback
+	pp.state = MENU_PUSH_STATE;
       else
-	p.state = PUSH_STATE;
+	pp.state = PUSH_STATE;
     }} return 1;
   case FL_RELEASE:
     // do nothing if they try to pick inactive items
-    if (p.current_item && !p.current_item->activevisible()) return 1;
+    if (pp.current_item && !pp.current_item->activevisible()) return 1;
     // Mouse must either be held down/dragged some, or this must be
     // the second click (not the one that popped up the menu):
-    if (!Fl::event_is_click() || p.state == PUSH_STATE ||
-	p.menubar && p.current_item && !p.current_item->submenu() // button
+    if (!Fl::event_is_click() || pp.state == PUSH_STATE ||
+	pp.menubar && pp.current_item && !pp.current_item->submenu() // button
 	) {
 #if 0 // makes the check/radio items leave the menu up
-      const Fl_Menu_Item* m = p.current_item;
+      const Fl_Menu_Item* m = pp.current_item;
       if (m && button && (m->flags & (FL_MENU_TOGGLE|FL_MENU_RADIO))) {
 	((Fl_Menu_*)button)->picked(m);
-	p.p[p.menu_number]->redraw();
+	pp.p[pp.menu_number]->redraw();
       } else
 #endif
-	p.state = DONE_STATE;
+	pp.state = DONE_STATE;
     }
     return 1;
   }
@@ -569,11 +610,11 @@ const Fl_Menu_Item* Fl_Menu_Item::pulldown(
   }
   menuwindow mw(this, X, Y, W, H, initial_item, t, menubar);
   Fl::grab(mw);
-  menustate p; ::p = &p;
-  p.p[0] = &mw;
-  p.nummenus = 1;
-  p.menubar = menubar;
-  p.state = INITIAL_STATE;
+  menustate pp; p = &pp;
+  pp.p[0] = &mw;
+  pp.nummenus = 1;
+  pp.menubar = menubar;
+  pp.state = INITIAL_STATE;
 
   menuwindow* fakemenu = 0; // kludge for buttons in menubar
 
@@ -583,50 +624,50 @@ const Fl_Menu_Item* Fl_Menu_Item::pulldown(
     goto STARTUP;
   }
 
-  p.current_item = 0; p.menu_number = 0; p.item_number = -1;
+  pp.current_item = 0; pp.menu_number = 0; pp.item_number = -1;
   if (menubar) mw.handle(FL_DRAG); // find the initial menu
-  initial_item = p.current_item;
+  initial_item = pp.current_item;
   if (initial_item) goto STARTUP;
 
   // the main loop, runs until p.state goes to DONE_STATE:
   for (;;) {
 
     // make sure all the menus are shown:
-    {for (int k = menubar; k < p.nummenus; k++)
-      if (!p.p[k]->shown()) {
-	if (p.p[k]->title) p.p[k]->title->show();
-	p.p[k]->show();
+    {for (int k = menubar; k < pp.nummenus; k++)
+      if (!pp.p[k]->shown()) {
+	if (pp.p[k]->title) pp.p[k]->title->show();
+	pp.p[k]->show();
       }
     }
 
     // get events:
-    {const Fl_Menu_Item* oldi = p.current_item;
+    {const Fl_Menu_Item* oldi = pp.current_item;
     Fl::wait();
-    if (p.state == DONE_STATE) break; // done.
-    if (p.current_item == oldi) continue;}
+    if (pp.state == DONE_STATE) break; // done.
+    if (pp.current_item == oldi) continue;}
     // only do rest if item changes:
 
     delete fakemenu; fakemenu = 0; // turn off "menubar button"
 
-    if (!p.current_item) { // pointing at nothing
+    if (!pp.current_item) { // pointing at nothing
       // turn off selection in deepest menu, but don't erase other menus:
-      p.p[p.nummenus-1]->set_selected(-1);
+      pp.p[pp.nummenus-1]->set_selected(-1);
       continue;
     }
 
     delete fakemenu; fakemenu = 0;
     initial_item = 0; // stop the startup code
-    p.p[p.menu_number]->autoscroll(p.item_number);
+    pp.p[pp.menu_number]->autoscroll(pp.item_number);
 
   STARTUP:
-    menuwindow& cw = *p.p[p.menu_number];
-    const Fl_Menu_Item* m = p.current_item;
+    menuwindow& cw = *pp.p[pp.menu_number];
+    const Fl_Menu_Item* m = pp.current_item;
     if (!m->activevisible()) { // pointing at inactive item
       cw.set_selected(-1);
       initial_item = 0; // turn off startup code
       continue;
     }
-    cw.set_selected(p.item_number);
+    cw.set_selected(pp.item_number);
 
     if (m==initial_item) initial_item=0; // stop the startup code if item found
     if (m->submenu()) {
@@ -636,57 +677,57 @@ const Fl_Menu_Item* Fl_Menu_Item::pulldown(
       else menutable = (Fl_Menu_Item*)(m)->user_data_;
       // figure out where new menu goes:
       int nX, nY;
-      if (!p.menu_number && p.menubar) {	// menu off a menubar:
-	nX = cw.x() + cw.titlex(p.item_number);
+      if (!pp.menu_number && pp.menubar) {	// menu off a menubar:
+	nX = cw.x() + cw.titlex(pp.item_number);
 	nY = cw.y() + cw.h();
 	initial_item = 0;
       } else {
 	nX = cw.x() + cw.w();
-	nY = cw.y() + p.item_number * cw.itemheight;
+	nY = cw.y() + pp.item_number * cw.itemheight;
 	title = 0;
       }
       if (initial_item) { // bring up submenu containing initial item:
-	menuwindow* n = new menuwindow(menutable,X,Y,W,H,initial_item,title);
-	p.p[p.nummenus++] = n;
+	menuwindow* n = new menuwindow(menutable,X,Y,W,H,initial_item,title,0,0,cw.x());
+	pp.p[pp.nummenus++] = n;
 	// move all earlier menus to line up with this new one:
 	if (n->selected>=0) {
 	  int dy = n->y()-nY;
 	  int dx = n->x()-nX;
-	  for (int menu = 0; menu <= p.menu_number; menu++) {
-	    menuwindow* t = p.p[menu];
-	    int nx = t->x()+dx; if (nx < 0) {nx = 0; dx = -t->x();}
-	    int ny = t->y()+dy; if (ny < 0) {ny = 0; dy = -t->y();}
-	    t->position(nx, ny);
+	  for (int menu = 0; menu <= pp.menu_number; menu++) {
+	    menuwindow* tt = pp.p[menu];
+	    int nx = tt->x()+dx; if (nx < 0) {nx = 0; dx = -tt->x();}
+	    int ny = tt->y()+dy; if (ny < 0) {ny = 0; dy = -tt->y();}
+	    tt->position(nx, ny);
 	  }
-	  setitem(p.nummenus-1, n->selected);
+	  setitem(pp.nummenus-1, n->selected);
 	  goto STARTUP;
 	}
-      } else if (p.nummenus > p.menu_number+1 &&
-		 p.p[p.menu_number+1]->menu == menutable) {
+      } else if (pp.nummenus > pp.menu_number+1 &&
+		 pp.p[pp.menu_number+1]->menu == menutable) {
 	// the menu is already up:
-	while (p.nummenus > p.menu_number+2) delete p.p[--p.nummenus];
-	p.p[p.nummenus-1]->set_selected(-1);
+	while (pp.nummenus > pp.menu_number+2) delete pp.p[--pp.nummenus];
+	pp.p[pp.nummenus-1]->set_selected(-1);
       } else {
 	// delete all the old menus and create new one:
-	while (p.nummenus > p.menu_number+1) delete p.p[--p.nummenus];
-	p.p[p.nummenus++]= new menuwindow(menutable, nX, nY,
-					  title?1:0, 0, 0, title, 0, menubar);
+	while (pp.nummenus > pp.menu_number+1) delete pp.p[--pp.nummenus];
+	pp.p[pp.nummenus++]= new menuwindow(menutable, nX, nY,
+					  title?1:0, 0, 0, title, 0, menubar, cw.x());
       }
     } else { // !m->submenu():
-      while (p.nummenus > p.menu_number+1) delete p.p[--p.nummenus];
-      if (!p.menu_number && p.menubar) {
+      while (pp.nummenus > pp.menu_number+1) delete pp.p[--pp.nummenus];
+      if (!pp.menu_number && pp.menubar) {
 	// kludge so "menubar buttons" turn "on" by using menu title:
 	fakemenu = new menuwindow(0,
-				  cw.x()+cw.titlex(p.item_number),
+				  cw.x()+cw.titlex(pp.item_number),
 				  cw.y()+cw.h(), 0, 0,
 				  0, m, 0, 1);
 	fakemenu->title->show();
       }
     }
   }
-  const Fl_Menu_Item* m = p.current_item;
+  const Fl_Menu_Item* m = pp.current_item;
   delete fakemenu;
-  while (p.nummenus>1) delete p.p[--p.nummenus];
+  while (pp.nummenus>1) delete pp.p[--pp.nummenus];
   mw.hide();
   Fl::release();
   return m;
@@ -697,12 +738,12 @@ Fl_Menu_Item::popup(
   int X, int Y,
   const char* title,
   const Fl_Menu_Item* picked,
-  const Fl_Menu_* button
+  const Fl_Menu_* but
   ) const
 {
   static Fl_Menu_Item dummy; // static so it is all zeros
   dummy.text = title;
-  return pulldown(X, Y, 0, 0, picked, button, title ? &dummy : 0);
+  return pulldown(X, Y, 0, 0, picked, but, title ? &dummy : 0);
 }
 
 // Search only the top level menu for a shortcut.  Either &x in the
@@ -743,5 +784,5 @@ const Fl_Menu_Item* Fl_Menu_Item::test_shortcut() const {
 }
 
 //
-// End of "$Id: Fl_Menu.cxx,v 1.18.2.11 2000/11/20 19:02:20 easysw Exp $".
+// End of "$Id: Fl_Menu.cxx,v 1.18.2.12.2.23 2003/01/30 21:42:05 easysw Exp $".
 //
