@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Menu_.cxx,v 1.7.2.7 2000/06/05 21:20:54 mike Exp $"
+// "$Id: Fl_Menu_.cxx,v 1.7.2.8.2.6 2003/01/30 21:42:11 easysw Exp $"
 //
 // Common menu code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -28,11 +28,11 @@
 // This provides storage for a menu item, functions to add/modify/delete
 // items, and a call for when the user picks a menu item.
 
-// More code in Fl_Menu_add.C
+// More code in Fl_Menu_add.cxx
 
 #include <FL/Fl.H>
 #include <FL/Fl_Menu_.H>
-#include <string.h>
+#include "flstring.h"
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -97,7 +97,7 @@ Fl_Menu_::Fl_Menu_(int X,int Y,int W,int H,const char* l)
   alloc = 0;
   selection_color(FL_SELECTION_COLOR);
   textfont(FL_HELVETICA);
-  textsize(FL_NORMAL_SIZE);
+  textsize((uchar)FL_NORMAL_SIZE);
   textcolor(FL_BLACK);
   down_box(FL_NO_BOX);
 }
@@ -112,43 +112,21 @@ void Fl_Menu_::menu(const Fl_Menu_Item* m) {
   value_ = menu_ = (Fl_Menu_Item*)m;
 }
 
-#if 1
 // this version is ok with new Fl_Menu_add code with fl_menu_array_owner:
 
-void Fl_Menu_::copy(const Fl_Menu_Item* m, void* user_data) {
+void Fl_Menu_::copy(const Fl_Menu_Item* m, void* ud) {
   int n = m->size();
   Fl_Menu_Item* newMenu = new Fl_Menu_Item[n];
   memcpy(newMenu, m, n*sizeof(Fl_Menu_Item));
   menu(newMenu);
   alloc = 1; // make destructor free array, but not strings
   // for convienence, provide way to change all the user data pointers:
-  if (user_data) for (; n--;) {
-    if (newMenu->callback_) newMenu->user_data_ = user_data;
+  if (ud) for (; n--;) {
+    if (newMenu->callback_) newMenu->user_data_ = ud;
     newMenu++;
   }
 }
 
-#else
-// This is Guillaume Nodet's fixed version for the older Fl_Menu_add
-// that enlarged the array at powers of 2:
-
-void Fl_Menu_::copy(const Fl_Menu_Item* m, void* user_data) {
-  int i, s = m->size(), n=s;
-  for (i=0; n; n>>=1, i++);
-  n = 1 << i;
-  Fl_Menu_Item* newMenu = new Fl_Menu_Item[n];
-  memcpy(newMenu, m, s*sizeof(Fl_Menu_Item));
-  memset(newMenu+s, 0, (n-s)*sizeof(Fl_Menu_Item));
-  menu(newMenu);
-  alloc = 1; // make destructor free it
-  // for convienence, provide way to change all the user data pointers:
-  if (user_data) for (; s--;) {
-    if (newMenu->callback_) newMenu->user_data_ = user_data;
-    newMenu++;
-  }
-}
-#endif
-
 Fl_Menu_::~Fl_Menu_() {
   clear();
 }
@@ -172,5 +150,5 @@ void Fl_Menu_::clear() {
 }
 
 //
-// End of "$Id: Fl_Menu_.cxx,v 1.7.2.7 2000/06/05 21:20:54 mike Exp $".
+// End of "$Id: Fl_Menu_.cxx,v 1.7.2.8.2.6 2003/01/30 21:42:11 easysw Exp $".
 //
