@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Pack.cxx,v 1.6.2.3 2000/06/20 05:47:37 bill Exp $"
+// "$Id: Fl_Pack.cxx,v 1.6.2.4.2.4 2003/01/30 21:42:25 easysw Exp $"
 //
 // Packing widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -32,8 +32,8 @@
 #include <FL/Fl_Pack.H>
 #include <FL/fl_draw.H>
 
-Fl_Pack::Fl_Pack(int x,int y,int w ,int h,const char *l)
-: Fl_Group(x, y, w, h, l) {
+Fl_Pack::Fl_Pack(int X, int Y, int W, int H,const char *l)
+: Fl_Group(X, Y, W, H, l) {
   resizable(0);
   spacing_ = 0;
   // type(VERTICAL); // already set like this
@@ -44,10 +44,30 @@ void Fl_Pack::draw() {
   int ty = y()+Fl::box_dy(box());
   int tw = w()-Fl::box_dw(box());
   int th = h()-Fl::box_dh(box());
+  int rw, rh;
   int current_position = horizontal() ? tx : ty;
   int maximum_position = current_position;
   uchar d = damage();
   Fl_Widget*const* a = array();
+  if (horizontal()) {
+    rw = -spacing_;
+    rh = th;
+
+    for (int i = children(); i--;)
+      if (child(i)->visible()) {
+	if (child(i) != this->resizable()) rw += child(i)->w();
+	rw += spacing_;
+      }
+  } else {
+    rw = tw;
+    rh = -spacing_;
+
+    for (int i = children(); i--;)
+      if (child(i)->visible()) {
+	if (child(i) != this->resizable()) rh += child(i)->h();
+	rh += spacing_;
+      }
+  }
   for (int i = children(); i--;) {
     Fl_Widget* o = *a++;
     if (o->visible()) {
@@ -66,9 +86,9 @@ void Fl_Pack::draw() {
       // Last child, if resizable, takes all remaining room
       if(i == 0 && o == this->resizable()) {
        if(horizontal())
-         W = this->w() - Fl::box_dw(box()) - maximum_position;
+         W = tw - rw;
        else
-         H = this->h() - Fl::box_dh(box()) - maximum_position;
+         H = th - rh;
       }
       if (spacing_ && current_position>maximum_position &&
   	  (X != o->x() || Y != o->y() || d&FL_DAMAGE_ALL)) {
@@ -112,5 +132,5 @@ void Fl_Pack::draw() {
 }
 
 //
-// End of "$Id: Fl_Pack.cxx,v 1.6.2.3 2000/06/20 05:47:37 bill Exp $".
+// End of "$Id: Fl_Pack.cxx,v 1.6.2.4.2.4 2003/01/30 21:42:25 easysw Exp $".
 //
