@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Scroll.cxx,v 1.7.2.5 2000/11/20 15:30:54 easysw Exp $"
+// "$Id: Fl_Scroll.cxx,v 1.7.2.6.2.4 2003/01/30 21:42:37 easysw Exp $"
 //
 // Scroll widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -29,9 +29,8 @@
 
 // Insure the scrollbars are the last children:
 void Fl_Scroll::fix_scrollbar_order() {
-  Fl_Widget*const* a = array();
+  Fl_Widget** a = (Fl_Widget**)array();
   if (a[children()-1] != &scrollbar) {
-    Fl_Widget** a = (Fl_Widget**)array();
     int i,j; for (i = j = 0; j < children(); j++)
       if (a[j] != &hscrollbar && a[j] != &scrollbar) a[i++] = a[j];
     a[i++] = &hscrollbar;
@@ -51,10 +50,20 @@ void Fl_Scroll::draw_clip(void* v,int X, int Y, int W, int H) {
   int R = X; int B = Y; // track bottom & right edge of all children
   for (int i=s->children()-2; i--;) {
     Fl_Widget& o = **a++;
+    int NR, NB;
     s->draw_child(o);
     s->draw_outside_label(o);
-    if (o.x()+o.w() > R) R = o.x()+o.w();
-    if (o.y()+o.h() > B) B = o.y()+o.h();
+    NR = o.x()+o.w();
+    NB = o.y()+o.h();
+    if ((o.align() & (FL_ALIGN_BOTTOM | FL_ALIGN_RIGHT)) &&
+        !(o.align() & FL_ALIGN_INSIDE)) {
+      int LW = 0, LH = 0;
+      o.measure_label(LW, LH);
+      if (o.align() & FL_ALIGN_BOTTOM) NB += LH;
+      else NR += LW;
+    }
+    if (NR > R) R = NR;
+    if (NB > B) B = NB;
   }
   // fill any area to right & bottom of widgets:
   if (R < X+W && B > Y) {
@@ -249,5 +258,5 @@ int Fl_Scroll::handle(int event) {
 }
 
 //
-// End of "$Id: Fl_Scroll.cxx,v 1.7.2.5 2000/11/20 15:30:54 easysw Exp $".
+// End of "$Id: Fl_Scroll.cxx,v 1.7.2.6.2.4 2003/01/30 21:42:37 easysw Exp $".
 //
