@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Tabs.cxx,v 1.6.2.10 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: Fl_Tabs.cxx,v 1.6.2.10.2.17 2003/01/30 21:42:46 easysw Exp $"
 //
 // Tab widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -29,58 +29,57 @@
 // Each child widget is a card, and it's label() is printed on the card tab.
 // Clicking the tab makes that card visible.
 
+#include <stdio.h>
 #include <FL/Fl.H>
 #include <FL/Fl_Tabs.H>
 #include <FL/fl_draw.H>
 
-#define BORDER 10
-#define TABSLOPE 5
-#define EXTRASPACE 5
+#define BORDER 2
+#define EXTRASPACE 10
 
 // return the left edges of each tab (plus a fake left edge for a tab
 // past the right-hand one).  These position are actually of the left
 // edge of the slope.  They are either seperated by the correct distance
-// or by TABSLOPE or by zero.
+// or by EXTRASPACE or by zero.
 // Return value is the index of the selected item.
 
-int Fl_Tabs::tab_positions(int* p, int* w) {
-  int selected = -1;
+int Fl_Tabs::tab_positions(int* p, int* wp) {
+  int selected = 0;
   Fl_Widget*const* a = array();
   int i;
   p[0] = 0;
   for (i=0; i<children(); i++) {
     Fl_Widget* o = *a++;
     if (o->visible()) selected = i;
-    if (o->label()) {
-      int wt = 0; int ht = 0; o->measure_label(wt,ht);
-      w[i] = wt+TABSLOPE+EXTRASPACE;
-      //if (2*TABSLOPE > w[i]) w[i] = 2*TABSLOPE;
-    } else 
-      w[i] = 2*TABSLOPE;
-    p[i+1] = p[i]+w[i];
+
+    int wt = 0; int ht = 0;
+    o->measure_label(wt,ht);
+
+    wp[i]  = wt+EXTRASPACE;
+    p[i+1] = p[i]+wp[i]+BORDER;
   }
-  int r = this->w()-TABSLOPE-1;
+  int r = w();
   if (p[i] <= r) return selected;
   // uh oh, they are too big:
   // pack them against right edge:
   p[i] = r;
   for (i = children(); i--;) {
-    int l = r-w[i];
-    if (p[i+1]-TABSLOPE < l) l = p[i+1]-TABSLOPE;
+    int l = r-wp[i];
+    if (p[i+1] < l) l = p[i+1];
     if (p[i] <= l) break;
     p[i] = l;
-    r -= TABSLOPE;
+    r -= EXTRASPACE;
   }
   // pack them against left edge and truncate width if they still don't fit:
   for (i = 0; i<children(); i++) {
-    if (p[i] >= i*TABSLOPE) break;
-    p[i] = i*TABSLOPE;
-    int W = this->w()-1-TABSLOPE*(children()-i) - p[i];
-    if (w[i] > W) w[i] = W;
+    if (p[i] >= i*EXTRASPACE) break;
+    p[i] = i*EXTRASPACE;
+    int W = w()-1-EXTRASPACE*(children()-i) - p[i];
+    if (wp[i] > W) wp[i] = W;
   }
   // adjust edges according to visiblity:
   for (i = children(); i > selected; i--) {
-    p[i] = p[i-1]+w[i-1];
+    p[i] = p[i-1]+wp[i-1];
   }
   return selected;
 }
@@ -96,13 +95,8 @@ int Fl_Tabs::tab_height() {
     if (o->y()+o->h() > H2) H2 = o->y()+o->h();
   }
   H2 = y()+h()-H2;
-  if (H2 > H) {
-    H = H2-Fl::box_dy(box());
-    return (H <= 0) ? 0 : -H;
-  } else {
-    H = H-Fl::box_dy(box());
-    return (H <= 0) ? 0 : H;
-  }
+  if (H2 > H) return (H2 <= 0) ? 0 : -H2;
+  else return (H <= 0) ? 0 : H;
 }
 
 // this is used by fluid to pick tabs:
@@ -114,11 +108,10 @@ Fl_Widget *Fl_Tabs::which(int event_x, int event_y) {
     if (event_y > y()+H || event_y < y()) return 0;
   }
   if (event_x < x()) return 0;
-  int p[128], w[128];
-  int selected = tab_positions(p, w);
-  int d = (event_y-(H>=0?y():y()+h()))*TABSLOPE/H;
+  int p[128], wp[128];
+  tab_positions(p, wp);
   for (int i=0; i<children(); i++) {
-    if (event_x < x()+p[i+1]+(i<selected ? TABSLOPE-d : d)) return child(i);
+    if (event_x < x()+p[i+1]) return child(i);
   }
   return 0;
 }
@@ -126,25 +119,63 @@ Fl_Widget *Fl_Tabs::which(int event_x, int event_y) {
 int Fl_Tabs::handle(int event) {
 
   Fl_Widget *o;
+  int i;
 
   switch (event) {
 
   case FL_PUSH: {
     int H = tab_height();
     if (H >= 0) {
-      if (Fl::event_y() > y()+H) goto DEFAULT;
+      if (Fl::event_y() > y()+H) return Fl_Group::handle(event);
     } else {
-      if (Fl::event_y() < y()+h()+H) goto DEFAULT;
+      if (Fl::event_y() < y()+h()+H) return Fl_Group::handle(event);
     }}
   case FL_DRAG:
   case FL_RELEASE:
     o = which(Fl::event_x(), Fl::event_y());
     if (event == FL_RELEASE) {push(0); if (o && value(o)) do_callback();}
     else push(o);
+    if (Fl::visible_focus() && event == FL_RELEASE) Fl::focus(this);
     return 1;
-
+  case FL_FOCUS:
+  case FL_UNFOCUS:
+    if (!Fl::visible_focus()) return Fl_Group::handle(event);
+    if (Fl::event() == FL_RELEASE ||
+	Fl::event() == FL_SHORTCUT ||
+	Fl::event() == FL_KEYBOARD) {
+      int H = tab_height();
+      if (H >= 0) {
+        H += Fl::box_dy(box());
+	damage(FL_DAMAGE_SCROLL, x(), y(), w(), H);
+      } else {
+        H = Fl::box_dy(box()) - H;
+        damage(FL_DAMAGE_SCROLL, x(), y() + h() - H, w(), H);
+      }
+      return 1;
+    } else return Fl_Group::handle(event);
+  case FL_KEYBOARD:
+    switch (Fl::event_key()) {
+      case FL_Left:
+        if (child(0)->visible()) return 0;
+	for (i = 1; i < children(); i ++)
+	  if (child(i)->visible()) break;
+	value(child(i - 1));
+	do_callback();
+        return 1;
+      case FL_Right:
+        if (child(children() - 1)->visible()) return 0;
+	for (i = 0; i < children(); i ++)
+	  if (child(i)->visible()) break;
+	value(child(i + 1));
+	do_callback();
+        return 1;
+      case FL_Down:
+        redraw();
+        return Fl_Group::handle(FL_FOCUS);
+      default:
+        break;
+    }
   default:
-  DEFAULT:
     return Fl_Group::handle(event);
 
   }
@@ -153,7 +184,7 @@ int Fl_Tabs::handle(int event) {
 int Fl_Tabs::push(Fl_Widget *o) {
   if (push_ == o) return 0;
   if (push_ && !push_->visible() || o && !o->visible())
-    damage(FL_DAMAGE_EXPOSE);
+    redraw();
   push_ = o;
   return 1;
 }
@@ -196,75 +227,71 @@ enum {LEFT, RIGHT, SELECTED};
 void Fl_Tabs::draw() {
   Fl_Widget *v = value();
   int H = tab_height();
+
   if (damage() & FL_DAMAGE_ALL) { // redraw the entire thing:
-    fl_color(color());
-    fl_rectf(x(), y()+(H>=0?0:h()+H), w(), H>=0?H:-H);
-    fl_clip(x(), y()+(H>=0?H:0), w(), h()-(H>=0?H:-H));
-    draw_box(box(), x(), y(), w(), h(), v ? v->color() : color());
-    fl_pop_clip();
+    draw_box(box(), x(), y()+(H>=0?H:0), w(), h()-(H>=0?H:-H), v ? v->color() : color());
     if (v) draw_child(*v);
   } else { // redraw the child
     if (v) update_child(*v);
   }
-  if (damage() & (FL_DAMAGE_EXPOSE|FL_DAMAGE_ALL)) {
-    int p[128]; int w[128];
-    int selected = tab_positions(p,w);
+  if (damage() & (FL_DAMAGE_SCROLL|FL_DAMAGE_ALL)) {
+    int p[128]; int wp[128];
+    int selected = tab_positions(p,wp);
     int i;
     Fl_Widget*const* a = array();
     for (i=0; i<selected; i++)
-      draw_tab(x()+p[i], x()+p[i+1], w[i], H, a[i], LEFT);
+      draw_tab(x()+p[i], x()+p[i+1], wp[i], H, a[i], LEFT);
     for (i=children()-1; i > selected; i--)
-      draw_tab(x()+p[i], x()+p[i+1], w[i], H, a[i], RIGHT);
+      draw_tab(x()+p[i], x()+p[i+1], wp[i], H, a[i], RIGHT);
     if (v) {
       i = selected;
-      draw_tab(x()+p[i], x()+p[i+1], w[i], H, a[i], SELECTED);
-    } else {
-      // draw the edge when no selection:
-      fl_color(H >= 0 ? FL_LIGHT3 : FL_DARK3);
-      fl_xyline(x(), H >= 0 ? y()+H : y()+h()+H, x()+this->w());
+      draw_tab(x()+p[i], x()+p[i+1], wp[i], H, a[i], SELECTED);
     }
   }
 }
 
 void Fl_Tabs::draw_tab(int x1, int x2, int W, int H, Fl_Widget* o, int what) {
-  if (x2 < x1+W) {
-    if (what == LEFT) {
-      if (x1+W < x2+TABSLOPE) x2 = x1+W;
-      else x2 += TABSLOPE;
-    } else {
-      if (x1+W < x2+TABSLOPE) x1 = x2-W;
-      else x1 -= TABSLOPE;
-    }
-  }
   int sel = (what == SELECTED);
-  fl_color(o->color());
+  int dh = Fl::box_dh(box());
+  int dy = Fl::box_dy(box());
+
+  if ((x2 < x1+W) && what == RIGHT) x1 = x2 - W;
+
   if (H >= 0) {
-    fl_polygon(x1, y()+H+sel, x1+TABSLOPE, y(), x2, y(),
-	       x2+TABSLOPE, y()+H+sel);
-    fl_color(!sel && o==push_ ? FL_DARK3 : FL_LIGHT3);
-    fl_line(x1, y()+H, x1+TABSLOPE, y(), x2, y());
-    if (sel) {
-      if (x1>x()) fl_xyline(x(), y()+H, x1);
-      if (x2+TABSLOPE < x()+w()-1) fl_xyline(x2+TABSLOPE, y()+H, x()+w()-1);
-    }
-    fl_color(!sel && o==push_ ? FL_LIGHT3 : FL_DARK3);
-    fl_line(x2, y(), x2+TABSLOPE, y()+H);
+    if (sel) fl_clip(x1, y(), x2 - x1, H + dh - dy);
+    else fl_clip(x1, y(), x2 - x1, H);
+
+    H += dh;
+
+    draw_box(box(), x1, y(), W, H,
+             sel ? fl_color_average(selection_color(), o->selection_color(), 0.5f)
+	         : o->selection_color());
+
+    o->draw_label(x1, y(), W, H, FL_ALIGN_CENTER);
+
+    if (Fl::focus() == this && o->visible())
+      draw_focus(box(), x1, y(), W, H);
+
+    fl_pop_clip();
   } else {
-    fl_polygon(x1, y()+h()+H-sel, x1+TABSLOPE, y()+h(), x2, y()+h(),
-	       x2+TABSLOPE, y()+h()+H-sel);
-    fl_color(!sel && o==push_ ? FL_LIGHT3 : FL_DARK3);
-    fl_line(x1+TABSLOPE, y()+h()-1, x2, y()+h()-1, x2+TABSLOPE, y()+h()+H);
-    if (sel) {
-      if (x1>x()) fl_xyline(x(), y()+h()+H, x1);
-      if (x2+TABSLOPE < x()+w()-1) fl_xyline(x2+TABSLOPE, y()+h()+H,x()+w()-1);
-    }
-    fl_color(!sel && o==push_ ? FL_DARK3 : FL_LIGHT3);
-    fl_line(x1, y()+h()+H, x1+TABSLOPE, y()+h()-1);
+    H = -H;
+
+    if (sel) fl_clip(x1, y() + h() - H - dy, x2 - x1, H + dy);
+    else fl_clip(x1, y() + h() - H, x2 - x1, H);
+
+    H += dh;
+
+    draw_box(box(), x1, y() + h() - H, W, H,
+             sel ? fl_color_average(selection_color(), o->selection_color(), 0.5f)
+	         : o->selection_color());
+
+    o->draw_label(x1, y() + h() - H, W, H, FL_ALIGN_CENTER);
+
+    if (Fl::focus() == this && o->visible())
+      draw_focus(box(), x1, y() + h() - H, W, H);
+
+    fl_pop_clip();
   }
-  if (W > TABSLOPE+EXTRASPACE/2)
-    o->draw_label((what==LEFT ? x1 : x2-W)+(TABSLOPE+EXTRASPACE/2),
-		  y()+(H<0?h()+H-2:0), W-(TABSLOPE+EXTRASPACE/2),
-		  (H<0?-H:H)+3, FL_ALIGN_CENTER);
 }
 
 Fl_Tabs::Fl_Tabs(int X,int Y,int W, int H, const char *l) :
@@ -275,5 +302,5 @@ Fl_Tabs::Fl_Tabs(int X,int Y,int W, int H, const char *l) :
 }
 
 //
-// End of "$Id: Fl_Tabs.cxx,v 1.6.2.10 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: Fl_Tabs.cxx,v 1.6.2.10.2.17 2003/01/30 21:42:46 easysw Exp $".
 //
