@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_win32.cxx,v 1.33.2.30 2000/11/20 14:17:43 easysw Exp $"
+// "$Id: Fl_win32.cxx,v 1.33.2.37.2.44 2003/01/30 21:43:11 easysw Exp $"
 //
 // WIN32-specific code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -24,39 +24,80 @@
 //
 
 // This file contains win32-specific code for fltk which is always linked
-// in.  Search other files for "WIN32" or filenames ending in _win32.C
+// in.  Search other files for "WIN32" or filenames ending in _win32.cxx
 // for other system-specific code.
 
-#include <config.h>
 #include <FL/Fl.H>
-#include <FL/win32.H>
+#include <FL/x.H>
 #include <FL/Fl_Window.H>
-#include <string.h>
+#include "flstring.h"
+#include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <time.h>
-#include <winsock.h>
-#include <ctype.h>
+#ifdef __CYGWIN__
+#  include <sys/time.h>
+#  include <unistd.h>
+#else
+#  include <winsock.h>
+#endif
+#include <winuser.h>
+#include <commctrl.h>
+
+// The following include files require GCC 3.x or a non-GNU compiler...
+#if !defined(__GNUC__) || __GNUC__ >= 3
+#  include <ole2.h>
+#  include <ShellApi.h>
+#endif // !__GNUC__ || __GNUC__ >= 3
+
 
 //
 // USE_ASYNC_SELECT - define it if you have WSAAsyncSelect()...
 //
+// This currently doesn't appear to work; needs to be fixed!
+//
 
-// MRS - this currently doesn't appear to work anymore; needs to be fixed!
 //#define USE_ASYNC_SELECT
 
+
+//
+// USE_TRACK_MOUSE - define it if you have TrackMouseEvent()...
+//
+// Apparently, at least some versions of Cygwin/MingW don't provide
+// the TrackMouseEvent() function.  You can define this by hand
+// if you have it - this is only needed to support subwindow
+// enter/leave notification under Windows.
+//
+
+//#define USE_TRACK_MOUSE
+
+#if !defined(__GNUC__)
+#  define USE_TRACK_MOUSE
+#endif // !__GNUC__
+
+
 //
 // WM_SYNCPAINT is an "undocumented" message, which is finally defined in
 // VC++ 6.0.
 //
+
 #ifndef WM_SYNCPAINT
 #  define WM_SYNCPAINT 0x0088
-#endif /* !WM_SYNCPAINT */
+#endif
 
 #ifndef WM_MOUSELEAVE
 #  define WM_MOUSELEAVE 0x02a3
 #endif
 
+#ifndef WM_MOUSEWHEEL
+#  define WM_MOUSEWHEEL 0x020a
+#endif
+
+#ifndef WHEEL_DELTA
+#  define WHEEL_DELTA 120	// according to MSDN.
+#endif
+
+
 //
 // WM_FLSELECT is the user-defined message that we get when one of
 // the sockets has pending data, etc.
@@ -64,6 +105,7 @@
 
 #define WM_FLSELECT	(WM_USER+0x0400)
 
+
 ////////////////////////////////////////////////////////////////
 // interface to poll/select call:
 
@@ -74,7 +116,7 @@
 // Microsoft provides the Berkeley select() call and an asynchronous
 // select function that sends a WIN32 message when the select condition
 // exists...
-
+static int maxfd = 0;
 #ifndef USE_ASYNC_SELECT
 static fd_set fdsets[3];
 #endif // !USE_ASYNC_SELECT
@@ -83,6 +125,10 @@ static fd_set fdsets[3];
 #define POLLOUT 4
 #define POLLERR 8
 
+#if !defined(__GNUC__) || __GNUC__ >= 3
+extern IDropTarget *flIDropTarget;
+#endif // !__GNUC__ || __GNUC__ >= 3
+
 static int nfds = 0;
 static int fd_array_size = 0;
 static struct FD {
@@ -100,7 +146,7 @@ void Fl::add_fd(int n, int events, void (*cb)(int, void*), void *v) {
     fd = (FD*)realloc(fd, fd_array_size*sizeof(FD));
   }
   fd[i].fd = n;
-  fd[i].events = events;
+  fd[i].events = (short)events;
   fd[i].cb = cb;
   fd[i].arg = v;
 
@@ -114,6 +160,7 @@ void Fl::add_fd(int n, int events, void (*cb)(int, void*), void *v) {
   if (events & POLLIN) FD_SET(n, &fdsets[0]);
   if (events & POLLOUT) FD_SET(n, &fdsets[1]);
   if (events & POLLERR) FD_SET(n, &fdsets[2]);
+  if (n > maxfd) maxfd = n;
 #endif // USE_ASYNC_SELECT
 }
 
@@ -125,7 +172,7 @@ void Fl::remove_fd(int n, int events) {
   int i,j;
   for (i=j=0; i<nfds; i++) {
     if (fd[i].fd == n) {
-      int e = fd[i].events & ~events;
+      short e = fd[i].events & ~events;
       if (!e) continue; // if no events left, delete this fd
       fd[i].events = e;
     }
@@ -150,6 +197,18 @@ void Fl::remove_fd(int n) {
   remove_fd(n, -1);
 }
 
+// these pointers are set by the Fl::lock() function:
+static void nothing() {}
+void (*fl_lock_function)() = nothing;
+void (*fl_unlock_function)() = nothing;
+
+static void* thread_message_;
+void* Fl::thread_message() {
+  void* r = thread_message_;
+  thread_message_ = 0;
+  return r;
+}
+
 MSG fl_msg;
 
 // This is never called with time_to_wait < 0.0.
@@ -173,8 +232,7 @@ int fl_wait(double time_to_wait) {
     fdt[0] = fdsets[0];
     fdt[1] = fdsets[1];
     fdt[2] = fdsets[2];
-
-    if (::select(0,&fdt[0],&fdt[1],&fdt[2],&t)) {
+    if (::select(maxfd+1,&fdt[0],&fdt[1],&fdt[2],&t)) {
       // We got something - do the callback!
       for (int i = 0; i < nfds; i ++) {
 	int f = fd[i].fd;
@@ -185,19 +243,28 @@ int fl_wait(double time_to_wait) {
 	if (fd[i].events & revents) fd[i].cb(f, fd[i].arg);
       }
       time_to_wait = 0.0; // just peek for any messages
+#ifdef __CYGWIN__
+    }
+#else
     } else {
       // we need to check them periodically, so set a short timeout:
       if (time_to_wait > .001) time_to_wait = .001;
     }
+#endif
   }
 #endif // USE_ASYNC_SELECT
 
+  fl_unlock_function();
+
   if (time_to_wait < 2147483.648) {
     // Perform the requested timeout...
     have_message = PeekMessage(&fl_msg, NULL, 0, 0, PM_REMOVE);
     if (!have_message) {
       int t = (int)(time_to_wait * 1000.0 + .5);
-      if (t <= 0) return 0; // too short to measure
+      if (t <= 0) { // too short to measure
+        fl_lock_function();
+	return 0;
+      }
       timerid = SetTimer(NULL, 0, t, NULL);
       have_message = GetMessage(&fl_msg, NULL, 0, 0);
       KillTimer(NULL, timerid);
@@ -206,6 +273,8 @@ int fl_wait(double time_to_wait) {
     have_message = GetMessage(&fl_msg, NULL, 0, 0);
   }
 
+  fl_lock_function();
+
   // Execute the message we got, and all other pending messages:
   while (have_message) {
 #ifdef USE_ASYNC_SELECT
@@ -219,6 +288,10 @@ int fl_wait(double time_to_wait) {
       // looks like it is best to do the dispatch-message anyway:
     }
 #endif
+
+    if (fl_msg.message == fl_wake_msg)  // Used for awaking wait() from another thread
+      thread_message_ = (void*)fl_msg.wParam;
+
     TranslateMessage(&fl_msg);
     DispatchMessage(&fl_msg);
     have_message = PeekMessage(&fl_msg, NULL, 0, 0, PM_REMOVE);
@@ -286,6 +359,65 @@ void Fl::get_mouse(int &x, int &y) {
   y = p.y;
 }
 
+////////////////////////////////////////////////////////////////
+// code used for selections:
+
+char *fl_selection_buffer[2];
+int fl_selection_length[2];
+int fl_selection_buffer_length[2];
+char fl_i_own_selection[2];
+
+// call this when you create a selection:
+void Fl::copy(const char *stuff, int len, int clipboard) {
+  if (!stuff || len<0) return;
+  if (len+1 > fl_selection_buffer_length[clipboard]) {
+    delete[] fl_selection_buffer[clipboard];
+    fl_selection_buffer[clipboard] = new char[len+100];
+    fl_selection_buffer_length[clipboard] = len+100;
+  }
+  memcpy(fl_selection_buffer[clipboard], stuff, len);
+  fl_selection_buffer[clipboard][len] = 0; // needed for direct paste
+  fl_selection_length[clipboard] = len;
+  if (clipboard) {
+    // set up for "delayed rendering":
+    if (OpenClipboard(fl_xid(Fl::first_window()))) {
+      EmptyClipboard();
+      SetClipboardData(CF_TEXT, NULL);
+      CloseClipboard();
+    }
+    fl_i_own_selection[clipboard] = 1;
+  }
+}
+
+// Call this when a "paste" operation happens:
+void Fl::paste(Fl_Widget &receiver, int clipboard) {
+  if (!clipboard || fl_i_own_selection[clipboard]) {
+    // We already have it, do it quickly without window server.
+    // Notice that the text is clobbered if set_selection is
+    // called in response to FL_PASTE!
+    Fl::e_text = fl_selection_buffer[clipboard];
+    Fl::e_length = fl_selection_length[clipboard];
+    receiver.handle(FL_PASTE);
+  } else {
+    if (!OpenClipboard(NULL)) return;
+    HANDLE h = GetClipboardData(CF_TEXT);
+    if (h) {
+      Fl::e_text = (LPSTR)GlobalLock(h);
+      LPSTR a,b;
+      a = b = Fl::e_text;
+      while (*a) { // strip the CRLF pairs ($%$#@^)
+	if (*a == '\r' && a[1] == '\n') a++;
+	else *b++ = *a++;
+      }
+      *b = 0;
+      Fl::e_length = b - Fl::e_text;
+      receiver.handle(FL_PASTE);
+      GlobalUnlock(h);
+    }
+    CloseClipboard();
+  }
+}
+
 ////////////////////////////////////////////////////////////////
 
 HWND fl_capture;
@@ -346,7 +478,7 @@ static int mouse_event(Fl_Window *window, int what, int button,
 }
 
 // convert a MSWindows VK_x to an Fltk (X) Keysym:
-// See also the inverse converter in Fl_get_key_win32.C
+// See also the inverse converter in Fl_get_key_win32.cxx
 // This table is in numeric order by VK:
 static const struct {unsigned short vk, fltk, extended;} vktab[] = {
   {VK_BACK,	FL_BackSpace},
@@ -411,28 +543,26 @@ static int ms2fltk(int vk, int extended) {
 }
 
 #if USE_COLORMAP
-extern HPALETTE fl_select_palette(void); // in fl_color_win32.C
+extern HPALETTE fl_select_palette(void); // in fl_color_win32.cxx
 #endif
 
 static Fl_Window* resize_bug_fix;
 
+extern void fl_save_pen(void);
+extern void fl_restore_pen(void);
+
 static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
-
-#if 0
-  // Not sure what this is, it may be left over from earlier attempts to
-  // treat WM_PAINT as an expose event, rather than painting in response
-  // to it.
-  static int cnt=0;
-  if(uMsg == WM_SYNCPAINT) {
-    if(cnt) {
-      InvalidateRect(fl_window,0,FALSE);
-      cnt = 0;
-    } else cnt = 1;
-  } else if (uMsg == WM_PAINT) cnt = 0;
-#endif
-
+  // Copy the message to fl_msg so add_handler code can see it, it is
+  // already there if this is called by DispatchMessage, but not if
+  // Windows calls this directly.
+  fl_msg.hwnd = hWnd;
   fl_msg.message = uMsg;
+  fl_msg.wParam = wParam;
+  fl_msg.lParam = lParam;
+  //fl_msg.time = ???
+  //fl_msg.pt = ???
+  //fl_msg.lPrivate = ???
 
   Fl_Window *window = fl_find(hWnd);
 
@@ -445,34 +575,47 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
     Fl::handle(FL_CLOSE, window);
     return 0;
 
-  case WM_PAINT: {
+  case WM_SYNCPAINT :
+  case WM_NCPAINT :
+  case WM_ERASEBKGND :
+    // Andreas Weitl - WM_SYNCPAINT needs to be passed to DefWindowProc
+    // so that Windows can generate the proper paint messages...
+    // Similarly, WM_NCPAINT and WM_ERASEBKGND need this, too...
+    break;
 
+  case WM_PAINT: {
+    Fl_Region R;
     Fl_X *i = Fl_X::i(window);
     i->wait_for_expose = 0;
-    // We need to merge this damage into fltk's damage.  I do this in
-    // reverse, telling Win32 about fltk's damage and then reading back
-    // the new accumulated region.
-    if (window->damage()) {
-      // If there is no region the entire window is damaged
-      if (i->region) {
-	InvalidateRgn(hWnd,i->region,FALSE);
-	GetUpdateRgn(hWnd,i->region,0);
-      }
+    if (!i->region && window->damage()) {
+      // Redraw the whole window...
+      i->region = CreateRectRgn(0, 0, window->w(), window->h());
     } else {
-      if (!i->region) i->region = CreateRectRgn(0,0,0,0);
-      GetUpdateRgn(hWnd,i->region,0);
+      // We need to merge WIN32's damage into FLTK's damage.
+      R = CreateRectRgn(0,0,0,0);
+      GetUpdateRgn(hWnd,R,0);
+
+      if (i->region) {
+        // Also tell WIN32 that we are drawing someplace else as well...
+        InvalidateRgn(hWnd, i->region, FALSE);
+        CombineRgn(i->region, i->region, R, RGN_OR);
+        XDestroyRegion(R);
+      } else {
+        i->region = R;
+      }
     }
-    window->clear_damage(window->damage()|FL_DAMAGE_EXPOSE);
+    window->clear_damage((uchar)(window->damage()|FL_DAMAGE_EXPOSE));
     // These next two statements should not be here, so that all update
-    // is deferred until Fl::flush() is called during idle.  However Win32
+    // is deferred until Fl::flush() is called during idle.  However WIN32
     // apparently is very unhappy if we don't obey it and draw right now.
     // Very annoying!
+    fl_GetDC(hWnd); // Make sure we have a DC for this window...
+    fl_save_pen();
     i->flush();
+    fl_restore_pen();
+    ValidateRgn(hWnd,i->region);
     window->clear_damage();
-    // This convinces MSWindows we have painted whatever they wanted
-    // us to paint, and stops it from sending WM_PAINT messages:
-    ValidateRgn(hWnd,NULL);
-    } break;
+    } return 0;
 
   case WM_LBUTTONDOWN:  mouse_event(window, 0, 1, wParam, lParam); return 0;
   case WM_LBUTTONDBLCLK:mouse_event(window, 1, 1, wParam, lParam); return 0;
@@ -483,13 +626,24 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
   case WM_RBUTTONDOWN:  mouse_event(window, 0, 3, wParam, lParam); return 0;
   case WM_RBUTTONDBLCLK:mouse_event(window, 1, 3, wParam, lParam); return 0;
   case WM_RBUTTONUP:    mouse_event(window, 2, 3, wParam, lParam); return 0;
-  case WM_MOUSEMOVE:    mouse_event(window, 3, 0, wParam, lParam); return 0;
 
-#ifdef WM_MOUSELEAVE
+  case WM_MOUSEMOVE:
+#ifdef USE_TRACK_MOUSE
+    if (Fl::belowmouse() != window) {
+      TRACKMOUSEEVENT tme;
+      tme.cbSize    = sizeof(TRACKMOUSEEVENT);
+      tme.dwFlags   = TME_LEAVE;
+      tme.hwndTrack = hWnd;
+      _TrackMouseEvent(&tme);
+    }
+#endif // USE_TRACK_MOUSE
+    mouse_event(window, 3, 0, wParam, lParam);
+    return 0;
+
   case WM_MOUSELEAVE:
-    Fl::handle(FL_LEAVE, window);
+    Fl::belowmouse(0);
+    if (!window->parent()) Fl::handle(FL_LEAVE, window);
     break;
-#endif /* WM_MOUSELEAVE */
 
   case WM_SETFOCUS:
     Fl::handle(FL_FOCUS, window);
@@ -506,6 +660,27 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
     }
     break;
 
+  case WM_ACTIVATEAPP:
+    // From eric@vfx.sel.sony.com, we should process WM_ACTIVATEAPP
+    // messages to restore the correct state of the shift/ctrl/alt/lock
+    // keys...  Added control, shift, alt, and meta keys, and changed
+    // to use GetAsyncKeyState and do it when wParam is 1
+    // (that means we have focus...)
+    if (wParam)
+    {
+      ulong state = 0;
+      if (GetAsyncKeyState(VK_CAPITAL)) state |= FL_CAPS_LOCK;
+      if (GetAsyncKeyState(VK_NUMLOCK)) state |= FL_NUM_LOCK;
+      if (GetAsyncKeyState(VK_SCROLL)) state |= FL_SCROLL_LOCK;
+      if (GetAsyncKeyState(VK_CONTROL)&~1) state |= FL_CTRL;
+      if (GetAsyncKeyState(VK_SHIFT)&~1) state |= FL_SHIFT;
+      if (GetAsyncKeyState(VK_MENU)) state |= FL_ALT;
+      if ((GetAsyncKeyState(VK_LWIN)|GetAsyncKeyState(VK_RWIN))&~1) state |= FL_META;
+      Fl::e_state = state;
+      return 0;
+    }
+    break;
+
   case WM_KEYDOWN:
   case WM_SYSKEYDOWN:
   case WM_KEYUP:
@@ -513,7 +688,7 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
     // save the keysym until we figure out the characters:
     Fl::e_keysym = ms2fltk(wParam,lParam&(1<<24));
     // See if TranslateMessage turned it into a WM_*CHAR message:
-    if (PeekMessage(&fl_msg, hWnd, WM_CHAR, WM_SYSDEADCHAR, 1)) {
+    if (PeekMessage(&fl_msg, hWnd, WM_CHAR, WM_SYSDEADCHAR, PM_REMOVE)) {
       uMsg = fl_msg.message;
       wParam = fl_msg.wParam;
       lParam = fl_msg.lParam;
@@ -532,7 +707,7 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
     if ((lParam&(1<<29)) //same as GetKeyState(VK_MENU)
 	&& uMsg != WM_CHAR) state |= FL_ALT;
     if (GetKeyState(VK_NUMLOCK)) state |= FL_NUM_LOCK;
-    if (GetKeyState(VK_LWIN)&~1 || GetKeyState(VK_RWIN)&~1) {
+    if ((GetKeyState(VK_LWIN)|GetKeyState(VK_RWIN))&~1) {
       // WIN32 bug?  GetKeyState returns garbage if the user hit the
       // meta key to pop up start menu.  Sigh.
       if ((GetAsyncKeyState(VK_LWIN)|GetAsyncKeyState(VK_RWIN))&~1)
@@ -540,7 +715,10 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
     }
     if (GetKeyState(VK_SCROLL)) state |= FL_SCROLL_LOCK;
     Fl::e_state = state;
-    if (lParam & (1<<31)) goto DEFAULT; // ignore up events after fixing shift
+    if (lParam & (1<<31)) { // key up events.
+      if (Fl::handle(FL_KEYUP, window)) return 0;
+      break;
+    }
     static char buffer[2];
     if (uMsg == WM_CHAR || uMsg == WM_SYSCHAR) {
       buffer[0] = char(wParam);
@@ -558,6 +736,15 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
     if (Fl::handle(FL_KEYBOARD,window)) return 0;
     break;}
 
+  case WM_MOUSEWHEEL: {
+    static int delta = 0; // running total of all motion
+    delta += (SHORT)(HIWORD(wParam));
+    Fl::e_dy = -delta / WHEEL_DELTA;
+    delta += Fl::e_dy * WHEEL_DELTA;
+    if (Fl::e_dy) Fl::handle(FL_MOUSEWHEEL, window);
+    return 0;
+  }
+
   case WM_GETMINMAXINFO:
     Fl_X::i(window)->set_minmax((LPMINMAXINFO)lParam);
     break;
@@ -602,11 +789,35 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
     fl_GetDC(hWnd);
     fl_select_palette();
     break;
-
 #endif
 
+  case WM_DESTROYCLIPBOARD:
+    fl_i_own_selection[1] = 0;
+    return 1;
+
+  case WM_RENDERALLFORMATS:
+    fl_i_own_selection[1] = 0;
+    // Windoze seems unhappy unless I do these two steps. Documentation
+    // seems to vary on whether opening the clipboard is necessary or
+    // is in fact wrong:
+    CloseClipboard();
+    OpenClipboard(NULL);
+    // fall through...
+  case WM_RENDERFORMAT: {
+    HANDLE h = GlobalAlloc(GHND, fl_selection_length[1]+1);
+    if (h) {
+      LPSTR p = (LPSTR)GlobalLock(h);
+      memcpy(p, fl_selection_buffer[1], fl_selection_length[1]);
+      p[fl_selection_length[1]] = 0;
+      GlobalUnlock(h);
+      SetClipboardData(CF_TEXT, h);
+    }
+    // Windoze also seems unhappy if I don't do this. Documentation very
+    // unclear on what is correct:
+    if (fl_msg.message == WM_RENDERALLFORMATS) CloseClipboard();
+    return 1;}
+
   default:
-  DEFAULT:
     if (Fl::handle(0,0)) return 0;
     break;
   }
@@ -629,7 +840,7 @@ int Fl_X::fake_X_wm(const Fl_Window* w,int &X,int &Y, int &bt,int &bx, int &by)
   int W, H, xoff, yoff, dx, dy;
   int ret = bx = by = bt = 0;
   if (w->border() && !w->parent()) {
-    if (w->maxw != w->minw || w->maxh != w->minh) {
+    if (w->size_range_set && (w->maxw != w->minw || w->maxh != w->minh)) {
       ret = 2;
       bx = GetSystemMetrics(SM_CXSIZEFRAME);
       by = GetSystemMetrics(SM_CYSIZEFRAME);
@@ -677,8 +888,12 @@ void Fl_Window::resize(int X,int Y,int W,int H) {
   int is_a_resize = (W != w() || H != h());
   int resize_from_program = (this != resize_bug_fix);
   if (!resize_from_program) resize_bug_fix = 0;
-  if (X != x() || Y != y()) set_flag(FL_FORCE_POSITION);
-    else {if (!is_a_resize) return; flags |= SWP_NOMOVE;}
+  if (X != x() || Y != y()) {
+    set_flag(FL_FORCE_POSITION);
+  } else {
+    if (!is_a_resize) return;
+    flags |= SWP_NOMOVE;
+  }
   if (is_a_resize) {
     Fl_Group::resize(X,Y,W,H);
     if (shown()) {redraw(); i->wait_for_expose = 1;}
@@ -686,7 +901,9 @@ void Fl_Window::resize(int X,int Y,int W,int H) {
     x(X); y(Y);
     flags |= SWP_NOSIZE;
   }
+  if (!border()) flags |= SWP_NOACTIVATE;
   if (resize_from_program && shown()) {
+    if (!resizable()) size_range(w(),h(),w(),h());
     int dummy, bt, bx, by;
     //Ignore window managing when resizing, so that windows (and more
     //specifically menus) can be moved offscreen.
@@ -702,11 +919,12 @@ void Fl_Window::resize(int X,int Y,int W,int H) {
 
 ////////////////////////////////////////////////////////////////
 
-void fl_fix_focus(); // in Fl.C
+void fl_fix_focus(); // in Fl.cxx
 
 char fl_show_iconic;	// hack for Fl_Window::iconic()
 // int fl_background_pixel = -1; // color to use for background
 HCURSOR fl_default_cursor;
+UINT fl_wake_msg = 0;
 int fl_disable_transient_for; // secret method of removing TRANSIENT_FOR
 
 Fl_X* Fl_X::make(Fl_Window* w) {
@@ -715,6 +933,8 @@ Fl_X* Fl_X::make(Fl_Window* w) {
   const char* class_name = /*w->xclass();
   if (!class_name) class_name =*/ "FLTK"; // create a "FLTK" WNDCLASS
 
+  const char* message_name = "FLTK::ThreadWakeup";
+
   WNDCLASSEX wc;
   // Documentation states a device context consumes about 800 bytes
   // of memory... so who cares? If 800 bytes per window is what it
@@ -735,6 +955,7 @@ Fl_X* Fl_X::make(Fl_Window* w) {
   wc.lpszClassName = class_name;
   wc.cbSize = sizeof(WNDCLASSEX);
   RegisterClassEx(&wc);
+  if (!fl_wake_msg) fl_wake_msg = RegisterWindowMessage(message_name);
 
   HWND parent;
   DWORD style = WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
@@ -765,8 +986,10 @@ Fl_X* Fl_X::make(Fl_Window* w) {
     styleEx |= WS_EX_WINDOWEDGE | WS_EX_CONTROLPARENT;
     int xwm = xp , ywm = yp , bt, bx, by;
     switch (fake_X_wm(w, xwm, ywm, bt, bx, by)) {
-      // No border (user for menus)
-      case 0: style |= WS_POPUP; break;
+      // No border (used for menus)
+      case 0: style |= WS_POPUP;
+              styleEx |= WS_EX_TOOLWINDOW;
+	      break;
 
       // Thin border and title bar
       case 1: style |= WS_DLGFRAME | WS_CAPTION; break;
@@ -797,7 +1020,7 @@ Fl_X* Fl_X::make(Fl_Window* w) {
       while (w->parent()) w = w->window();
       parent = fl_xid(w);
       if (!w->visible()) showit = 0;
-    }
+    } else if (Fl::grab()) parent = fl_xid(Fl::grab());
   }
 
   Fl_X* x = new Fl_X;
@@ -828,7 +1051,17 @@ Fl_X* Fl_X::make(Fl_Window* w) {
   // If we've captured the mouse, we dont want do activate any
   // other windows from the code, or we loose the capture.
   ShowWindow(x->xid, !showit ? SW_SHOWMINNOACTIVE :
-	     fl_capture? SW_SHOWNOACTIVATE : SW_SHOWNORMAL);
+	     (Fl::grab() || (style & WS_POPUP)) ? SW_SHOWNOACTIVATE : SW_SHOWNORMAL);
+
+  // Drag-n-drop requires GCC 3.x or a non-GNU compiler...
+#if !defined(__GNUC__) || __GNUC__ >= 3
+  // Register all windows for potential drag'n'drop operations
+  static char oleInitialized = 0;
+  if (!oleInitialized) { OleInitialize(0L); oleInitialized=1; }
+
+  RegisterDragDrop(x->xid, flIDropTarget);
+#endif // !__GNUC__ || __GNUC__ >= 3
+
   if (w->modal()) {Fl::modal_ = w; fl_fix_focus();}
   return x;
 }
@@ -864,11 +1097,12 @@ void Fl_X::set_minmax(LPMINMAXINFO minmax)
 
 ////////////////////////////////////////////////////////////////
 
-#include <FL/filename.H> // need so FL_EXPORT filename_name works
+#include <FL/filename.H> // need so FL_EXPORT fl_filename_name works
 
 // returns pointer to the filename, or null if name ends with '/'
-const char *filename_name(const char *name) {
+const char *fl_filename_name(const char *name) {
   const char *p,*q;
+  if (!name) return (0);
   q = name;
   if (q[0] && q[1]==':') q += 2; // skip leading drive letter
   for (p = q; *p; p++) if (*p == '/' || *p == '\\') q = p+1;
@@ -881,7 +1115,7 @@ void Fl_Window::label(const char *name,const char *iname) {
   if (shown() && !parent()) {
     if (!name) name = "";
     SetWindowText(i->xid, name);
-    // if (!iname) iname = filename_name(name);
+    // if (!iname) iname = fl_filename_name(name);
     // should do something with iname here...
   }
 }
@@ -892,7 +1126,7 @@ void Fl_Window::label(const char *name,const char *iname) {
 // If the box is a filled rectangle, we can make the redisplay *look*
 // faster by using X's background pixel erasing.  We can make it
 // actually *be* faster by drawing the frame only, this is done by
-// setting fl_boxcheat, which is seen by code in fl_drawbox.C:
+// setting fl_boxcheat, which is seen by code in fl_drawbox.cxx:
 // For WIN32 it looks like all windows share a background color, so
 // I use FL_GRAY for this and only do this cheat for windows that are
 // that color.
@@ -901,6 +1135,13 @@ void Fl_Window::label(const char *name,const char *iname) {
 //static inline int can_boxcheat(uchar b) {return (b==1 || (b&2) && b<=15);}
 
 void Fl_Window::show() {
+  image(Fl::scheme_bg_);
+  if (Fl::scheme_bg_) {
+    labeltype(FL_NORMAL_LABEL);
+    align(FL_ALIGN_CENTER | FL_ALIGN_INSIDE | FL_ALIGN_CLIP);
+  } else {
+    labeltype(FL_NO_LABEL);
+  }
   if (!shown()) {
     // if (can_boxcheat(box())) fl_background_pixel = fl_xpixel(color());
     Fl_X::make(this);
@@ -951,5 +1192,5 @@ void Fl_Window::make_current() {
 }
 
 //
-// End of "$Id: Fl_win32.cxx,v 1.33.2.30 2000/11/20 14:17:43 easysw Exp $".
+// End of "$Id: Fl_win32.cxx,v 1.33.2.37.2.44 2003/01/30 21:43:11 easysw Exp $".
 //
