@@ -1,9 +1,9 @@
 //
-// "$Id: fl_draw_pixmap.cxx,v 1.4.2.6 2000/06/05 21:21:07 mike Exp $"
+// "$Id: fl_draw_pixmap.cxx,v 1.4.2.8.2.12 2003/01/30 21:43:43 easysw Exp $"
 //
 // Pixmap drawing code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -33,18 +33,20 @@
 // as I want to discourage programs that require support files to work.
 // All data needed by a program ui should be compiled in!!!
 
-#include <config.h>
 #include <FL/Fl.H>
 #include <FL/fl_draw.H>
 #include <FL/x.H>
-#include <ctype.h>
 #include <stdio.h>
-#include <string.h>
+#include "flstring.h"
 
 static int ncolors, chars_per_pixel;
 
-int fl_measure_pixmap(/*const*/char * const *data, int &w, int &h) {
-  int i = sscanf(data[0],"%d %d %d %d",&w,&h,&ncolors,&chars_per_pixel);
+int fl_measure_pixmap(/*const*/ char* const* data, int &w, int &h) {
+  return fl_measure_pixmap((const char*const*)data,w,h);
+}
+
+int fl_measure_pixmap(const char * const *data, int &w, int &h) {
+  int i = sscanf(data[0],"%d%d%d%d",&w,&h,&ncolors,&chars_per_pixel);
   if (i<4 || w<=0 || h<=0 ||
       chars_per_pixel!=1 && chars_per_pixel!=2) return w=0;
   return 1;
@@ -68,11 +70,11 @@ static void cb1(void*v, int x, int y, int w, uchar* buf) {
   const uchar* p = d.data[y]+x;
   U64* q = (U64*)buf;
   for (int X=(w+1)/2; X--; p += 2) {
-#if WORDS_BIGENDIAN
+#  if WORDS_BIGENDIAN
     *q++ = (d.colors[p[0]]<<32) | d.colors[p[1]];
-#else
+#  else
     *q++ = (d.colors[p[1]]<<32) | d.colors[p[0]];
-#endif
+#  endif
   }
 }
 
@@ -86,11 +88,11 @@ static void cb2(void*v, int x, int y, int w, uchar* buf) {
     int index = *p++;
     U64* colors1 = d.byte1[*p++];
     int index1 = *p++;
-#if WORDS_BIGENDIAN
+#  if WORDS_BIGENDIAN
     *q++ = (colors[index]<<32) | colors1[index1];
-#else
+#  else
     *q++ = (colors1[index1]<<32) | colors[index];
-#endif
+#  endif
   }
 }
 
@@ -127,20 +129,19 @@ static void cb2(void*v, int x, int y, int w, uchar* buf) {
 
 #endif
 
-#ifdef WIN32
-// this is in Fl_arg.C:
-extern int fl_parse_color(const char*, uchar&, uchar&, uchar&);
-#endif
-
 uchar **fl_mask_bitmap; // if non-zero, create bitmap and store pointer here
 
-int fl_draw_pixmap(/*const*/char*const* di, int x, int y, Fl_Color bg) {
+int fl_draw_pixmap(/*const*/ char* const* data, int x,int y,Fl_Color bg) {
+  return fl_draw_pixmap((const char*const*)data,x,y,bg);
+}
+
+int fl_draw_pixmap(const char*const* di, int x, int y, Fl_Color bg) {
   pixmap_data d;
   if (!fl_measure_pixmap(di, d.w, d.h)) return 0;
   const uchar*const* data = (const uchar*const*)(di+1);
   int transparent_index = -1;
 
-  if (ncolors < 0) {	// fltk (non standard) compressed colormap
+  if (ncolors < 0) {	// FLTK (non standard) compressed colormap
     ncolors = -ncolors;
     const uchar *p = *data++;
     // if first color is ' ' it is transparent (put it later to make
@@ -149,9 +150,9 @@ int fl_draw_pixmap(/*const*/char*const* di, int x, int y, Fl_Color bg) {
       uchar* c = (uchar*)&d.colors[' '];
 #ifdef U64
       *(U64*)c = 0;
-#if WORDS_BIGENDIAN
+#  if WORDS_BIGENDIAN
       c += 4;
-#endif
+#  endif
 #endif
       transparent_index = ' ';
       Fl::get_color(bg, c[0], c[1], c[2]); c[3] = 0;
@@ -163,9 +164,9 @@ int fl_draw_pixmap(/*const*/char*const* di, int x, int y, Fl_Color bg) {
       uchar* c = (uchar*)&d.colors[*p++];
 #ifdef U64
       *(U64*)c = 0;
-#if WORDS_BIGENDIAN
+#  if WORDS_BIGENDIAN
       c += 4;
-#endif
+#  endif
 #endif
       *c++ = *p++;
       *c++ = *p++;
@@ -177,25 +178,26 @@ int fl_draw_pixmap(/*const*/char*const* di, int x, int y, Fl_Color bg) {
     for (int i=0; i<ncolors; i++) {
       const uchar *p = *data++;
       // the first 1 or 2 characters are the color index:
-      int index = *p++;
+      int ind = *p++;
       uchar* c;
       if (chars_per_pixel>1) {
 #ifdef U64
-	U64* colors = d.byte1[index];
-	if (!colors) colors = d.byte1[index] = new U64[256];
+	U64* colors = d.byte1[ind];
+	if (!colors) colors = d.byte1[ind] = new U64[256];
 #else
-	U32* colors = d.byte1[index];
-	if (!colors) colors = d.byte1[index] = new U32[256];
+	U32* colors = d.byte1[ind];
+	if (!colors) colors = d.byte1[ind] = new U32[256];
 #endif
 	c = (uchar*)&colors[*p];
-	index = (index<<8)+*p++;
+	ind = (ind<<8)+*p++;
       } else {
-	c = (uchar *)&d.colors[index];
+	c = (uchar *)&d.colors[ind];
       }
       // look for "c word", or last word if none:
       const uchar *previous_word = p;
       for (;;) {
-	while (*p && isspace(*p)) p++; uchar what = *p++;
+	while (*p && isspace(*p)) p++;
+	uchar what = *p++;
 	while (*p && !isspace(*p)) p++;
 	while (*p && isspace(*p)) p++;
 	if (!*p) {p = previous_word; break;}
@@ -205,21 +207,15 @@ int fl_draw_pixmap(/*const*/char*const* di, int x, int y, Fl_Color bg) {
       }
 #ifdef U64
       *(U64*)c = 0;
-#if WORDS_BIGENDIAN
+#  if WORDS_BIGENDIAN
       c += 4;
+#  endif
 #endif
-#endif
-#ifdef WIN32
-      if (fl_parse_color((const char*)p, c[0], c[1], c[2])) {;
-#else
-      XColor x;
-      if (XParseColor(fl_display, fl_colormap, (const char*)p, &x)) {
-	c[0] = x.red>>8; c[1] = x.green>>8; c[2] = x.blue>>8;
-#endif
-      } else { // assumme "None" or "#transparent" for any errors
-	// this should be transparent...
+      if (!fl_parse_color((const char*)p, c[0], c[1], c[2])) {
+        // assume "None" or "#transparent" for any errors
+	// "bg" should be transparent...
 	Fl::get_color(bg, c[0], c[1], c[2]);
-	transparent_index = index;
+	transparent_index = ind;
       }
     }
   }
@@ -230,11 +226,11 @@ int fl_draw_pixmap(/*const*/char*const* di, int x, int y, Fl_Color bg) {
     int W = (d.w+7)/8;
     uchar* bitmap = new uchar[W * d.h];
     *fl_mask_bitmap = bitmap;
-    for (int y = 0; y < d.h; y++) {
-      const uchar* p = data[y];
+    for (int Y = 0; Y < d.h; Y++) {
+      const uchar* p = data[Y];
       if (chars_per_pixel <= 1) {
-	for (int x = 0; x < W; x++) {
-	  int b = (*p++ != transparent_index);
+	for (int X = 0; X < W; X++) {
+	  uchar b = (*p++ != transparent_index);
 	  if (*p++ != transparent_index) b |= 2;
 	  if (*p++ != transparent_index) b |= 4;
 	  if (*p++ != transparent_index) b |= 8;
@@ -245,12 +241,12 @@ int fl_draw_pixmap(/*const*/char*const* di, int x, int y, Fl_Color bg) {
 	  *bitmap++ = b;
 	}
       } else {
-	for (int x = 0; x < W; x++) {
-	  int b = 0;
+	for (int X = 0; X < W; X++) {
+	  uchar b = 0;
 	  for (int i = 0; i < 8; i++) {
-	    int index = *p++;
-	    index = (index<<8) | (*p++);
-	    if (index != transparent_index) b |= (1<<i);
+	    int ind = *p++;
+	    ind = (ind<<8) | (*p++);
+	    if (ind != transparent_index) b |= (1<<i);
 	  }
 	  *bitmap++ = b;
 	}
@@ -259,10 +255,10 @@ int fl_draw_pixmap(/*const*/char*const* di, int x, int y, Fl_Color bg) {
   }
 
   fl_draw_image(chars_per_pixel==1 ? cb1 : cb2, &d, x, y, d.w, d.h, 4);
-  if (chars_per_pixel > 1) for (int i = 0; i < 256; i++) delete d.byte1[i];
+  if (chars_per_pixel > 1) for (int i = 0; i < 256; i++) delete[] d.byte1[i];
   return 1;
 }
 
 //
-// End of "$Id: fl_draw_pixmap.cxx,v 1.4.2.6 2000/06/05 21:21:07 mike Exp $".
+// End of "$Id: fl_draw_pixmap.cxx,v 1.4.2.8.2.12 2003/01/30 21:43:43 easysw Exp $".
 //
