@@ -1,9 +1,9 @@
 //
-// "$Id: fl_shortcut.cxx,v 1.4.2.8 2000/06/15 05:37:40 bill Exp $"
+// "$Id: fl_shortcut.cxx,v 1.4.2.9.2.9 2003/01/30 21:44:21 easysw Exp $"
 //
 // Shortcut support routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -44,8 +44,8 @@
 #include <FL/Fl_Button.H>
 #include <FL/fl_draw.H>
 #include <ctype.h>
-#include <string.h>
-#ifndef WIN32
+#include "flstring.h"
+#if !defined(WIN32) && !defined(__APPLE__)
 #include <FL/x.H>
 #endif
 
@@ -74,7 +74,7 @@ int Fl::test_shortcut(int shortcut) {
   return 0;
 }
 
-#ifdef WIN32 // if not X
+#if defined(WIN32) || defined(__APPLE__) // if not X
 // This table must be in numeric order by fltk (X) keysym number:
 struct Keyname {int key; const char* name;};
 static Keyname table[] = {
@@ -115,12 +115,20 @@ const char * fl_shortcut_label(int shortcut) {
   static char buf[20];
   char *p = buf;
   if (!shortcut) {*p = 0; return buf;}
+#ifdef UNUSED__APPLE__UNUSED
+  // \todo Mac :  we might want to change the symbols for Mac users - consider drawing Apple Symbols... .
+  if (shortcut & FL_SHIFT) {strcpy(p,"shift+"); p += 6;} //: Mac hollow up arrow
+  if (shortcut & FL_META)  {strcpy(p,"ctrl+"); p += 5;}  //: Mac 'cotrol'
+  if (shortcut & FL_ALT)   {strcpy(p,"option+"); p += 7;}   //: Mac 'Option' or fancy switch symbol
+  if (shortcut & FL_CTRL)  {strcpy(p,"cmd+"); p += 4;}  //: Mac Apple or Curlyflour
+#else
   if (shortcut & FL_META) {strcpy(p,"Meta+"); p += 5;}
   if (shortcut & FL_ALT) {strcpy(p,"Alt+"); p += 4;}
   if (shortcut & FL_SHIFT) {strcpy(p,"Shift+"); p += 6;}
   if (shortcut & FL_CTRL) {strcpy(p,"Ctrl+"); p += 5;}
+#endif // __APPLE__
   int key = shortcut & 0xFFFF;
-#ifdef WIN32 // if not X
+#if defined(WIN32) || defined(__APPLE__) // if not X
   if (key >= FL_F && key <= FL_F_Last) {
     *p++ = 'F';
     if (key > FL_F+9) *p++ = (key-FL_F)/10+'0';
@@ -173,14 +181,14 @@ int fl_old_shortcut(const char* s) {
 
 // Tests for &x shortcuts in button labels:
 
-int Fl_Widget::test_shortcut(const char *label) {
+int Fl_Widget::test_shortcut(const char *l) {
   char c = Fl::event_text()[0];
-  if (!c || !label) return 0;
+  if (!c || !l) return 0;
   for (;;) {
-    if (!*label) return 0;
-    if (*label++ == '&' && *label) {
-      if (*label == '&') label++;
-      else if (*label == c) return 1;
+    if (!*l) return 0;
+    if (*l++ == '&' && *l) {
+      if (*l == '&') l++;
+      else if (*l == c) return 1;
       else return 0;
     }
   }
@@ -192,5 +200,5 @@ int Fl_Widget::test_shortcut() {
 }
 
 //
-// End of "$Id: fl_shortcut.cxx,v 1.4.2.8 2000/06/15 05:37:40 bill Exp $".
+// End of "$Id: fl_shortcut.cxx,v 1.4.2.9.2.9 2003/01/30 21:44:21 easysw Exp $".
 //
