@@ -1,9 +1,9 @@
 //
-// "$Id: fl_symbols.cxx,v 1.8.2.2 2000/06/05 21:21:10 mike Exp $"
+// "$Id: fl_symbols.cxx,v 1.8.2.3.2.5 2003/01/30 21:44:25 easysw Exp $"
 //
 // Symbol drawing code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -34,7 +34,7 @@
 
 #include <FL/Fl.H>
 #include <FL/fl_draw.H>
-#include <string.h>
+#include "flstring.h"
 
 typedef struct {
   const char *name;
@@ -92,7 +92,7 @@ int fl_add_symbol(const char *name, void (*drawit)(Fl_Color), int scalable)
 int fl_return_arrow(int x,int y,int w,int h);
 
 // provided for back compatability:
-int fl_draw_symbol(const char *label,int x,int y,int w,int h,Fl_Color col) {
+int fl_draw_symbol(const char *label,int x,int y,int w,int h,Fl_Color col) {  
   const char *p = label;
   if (*p++ != '@') return 0;
   fl_init_symbols();
@@ -159,7 +159,7 @@ int fl_draw_symbol(const char *label,int x,int y,int w,int h,Fl_Color col) {
 
 //for the outline color
 static void set_outline_color(Fl_Color c) {
-  fl_color(fl_color_average(c, FL_BLACK, .5));
+  fl_color(fl_color_average(c, FL_BLACK, 0.5f));
 }
 
 static void rectangle(double x,double y,double x2,double y2,Fl_Color col) {
@@ -360,28 +360,6 @@ static void fl_init_symbols(void) {
   fl_add_symbol("||",		draw_doublebar,		1);
 }
 
-////////////////////////////////////////////////////////////////
-
-#include <FL/Fl_Widget.H>
-
-// this is the labeltype function:
-extern void fl_normal_label(const Fl_Label*, int, int, int, int, Fl_Align);
-static void fl_symbol_label(
-    const Fl_Label* o, int x, int y, int w, int h, Fl_Align align)
-{
-  if (!fl_draw_symbol(o->value, x, y, w, h, (Fl_Color)o->color))
-    fl_normal_label(o, x, y, w, h, align);
-}
-
-Fl_Labeltype define_FL_SYMBOL_LABEL() {
-  Fl::set_labeltype(_FL_SYMBOL_LABEL, fl_symbol_label, 0);
-  return _FL_SYMBOL_LABEL;
-}
-
-void Fl::enable_symbols() {
-  Fl::set_labeltype(FL_NORMAL_LABEL, fl_symbol_label, 0);
-}
-
 //
-// End of "$Id: fl_symbols.cxx,v 1.8.2.2 2000/06/05 21:21:10 mike Exp $".
+// End of "$Id: fl_symbols.cxx,v 1.8.2.3.2.5 2003/01/30 21:44:25 easysw Exp $".
 //
