@@ -1,9 +1,9 @@
 //
-// "$Id: gl_draw.cxx,v 1.7.2.3 2000/06/05 21:21:12 mike Exp $"
+// "$Id: gl_draw.cxx,v 1.7.2.5.2.10 2003/01/30 21:44:33 easysw Exp $"
 //
 // OpenGL drawing support routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -24,9 +24,9 @@
 //
 
 // Functions from <FL/gl.h>
-// See also Fl_Gl_Window and gl_start.C
+// See also Fl_Gl_Window and gl_start.cxx
 
-#include <config.h>
+#include "flstring.h"
 #if HAVE_GL
 
 #include <FL/Fl.H>
@@ -35,36 +35,40 @@
 #include <FL/fl_draw.H>
 #include "Fl_Gl_Choice.H"
 #include "Fl_Font.H"
-#include <string.h>
 
-void  gl_font(int fontid, int size) {fl_font(fontid, size);}
 int   gl_height() {return fl_height();}
 int   gl_descent() {return fl_descent();}
 double gl_width(const char* s) {return fl_width(s);}
 double gl_width(const char* s, int n) {return fl_width(s,n);}
 double gl_width(uchar c) {return fl_width(c);}
 
-void gl_draw(const char* str, int n) {
+void  gl_font(int fontid, int size) {
+  fl_font(fontid, size);
   if (!fl_fontsize->listbase) {
 #ifdef WIN32
     int base = fl_fontsize->metr.tmFirstChar;
-    int size = fl_fontsize->metr.tmLastChar-base+1;
+    int count = fl_fontsize->metr.tmLastChar-base+1;
     HFONT oldFid = (HFONT)SelectObject(fl_gc, fl_fontsize->fid);
     fl_fontsize->listbase = glGenLists(256);
-    wglUseFontBitmaps(fl_gc, base, size, fl_fontsize->listbase+base); 
+    wglUseFontBitmaps(fl_gc, base, count, fl_fontsize->listbase+base); 
     SelectObject(fl_gc, oldFid);
+#elif defined(__APPLE__)
+    // undefined characters automatically receive an empty GL list in aglUseFont
+    fl_fontsize->listbase = glGenLists(256);
+    aglUseFont(aglGetCurrentContext(), fl_fontsize->font, fl_fontsize->face,
+               fl_fontsize->size, 0, 256, fl_fontsize->listbase);
 #else
     int base = fl_xfont->min_char_or_byte2;
-    int size = fl_xfont->max_char_or_byte2-base+1;
+    int count = fl_xfont->max_char_or_byte2-base+1;
     fl_fontsize->listbase = glGenLists(256);
-    glXUseXFont(fl_xfont->fid, base, size, fl_fontsize->listbase+base);
+    glXUseXFont(fl_xfont->fid, base, count, fl_fontsize->listbase+base);
 #endif
   }
+  glListBase(fl_fontsize->listbase);
+}
 
-  glPushAttrib(GL_LIST_BIT);
-    glListBase(fl_fontsize->listbase);
-    glCallLists(n, GL_UNSIGNED_BYTE, str);
-  glPopAttrib();
+void gl_draw(const char* str, int n) {
+  glCallLists(n, GL_UNSIGNED_BYTE, str);
 }
 
 void gl_draw(const char* str, int n, int x, int y) {
@@ -155,5 +159,5 @@ void gl_draw_image(const uchar* b, int x, int y, int w, int h, int d, int ld) {
 #endif
 
 //
-// End of "$Id: gl_draw.cxx,v 1.7.2.3 2000/06/05 21:21:12 mike Exp $".
+// End of "$Id: gl_draw.cxx,v 1.7.2.5.2.10 2003/01/30 21:44:33 easysw Exp $".
 //
