@@ -1,9 +1,9 @@
 //
-// "$Id: boxtype.cxx,v 1.4.2.2 2000/06/05 21:21:15 mike Exp $"
+// "$Id: boxtype.cxx,v 1.4.2.3.2.3 2003/01/30 21:44:56 easysw Exp $"
 //
 // Boxtype test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -32,7 +32,7 @@
 int N = 0;
 #define W 150
 #define H 50
-#define ROWS 8
+#define ROWS 9
 
 Fl_Window *window;
 
@@ -82,6 +82,10 @@ int main(int argc, char ** argv) {
   bt("FL_ROUND_DOWN_BOX",FL_ROUND_DOWN_BOX);
   bt("FL_DIAMOND_UP_BOX",FL_DIAMOND_UP_BOX);
   bt("FL_DIAMOND_DOWN_BOX",FL_DIAMOND_DOWN_BOX);
+  bt("FL_PLASTIC_UP_BOX",FL_PLASTIC_UP_BOX);
+  bt("FL_PLASTIC_DOWN_BOX",FL_PLASTIC_DOWN_BOX);
+  bt("FL_PLASTIC_UP_FRAME",FL_PLASTIC_UP_FRAME);
+  bt("FL_PLASTIC_DOWN_FRAME",FL_PLASTIC_DOWN_FRAME);
   window->resizable(window);
   window->end();
   window->show(argc,argv);
@@ -89,5 +93,5 @@ int main(int argc, char ** argv) {
 }
 
 //
-// End of "$Id: boxtype.cxx,v 1.4.2.2 2000/06/05 21:21:15 mike Exp $".
+// End of "$Id: boxtype.cxx,v 1.4.2.3.2.3 2003/01/30 21:44:56 easysw Exp $".
 //
