@@ -1,9 +1,9 @@
 //
-// "$Id: input.cxx,v 1.5.2.2 2000/06/05 21:21:21 mike Exp $"
+// "$Id: input.cxx,v 1.5.2.4.2.5 2003/01/30 21:45:45 easysw Exp $"
 //
 // Input field test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -67,7 +67,7 @@ void color_cb(Fl_Widget* button, void* v) {
   uchar r,g,b; Fl::get_color(c, r,g,b);
   if (fl_color_chooser(0,r,g,b)) {
     Fl::set_color(c,r,g,b); Fl::redraw();
-    button->labelcolor(contrast(FL_BLACK,c));
+    button->labelcolor(fl_contrast(FL_BLACK,c));
     button->redraw();
   }
 }
@@ -77,13 +77,19 @@ int main(int argc, char **argv) {
 
   int y = 10;
   input[0] = new Fl_Input(70,y,300,30,"Normal:"); y += 35;
+  input[0]->tooltip("Normal input field");
   // input[0]->cursor_color(FL_SELECTION_COLOR);
   //  input[0]->maximum_size(20);
   // input[0]->static_value("this is a testgarbage");
   input[1] = new Fl_Float_Input(70,y,300,30,"Float:"); y += 35;
+  input[1]->tooltip("Input field for floating-point number");
   input[2] = new Fl_Int_Input(70,y,300,30,"Int:"); y += 35;
+  input[2]->tooltip("Input field for integer number");
   input[3] = new Fl_Secret_Input(70,y,300,30,"Secret:"); y += 35;
+  input[3]->tooltip("Input field for password");
   input[4] = new Fl_Multiline_Input(70,y,300,100,"Multiline:"); y += 105;
+  input[4]->tooltip("Input field for short text with newlines");
+  input[4]->wrap(1);
 
   for (int i = 0; i < 4; i++) {
     input[i]->when(0); input[i]->callback(cb);
@@ -93,23 +99,32 @@ int main(int argc, char **argv) {
   Fl_Button *b;
   b = new Fl_Toggle_Button(10,y,200,25,"FL_WHEN_&CHANGED");
   b->callback(toggle_cb, FL_WHEN_CHANGED); y += 25;
+  b->tooltip("Do callback each time the text changes");
   b = new Fl_Toggle_Button(10,y,200,25,"FL_WHEN_&RELEASE");
   b->callback(toggle_cb, FL_WHEN_RELEASE); y += 25;
+  b->tooltip("Do callback when widget loses focus");
   b = new Fl_Toggle_Button(10,y,200,25,"FL_WHEN_&ENTER_KEY");
   b->callback(toggle_cb, FL_WHEN_ENTER_KEY); y += 25;
+  b->tooltip("Do callback when user hits Enter key");
   b = new Fl_Toggle_Button(10,y,200,25,"FL_WHEN_&NOT_CHANGED");
   b->callback(toggle_cb, FL_WHEN_NOT_CHANGED); y += 25;
+  b->tooltip("Do callback even if the text is not changed");
   y += 5;
   b = new Fl_Button(10,y,200,25,"&print changed()");
   b->callback(button_cb);
+  b->tooltip("Print widgets that have changed() flag set");
 
   b = new Fl_Button(220,y1,100,25,"color"); y1 += 25;
   b->color(input[0]->color()); b->callback(color_cb, (void*)0);
+  b->tooltip("Color behind the text");
   b = new Fl_Button(220,y1,100,25,"selection_color"); y1 += 25;
   b->color(input[0]->selection_color()); b->callback(color_cb, (void*)1);
+  b->labelcolor(fl_contrast(FL_BLACK,b->color()));
+  b->tooltip("Color behind selected text");
   b = new Fl_Button(220,y1,100,25,"textcolor"); y1 += 25;
   b->color(input[0]->textcolor()); b->callback(color_cb, (void*)2);
-  b->labelcolor(contrast(FL_BLACK,b->color()));
+  b->labelcolor(fl_contrast(FL_BLACK,b->color()));
+  b->tooltip("Color of the text");
 
   window->end();
   window->show(argc,argv);
@@ -117,5 +132,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: input.cxx,v 1.5.2.2 2000/06/05 21:21:21 mike Exp $".
+// End of "$Id: input.cxx,v 1.5.2.4.2.5 2003/01/30 21:45:45 easysw Exp $".
 //
