@@ -1,9 +1,9 @@
 //
-// "$Id: menubar.cxx,v 1.6.2.8 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: menubar.cxx,v 1.6.2.8.2.6 2003/01/30 21:45:53 easysw Exp $"
 //
 // Menubar test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -35,6 +35,11 @@
 #include <string.h>
 #include <FL/fl_draw.H>
 
+void window_cb(Fl_Widget* w, void*) {
+  puts("window callback called");
+  ((Fl_Window *)w)->hide();
+}
+
 void test_cb(Fl_Widget* w, void*) {
   Fl_Menu_* mw = (Fl_Menu_*)w;
   const Fl_Menu_Item* m = mw->mvalue();
@@ -164,24 +169,6 @@ Fl_Menu_Item pulldown[] = {
 
 Fl_Menu_* menus[4];
 
-// turn MicroSoft style on/off
-void button_cb(Fl_Widget* w, void*) {
-  if (((Fl_Button*)w)->value()) {
-    for (int i = 0; i < 4; i++) {
-      menus[i]->down_box(FL_FLAT_BOX);
-      menus[i]->selection_color(137);
-      menus[i]->textfont(FL_HELVETICA);
-    }
-  } else {
-    for (int i = 0; i < 4; i++) {
-      menus[i]->down_box(FL_NO_BOX);
-      menus[i]->selection_color(FL_WHITE);
-      menus[i]->textfont(FL_BOLD|FL_ITALIC);
-    }
-  }
-  menus[0]->parent()->redraw();
-}
-
 int main(int argc, char **argv) {
   //Fl::set_color(Fl_Color(15),0,0,128);
   for (int i=0; i<99; i++) {
@@ -190,6 +177,7 @@ int main(int argc, char **argv) {
     hugemenu[i].text = strdup(buf);
   }
   Fl_Window window(WIDTH,400);
+  window.callback(window_cb);
   Fl_Menu_Bar menubar(0,0,WIDTH,30); menubar.menu(menutable);
   menubar.callback(test_cb);
   menus[0] = &menubar;
@@ -201,20 +189,19 @@ int main(int argc, char **argv) {
   menus[2] = &ch;
   Fl_Menu_Button mb(0,0,WIDTH,400,"&popup");
   mb.type(Fl_Menu_Button::POPUP3);
+  mb.box(FL_NO_BOX);
   mb.menu(menutable);
   mb.remove(1); // delete the "File" submenu
   mb.callback(test_cb);
   menus[3] = &mb;
   Fl_Box b(200,200,200,100,"Press right button\nfor a pop-up menu");
-  Fl_Toggle_Button t(250,50,150,25,"MicroSoft Style");
-  t.callback(button_cb);
   window.resizable(&mb);
-  window.size_range(300,20);
+  window.size_range(300,400,0,400);
   window.end();
   window.show(argc, argv);
   return Fl::run();
 }
 
 //
-// End of "$Id: menubar.cxx,v 1.6.2.8 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: menubar.cxx,v 1.6.2.8.2.6 2003/01/30 21:45:53 easysw Exp $".
 //
