@@ -1,9 +1,9 @@
 //
-// "$Id: shiny.cxx,v 1.5.2.2 2000/06/05 21:21:24 mike Exp $"
+// "$Id: shiny.cxx,v 1.5.2.3.2.4 2003/01/30 21:46:02 easysw Exp $"
 //
 // OpenGL "shiny buttons" test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2003 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -24,7 +24,7 @@
 //
 
 #include <config.h>
-#include "shiny_panel.cxx"
+#include "shiny_panel.h"
 #include <FL/fl_message.H>
 #include <stdio.h>
 
@@ -264,8 +264,8 @@ void invisible_box(int x, int y, int w, int h, Fl_Color c) {
   fl_pop_clip();
 }
 
-#define SHINY_BOX (Fl_Boxtype)30
-#define INVISIBLE_BOX (Fl_Boxtype)31
+#define SHINY_BOX (Fl_Boxtype)(FL_FREE_BOXTYPE + 0)
+#define INVISIBLE_BOX (Fl_Boxtype)(FL_FREE_BOXTYPE + 1)
 
 int main(int argc, char **argv) {
   window = make_panels();
@@ -294,14 +294,14 @@ int main(int argc, char **argv) {
   int i = 0;
   if (Fl::args(argc,argv,i) < argc) Fl::fatal(Fl::help);
 #if HAVE_GL
-  if (!Fl::gl_visual(FL_RGB)) Fl::fatal("Display does not do OpenGL");
+  if (!Fl::gl_visual(FL_RGB | FL_DOUBLE)) Fl::fatal("Display does not do OpenGL");
 #else
   fl_message("This demo does not work without OpenGL");
 #endif
-  window->show(argc,argv);
+  window->show();
   return Fl::run();
 }
 
 //
-// End of "$Id: shiny.cxx,v 1.5.2.2 2000/06/05 21:21:24 mike Exp $".
+// End of "$Id: shiny.cxx,v 1.5.2.3.2.4 2003/01/30 21:46:02 easysw Exp $".
 //
